@@ -61,7 +61,7 @@ void RunMod(void) {
                 break;
             else if (state == STATE_READ) {
                 Iso15693InitReader();
-                ReaderIso15693(0, &card);
+                ReaderIso15693(&card);
 
                 if (card.uidlen == 0) {
                     LED_D_OFF();
@@ -75,7 +75,8 @@ void RunMod(void) {
             } else if (state == STATE_EMUL) {
                 Iso15693InitTag();
                 Dbprintf("Starting simulation, press pm3-button to stop and go back to search state.");
-                SimTagIso15693(card.uid);
+                // default block size is 4
+                SimTagIso15693(card.uid, 4);
 
                 state = STATE_READ;
             }
