@@ -486,6 +486,13 @@ void Mifare1ksim(uint16_t flags, uint8_t exitAfterNReads, uint8_t *datain, uint1
     uint8_t *rats = NULL;
     uint8_t rats_len = 0;
 
+
+    // if fct is called with NULL we need to assign some memory since this pointer is passaed around
+    uint8_t datain_tmp[10] = {0};
+    if (datain == NULL) {
+        datain = datain_tmp;
+    }
+
     //Here, we collect UID,sector,keytype,NT,AR,NR,NT2,AR2,NR2
     // This will be used in the reader-only attack.
 
@@ -503,7 +510,7 @@ void Mifare1ksim(uint16_t flags, uint8_t exitAfterNReads, uint8_t *datain, uint1
     uint8_t mM = 0; //moebius_modifier for collection storage
 
     // Authenticate response - nonce
-    uint8_t rAUTH_NT[4];
+    uint8_t rAUTH_NT[4] = {0, 0, 0, 1};
     uint8_t rAUTH_NT_keystream[4];
     uint32_t nonce = 0;
 
@@ -587,8 +594,9 @@ void Mifare1ksim(uint16_t flags, uint8_t exitAfterNReads, uint8_t *datain, uint1
         // WUPA in HALTED state or REQA or WUPA in any other state
         if (receivedCmd_len == 1 && ((receivedCmd[0] == ISO14443A_CMD_REQA && cardSTATE != MFEMUL_HALTED) || receivedCmd[0] == ISO14443A_CMD_WUPA)) {
             selTimer = GetTickCount();
-            if (g_dbglevel >= DBG_EXTENDED)
-                Dbprintf("EmSendPrecompiledCmd(&responses[ATQA]);");
+            if (g_dbglevel >= DBG_EXTENDED) {
+                //Dbprintf("EmSendPrecompiledCmd(&responses[ATQA]);");
+            }
             EmSendPrecompiledCmd(&responses[ATQA]);
 
             FpgaDisableTracing();
@@ -727,7 +735,7 @@ void Mifare1ksim(uint16_t flags, uint8_t exitAfterNReads, uint8_t *datain, uint1
             case MFEMUL_WORK: {
 
                 if (g_dbglevel >= DBG_EXTENDED) {
-                    Dbprintf("[MFEMUL_WORK] Enter in case");
+                    // Dbprintf("[MFEMUL_WORK] Enter in case");
                 }
 
                 if (receivedCmd_len == 0) {
@@ -788,7 +796,15 @@ void Mifare1ksim(uint16_t flags, uint8_t exitAfterNReads, uint8_t *datain, uint1
                         EmSendCmd(rAUTH_NT, sizeof(rAUTH_NT));
                         FpgaDisableTracing();
 
-                        if (g_dbglevel >= DBG_EXTENDED) Dbprintf("[MFEMUL_WORK] Reader authenticating for block %d (0x%02x) with key %c - nonce: %02X - ciud: %02X", receivedCmd_dec[1], receivedCmd_dec[1], (cardAUTHKEY == 0) ? 'A' : 'B', rAUTH_NT, cuid);
+                        if (g_dbglevel >= DBG_EXTENDED) {
+                            Dbprintf("[MFEMUL_WORK] Reader authenticating for block %d (0x%02x) with key %c - nonce: %08X - cuid: %08X",
+                                     receivedCmd_dec[1],
+                                     receivedCmd_dec[1],
+                                     (cardAUTHKEY == 0) ? 'A' : 'B',
+                                     nonce,
+                                     cuid
+                                    );
+                        }
                     } else {
                         // nested authentication
                         /*
@@ -801,7 +817,13 @@ void Mifare1ksim(uint16_t flags, uint8_t exitAfterNReads, uint8_t *datain, uint1
                         EmSendCmdPar(response, 4, response_par);
                         FpgaDisableTracing();
 
-                        if (g_dbglevel >= DBG_EXTENDED) Dbprintf("[MFEMUL_WORK] Reader doing nested authentication for block %d (0x%02x) with key %c", receivedCmd_dec[1], receivedCmd_dec[1], (cardAUTHKEY == 0) ? 'A' : 'B');
+                        if (g_dbglevel >= DBG_EXTENDED) {
+                            Dbprintf("[MFEMUL_WORK] Reader doing nested authentication for block %d (0x%02x) with key %c",
+                                     receivedCmd_dec[1],
+                                     receivedCmd_dec[1],
+                                     (cardAUTHKEY == 0) ? 'A' : 'B'
+                                    );
+                        }
                     }
 
                     cardSTATE = MFEMUL_AUTH1;
@@ -824,7 +846,7 @@ void Mifare1ksim(uint16_t flags, uint8_t exitAfterNReads, uint8_t *datain, uint1
                     break;
                 }
 
-                // case MFEMUL_WORK => if Cmd is Read, Write, Inc, Dec, Restore, Transfert
+                // case MFEMUL_WORK => if Cmd is Read, Write, Inc, Dec, Restore, Transfer
                 if (receivedCmd_len == 4 && (receivedCmd_dec[0] == ISO14443A_CMD_READBLOCK
                                              || receivedCmd_dec[0] == ISO14443A_CMD_WRITEBLOCK
                                              || receivedCmd_dec[0] == MIFARE_CMD_INC
@@ -1050,6 +1072,7 @@ void Mifare1ksim(uint16_t flags, uint8_t exitAfterNReads, uint8_t *datain, uint1
                     } else {
                         EmSend4bit(encrypted_data ? mf_crypto1_encrypt4bit(pcs, CARD_NACK_NA) : CARD_NACK_NA);
                         FpgaDisableTracing();
+                        cardSTATE_TO_IDLE();
                         if (g_dbglevel >= DBG_EXTENDED)
                             Dbprintf("[MFEMUL_WORK] RCV RATS => NACK");
                     }
@@ -1073,7 +1096,7 @@ void Mifare1ksim(uint16_t flags, uint8_t exitAfterNReads, uint8_t *datain, uint1
                     } else {
                         EmSend4bit(encrypted_data ? mf_crypto1_encrypt4bit(pcs, CARD_NACK_NA) : CARD_NACK_NA);
                         FpgaDisableTracing();
-
+                        cardSTATE_TO_IDLE();
                         if (g_dbglevel >= DBG_EXTENDED)
                             Dbprintf("[MFEMUL_WORK] RCV NXP DESELECT => NACK");
                     }
