@@ -1,7 +1,18 @@
 //-----------------------------------------------------------------------------
-// This code is licensed to you under the terms of the GNU GPL, version 2 or,
-// at your option, any later version. See the LICENSE.txt file for the text of
-// the license.
+// Copyright (C) Jonathan Westhues, Mar 2006
+// Copyright (C) Proxmark3 contributors. See AUTHORS.md for details.
+//
+// This program is free software: you can redistribute it and/or modify
+// it under the terms of the GNU General Public License as published by
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
+//
+// This program is distributed in the hope that it will be useful,
+// but WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU General Public License for more details.
+//
+// See LICENSE.txt for the text of the license.
 //-----------------------------------------------------------------------------
 // Main code for the bootloader
 //-----------------------------------------------------------------------------
@@ -221,7 +232,7 @@ static void flash_mode(void) {
             }
         }
 
-        if (g_common_area.flags.button_pressed && !BUTTON_PRESS()) {
+        if (g_common_area.flags.button_pressed && BUTTON_PRESS() == false) {
             g_common_area.flags.button_pressed = 0;
         }
         if (!g_common_area.flags.button_pressed && BUTTON_PRESS()) {
