@@ -591,7 +591,7 @@ static int CmdHFFelicaAuthentication1(const char *Cmd) {
         arg_str0(NULL, "sn",  "<hex>", "number of service, 1 byte"),
         arg_str0(NULL, "scl", "<hex>", "service code list, 2 bytes"),
         arg_str0("k", "key",  "<hex>", "3des key, 16 bytes"),
-        arg_lit0("v", "verbose", "verbose helptext"),
+        arg_lit0("v", "verbose", "verbose output"),
         arg_param_end
     };
     CLIExecWithReturn(ctx, Cmd, argtable, true);
@@ -804,7 +804,7 @@ static int CmdHFFelicaAuthentication2(const char *Cmd) {
         arg_str0("i", NULL, "<hex>", "set custom IDm"),
         arg_str0("c", "cc", "<hex>", "M3c card challenge, 8 bytes"),
         arg_str0("k", "key",  "<hex>", "3des M3c decryption key, 16 bytes"),
-        arg_lit0("v", "verbose", "verbose helptext"),
+        arg_lit0("v", "verbose", "verbose output"),
         arg_param_end
     };
     CLIExecWithReturn(ctx, Cmd, argtable, true);
@@ -970,7 +970,7 @@ static int CmdHFFelicaWritePlain(const char *Cmd) {
         arg_str0(NULL, "scl", "<hex>", "service code list"),
         arg_str0(NULL, "bn",  "<hex>", "number of block"),
         arg_str0(NULL, "ble", "<hex>", "block list element (def 2|3 bytes)"),
-        arg_lit0("v", "verbose", "verbose helptext"),
+        arg_lit0("v", "verbose", "verbose output"),
         arg_param_end
     };
     CLIExecWithReturn(ctx, Cmd, argtable, true);
@@ -1136,7 +1136,7 @@ static int CmdHFFelicaReadPlain(const char *Cmd) {
         arg_str0(NULL, "scl", "<hex>", "service code list"),
         arg_str0(NULL, "bn",  "<hex>", "number of block"),
         arg_str0(NULL, "ble", "<hex>", "block list element (def 2|3 bytes)"),
-        arg_lit0("v", "verbose", "verbose helptext"),
+        arg_lit0("v", "verbose", "verbose output"),
         arg_param_end
     };
     CLIExecWithReturn(ctx, Cmd, argtable, true);
@@ -1364,7 +1364,7 @@ static int CmdHFFelicaRequestSpecificationVersion(const char *Cmd) {
         arg_param_begin,
         arg_str0("i", NULL, "<hex>", "set custom IDm"),
         arg_str0("r", NULL, "<hex>", "set custom reserve"),
-        arg_lit0("v", "verbose", "verbose helptext"),
+        arg_lit0("v", "verbose", "verbose output"),
         arg_param_end
     };
     CLIExecWithReturn(ctx, Cmd, argtable, true);
@@ -1471,7 +1471,7 @@ static int CmdHFFelicaResetMode(const char *Cmd) {
         arg_param_begin,
         arg_str0("i", NULL, "<hex>", "set custom IDm"),
         arg_str0("r", NULL, "<hex>", "set custom reserve"),
-        arg_lit0("v", "verbose", "verbose helptext"),
+        arg_lit0("v", "verbose", "verbose output"),
         arg_param_end
     };
     CLIExecWithReturn(ctx, Cmd, argtable, true);
@@ -1702,14 +1702,11 @@ static int CmdHFFelicaRequestService(const char *Cmd) {
 
     uint8_t datalen = 13; // length (1) + CMD (1) + IDm(8) + Node Number (1) + Node Code List (2)
 
-    uint8_t flags = FELICA_APPEND_CRC;
+    uint8_t flags = (FELICA_APPEND_CRC | FELICA_RAW);
     if (custom_IDm) {
         flags |= FELICA_NO_SELECT;
     }
 
-    if (datalen > 0) {
-        flags |= FELICA_RAW;
-    }
     // Todo activate once datalen isn't hardcoded anymore...
     if (custom_IDm == false && check_last_idm(data, datalen) == false) {
         return PM3_EINVARG;
@@ -1777,13 +1774,11 @@ static int CmdHFFelicaSniff(const char *Cmd) {
     if (payload.samples > 9999) {
         payload.samples = 9999;
         PrintAndLogEx(INFO, "Too large samples to skip value, using max value 9999");
-        return PM3_EINVARG;
     }
 
     if (payload.triggers  > 9999) {
         payload.triggers  = 9999;
         PrintAndLogEx(INFO, "Too large trigger to skip value, using max value 9999");
-        return PM3_EINVARG;
     }
 
 
@@ -2074,7 +2069,7 @@ static int CmdHFFelicaDumpLite(const char *Cmd) {
         return PM3_EOPABORTED;
     }
 
-    uint32_t tracelen = resp.oldarg[1];
+    uint16_t tracelen = resp.oldarg[1];
     if (tracelen == 0) {
         PrintAndLogEx(WARNING, "No trace data! Maybe not a FeliCa Lite card?");
         return PM3_ESOFT;
