@@ -44,6 +44,7 @@
 #include "cmdtrace.h"
 #include "util.h"
 #include "fileutils.h"   // laodFileJSONroot
+#include "protocols.h"   // ISO7816 APDU return codes
 
 #define DEF_FIDO_SIZE        2048
 #define DEF_FIDO_PARAM_FILE  "hf_fido2_defparams.json"
@@ -84,7 +85,7 @@ static int CmdHFFidoInfo(const char *Cmd) {
         return res;
     }
 
-    if (sw != 0x9000) {
+    if (sw != ISO7816_OK) {
         if (sw)
             PrintAndLogEx(INFO, "Not a FIDO card! APDU response: %04x - %s", sw, GetAPDUCodeDescription(sw >> 8, sw & 0xff));
         else
@@ -111,7 +112,7 @@ static int CmdHFFidoInfo(const char *Cmd) {
     if (res) {
         return res;
     }
-    if (sw != 0x9000) {
+    if (sw != ISO7816_OK) {
         PrintAndLogEx(ERR, "FIDO2 version doesn't exist (%04x - %s).", sw, GetAPDUCodeDescription(sw >> 8, sw & 0xff));
         return PM3_SUCCESS;
     }
@@ -137,22 +138,22 @@ static int CmdHFFidoRegister(const char *cmd) {
                   "challenge parameter (32b) and application parameter (32b).\n"
                   "The default config filename is  `fido2_defparams.json`\n"
                   "\n",
-                  "hf fido reg                 -> execute command with 2 parameters, filled 0x00\n"
-                  "hf fido reg --cp s0 --ap s1 -> execute command with plain parameters\n"
+                  "hf fido reg                   -> execute command with 2 parameters, filled 0x00\n"
+                  "hf fido reg --cp s0 --ap s1   -> execute command with plain parameters\n"
                   "hf fido reg --cpx 000102030405060708090a0b0c0d0e0f000102030405060708090a0b0c0d0e0f --apx 000102030405060708090a0b0c0d0e0f000102030405060708090a0b0c0d0e0f\n"
-                  "hf fido reg -f fido2-params -> execute command with custom config file\n"
+                  "hf fido reg -f fido2-params   -> execute command with custom config file\n"
                  );
 
     void *argtable[] = {
         arg_param_begin,
-        arg_lit0("a",  "apdu",  "show APDU requests and responses"),
-        arg_litn("v",  "verbose",  0, 2, "show technical data. vv - show full certificates data"),
-        arg_lit0("t",  "tlv",   "Show DER certificate contents in TLV representation"),
-        arg_str0("f",  "file",  "<fn>", "JSON input file name for parameters"),
-        arg_str0(NULL,  "cp",   "<ascii>", "challenge parameter (1..16 chars)"),
-        arg_str0(NULL,  "ap",   "<ascii>",  "application parameter (1..16 chars)"),
-        arg_str0(NULL,  "cpx",   "<hex>", "challenge parameter (32 bytes hex)"),
-        arg_str0(NULL,  "apx",   "<hex>",  "application parameter (32 bytes hex)"),
+        arg_lit0("a",  "apdu", "Show APDU requests and responses"),
+        arg_litn("v",  "verbose",  0, 2, "Verbose mode. vv - show full certificates data"),
+        arg_lit0("t",  "tlv",  "Show DER certificate contents in TLV representation"),
+        arg_str0("f",  "file", "<fn>",  "JSON input file name for parameters"),
+        arg_str0(NULL, "cp",   "<str>", "Challenge parameter (1..16 chars)"),
+        arg_str0(NULL, "ap",   "<str>", "Application parameter (1..16 chars)"),
+        arg_str0(NULL, "cpx",  "<hex>", "Challenge parameter (32 bytes hex)"),
+        arg_str0(NULL, "apx",  "<hex>", "Application parameter (32 bytes hex)"),
         arg_param_end
     };
     CLIExecWithReturn(ctx, cmd, argtable, true);
@@ -262,7 +263,7 @@ static int CmdHFFidoRegister(const char *cmd) {
         return res;
     }
 
-    if (sw != 0x9000) {
+    if (sw != ISO7816_OK) {
         PrintAndLogEx(ERR, "Can't select FIDO application. APDU response status: %04x - %s", sw, GetAPDUCodeDescription(sw >> 8, sw & 0xff));
         DropField();
         json_decref(root);
@@ -277,7 +278,7 @@ static int CmdHFFidoRegister(const char *cmd) {
         return res;
     }
 
-    if (sw != 0x9000) {
+    if (sw != ISO7816_OK) {
         PrintAndLogEx(ERR, "ERROR execute register command. APDU response status: %04x - %s", sw, GetAPDUCodeDescription(sw >> 8, sw & 0xff));
         return PM3_ESOFT;
     }
@@ -372,11 +373,15 @@ static int CmdHFFidoRegister(const char *cmd) {
     PrintAndLogEx(INFO, "");
     PrintAndLogEx(INFO, "auth command: ");
     char command[500] = {0};
-    sprintf(command, "hf fido auth --kh %s", sprint_hex_inrow(&buf[67], keyHandleLen));
-    if (chlen)
-        sprintf(command + strlen(command), " --%s %s", cpplain ? "cp" : "cpx", cpplain ? (char *)cdata : sprint_hex_inrow(cdata, 32));
-    if (applen)
-        sprintf(command + strlen(command), " --%s %s", applain ? "cp" : "cpx", applain ? (char *)adata : sprint_hex_inrow(adata, 32));
+    snprintf(command, sizeof(command), "hf fido auth --kh %s", sprint_hex_inrow(&buf[67], keyHandleLen));
+    if (chlen) {
+        size_t command_len = strlen(command);
+        snprintf(command + command_len, sizeof(command) - command_len, " --%s %s", cpplain ? "cp" : "cpx", cpplain ? (char *)cdata : sprint_hex_inrow(cdata, 32));
+    }
+    if (applen) {
+        size_t command_len = strlen(command);
+        snprintf(command + command_len, sizeof(command) - command_len, " --%s %s", applain ? "cp" : "cpx", applain ? (char *)adata : sprint_hex_inrow(adata, 32));
+    }
     PrintAndLogEx(INFO, "%s", command);
 
     if (root) {
@@ -409,18 +414,18 @@ static int CmdHFFidoAuthenticate(const char *cmd) {
 
     void *argtable[] = {
         arg_param_begin,
-        arg_lit0("a",  "apdu",      "show APDU reqests and responses"),
-        arg_lit0("v",  "verbose",   "show technical data"),
+        arg_lit0("a",  "apdu",      "Show APDU requests and responses"),
+        arg_lit0("v",  "verbose",   "Verbose mode"),
         arg_rem("default mode:",    "dont-enforce-user-presence-and-sign"),
         arg_lit0("u",  "user",      "mode: enforce-user-presence-and-sign"),
         arg_lit0("c",  "check",     "mode: check-only"),
-        arg_str0("f",  "file",    "<fn>", "JSON input file name for parameters"),
-        arg_str0("k",  "key",    "<hex>", "public key to verify signature"),
-        arg_str0(NULL, "kh",     "<hex>",  "key handle (var 0..255b)"),
-        arg_str0(NULL, "cp",   "<ascii>", "challenge parameter (1..16 chars)"),
-        arg_str0(NULL, "ap",   "<ascii>",  "application parameter (1..16 chars)"),
-        arg_str0(NULL, "cpx",    "<hex>", "challenge parameter (32 bytes hex)"),
-        arg_str0(NULL, "apx",    "<hex>",  "application parameter (32 bytes hex)"),
+        arg_str0("f",  "file",  "<fn>",  "JSON file name for parameters"),
+        arg_str0("k",  "key",   "<hex>", "Public key to verify signature"),
+        arg_str0(NULL, "kh",    "<hex>", "Key handle (var 0..255b)"),
+        arg_str0(NULL, "cp",    "<str>", "Challenge parameter (1..16 chars)"),
+        arg_str0(NULL, "ap",    "<str>", "Application parameter (1..16 chars)"),
+        arg_str0(NULL, "cpx",   "<hex>", "Challenge parameter (32 bytes hex)"),
+        arg_str0(NULL, "apx",   "<hex>", "Application parameter (32 bytes hex)"),
         arg_param_end
     };
     CLIExecWithReturn(ctx, cmd, argtable, true);
@@ -580,7 +585,7 @@ static int CmdHFFidoAuthenticate(const char *cmd) {
         return res;
     }
 
-    if (sw != 0x9000) {
+    if (sw != ISO7816_OK) {
         PrintAndLogEx(ERR, "Can't select FIDO application. APDU response status: %04x - %s", sw, GetAPDUCodeDescription(sw >> 8, sw & 0xff));
         DropField();
         json_decref(root);
@@ -595,7 +600,7 @@ static int CmdHFFidoAuthenticate(const char *cmd) {
         return res;
     }
 
-    if (sw != 0x9000) {
+    if (sw != ISO7816_OK) {
         PrintAndLogEx(ERR, "ERROR execute authentication command. APDU response status: %04x - %s", sw, GetAPDUCodeDescription(sw >> 8, sw & 0xff));
         json_decref(root);
         return PM3_ESOFT;
@@ -635,7 +640,7 @@ static int CmdHFFidoAuthenticate(const char *cmd) {
                     PrintAndLogEx(WARNING, "Other signature check error: %x %s", (res < 0) ? -res : res, ecdsa_get_error(res));
                 }
             } else {
-                PrintAndLogEx(SUCCESS, "Signature is (" _GREEN_("ok") " )");
+                PrintAndLogEx(SUCCESS, "Signature is ( " _GREEN_("ok") " )");
             }
         } else {
             PrintAndLogEx(WARNING, "No public key provided. can't check signature.");
@@ -671,11 +676,11 @@ static int CmdHFFido2MakeCredential(const char *cmd) {
 
     void *argtable[] = {
         arg_param_begin,
-        arg_lit0("a", "apdu", "show APDU reqests and responses"),
-        arg_litn("v", "verbose", 0, 2, "show technical data. vv - show full certificates data"),
-        arg_lit0("t", "tlv", "Show DER certificate contents in TLV representation"),
-        arg_lit0("c", "cbor", "show CBOR decoded data"),
-        arg_str0("f", "file", "<fn>", "parameter JSON file name"),
+        arg_lit0("a", "apdu", "Show APDU requests and responses"),
+        arg_litn("v", "verbose", 0, 2, "Verbose mode. vv - show full certificates data"),
+        arg_lit0("t", "tlv",  "Show DER certificate contents in TLV representation"),
+        arg_lit0("c", "cbor", "Show CBOR decoded data"),
+        arg_str0("f", "file", "<fn>", "Parameter JSON file name"),
         arg_param_end
     };
     CLIExecWithReturn(ctx, cmd, argtable, true);
@@ -720,7 +725,7 @@ static int CmdHFFido2MakeCredential(const char *cmd) {
         return res;
     }
 
-    if (sw != 0x9000) {
+    if (sw != ISO7816_OK) {
         PrintAndLogEx(ERR, "Can't select FIDO application. APDU response status: %04x - %s", sw, GetAPDUCodeDescription(sw >> 8, sw & 0xff));
         DropField();
         json_decref(root);
@@ -748,7 +753,7 @@ static int CmdHFFido2MakeCredential(const char *cmd) {
         return res;
     }
 
-    if (sw != 0x9000) {
+    if (sw != ISO7816_OK) {
         PrintAndLogEx(ERR, "ERROR execute make credential command. APDU response status: %04x - %s", sw, GetAPDUCodeDescription(sw >> 8, sw & 0xff));
         json_decref(root);
         return PM3_EFILE;
@@ -790,11 +795,11 @@ static int CmdHFFido2GetAssertion(const char *cmd) {
 
     void *argtable[] = {
         arg_param_begin,
-        arg_lit0("a", "apdu", "show APDU reqests and responses"),
-        arg_litn("v", "verbose", 0, 2, "show technical data. vv - show full certificates data"),
-        arg_lit0("c", "cbor", "show CBOR decoded data"),
-        arg_lit0("l", "list", "add CredentialId from json to allowList"),
-        arg_str0("f", "file", "<fn>", "parameter JSON file name"),
+        arg_lit0("a", "apdu", "Show APDU requests and responses"),
+        arg_litn("v", "verbose", 0, 2, "Verbose mode. vv - show full certificates data"),
+        arg_lit0("c", "cbor", "Show CBOR decoded data"),
+        arg_lit0("l", "list", "Add CredentialId from json to allowList"),
+        arg_str0("f", "file", "<fn>", "Parameter JSON file name"),
         arg_param_end
     };
     CLIExecWithReturn(ctx, cmd, argtable, true);
@@ -839,7 +844,7 @@ static int CmdHFFido2GetAssertion(const char *cmd) {
         return res;
     }
 
-    if (sw != 0x9000) {
+    if (sw != ISO7816_OK) {
         PrintAndLogEx(ERR, "Can't select FIDO application. APDU response status: %04x - %s", sw, GetAPDUCodeDescription(sw >> 8, sw & 0xff));
         DropField();
         json_decref(root);
@@ -867,7 +872,7 @@ static int CmdHFFido2GetAssertion(const char *cmd) {
         return res;
     }
 
-    if (sw != 0x9000) {
+    if (sw != ISO7816_OK) {
         PrintAndLogEx(ERR, "ERROR execute get assertion command. APDU response status: %04x - %s", sw, GetAPDUCodeDescription(sw >> 8, sw & 0xff));
         json_decref(root);
         return PM3_ESOFT;
