@@ -24,9 +24,9 @@
 
 int CmdHFiClass(const char *Cmd);
 
-int info_iclass(void);
-int read_iclass_csn(bool loop, bool verbose);
-void printIclassDumpContents(uint8_t *iclass_dump, uint8_t startblock, uint8_t endblock, size_t filesize);
+int info_iclass(bool shallow_mod);
+int read_iclass_csn(bool loop, bool verbose, bool shallow_mod);
+void printIclassDumpContents(uint8_t *iclass_dump, uint8_t startblock, uint8_t endblock, size_t filesize, bool dense_output);
 void HFiClassCalcDivKey(uint8_t *CSN, uint8_t *KEY, uint8_t *div_key, bool elite);
 
 void GenerateMacFrom(uint8_t *CSN, uint8_t *CCNR, bool use_raw, bool use_elite, uint8_t *keys, uint32_t keycnt, iclass_premac_t *list);
