@@ -17,13 +17,9 @@
 //-----------------------------------------------------------------------------
 #include "cmdhflegic.h"
 
-#include <stdio.h> // for Mingw readline
 #include <ctype.h> // tolower
 
-#ifdef HAVE_READLINE
-#include <readline/readline.h>
-#endif
-
+#include "pm3line.h"      // pm3line_read, pm3line_free
 #include "cliparser.h"
 #include "cmdparser.h"    // command_t
 #include "comms.h"        // clearCommandBuffer
@@ -34,7 +30,11 @@
 
 static int CmdHelp(const char *Cmd);
 
-#define MAX_LENGTH 1024
+#define LEGIC_PRIME_MIM22   22
+#define LEGIC_PRIME_MIM256  256
+#define LEGIC_PRIME_MIM1024 1024
+#define LEGIC_BLOCK_SIZE    8
+#define LEGIC_PACKET_SIZE   (PM3_CMD_DATA_SIZE - sizeof(legic_packet_t))
 
 static bool legic_xor(uint8_t *data, uint16_t cardsize) {
 
@@ -46,14 +46,14 @@ static bool legic_xor(uint8_t *data, uint16_t cardsize) {
     uint8_t crc = data[4];
     uint32_t calc_crc = CRC8Legic(data, 4);
     if (crc != calc_crc) {
-        PrintAndLogEx(INFO, "Crc mismatch, obsfuscation not possible");
+        PrintAndLogEx(INFO, "CRC mismatch, obsfuscation not possible");
         return false;
     }
 
     for (uint16_t i = 22; i < cardsize; i++) {
         data[i] ^= crc;
     }
-    PrintAndLogEx(SUCCESS, "applying xoring of data done!");
+    PrintAndLogEx(SUCCESS, "Applying xoring of data done!");
     return true;
 }
 
@@ -112,11 +112,11 @@ static int CmdLegicInfo(const char *Cmd) {
 
     PrintAndLogEx(SUCCESS, " " _CYAN_("CDF: System Area"));
     PrintAndLogEx(NORMAL, "------------------------------------------------------");
-    PrintAndLogEx(SUCCESS, "MCD: " _GREEN_("%02X") " MSN: " _GREEN_("%s") " MCC: " _GREEN_("%02X") " (%s)",
+    PrintAndLogEx(SUCCESS, "MCD: " _GREEN_("%02X") " MSN: " _GREEN_("%s") " MCC: " _GREEN_("%02X") " ( %s )",
                   data[0],
                   sprint_hex(data + 1, 3),
                   data[4],
-                  (calc_crc == crc) ? _GREEN_("OK") : _RED_("Fail")
+                  (calc_crc == crc) ? _GREEN_("ok") : _RED_("fail")
                  );
 
     // MCD = Manufacturer ID (should be list meaning something?)
@@ -267,12 +267,12 @@ static int CmdLegicInfo(const char *Cmd) {
                           (segment_flag & 0x4) >> 2,
                           (segment_flag & 0x8) >> 3
                          );
-            PrintAndLogEx(SUCCESS, "            | WRP: %02u, WRC: %02u, RD: %01u, CRC: 0x%02X (%s)",
+            PrintAndLogEx(SUCCESS, "            | WRP: %02u, WRC: %02u, RD: %01u, CRC: 0x%02X ( %s )",
                           wrp,
                           wrc,
                           ((data[i + 3] ^ crc) & 0x80) >> 7,
                           segCRC,
-                          (segCRC == segCalcCRC) ? _GREEN_("OK") : _RED_("Fail")
+                          (segCRC == segCalcCRC) ? _GREEN_("ok") : _RED_("fail")
                          );
 
             i += 5;
@@ -397,33 +397,30 @@ static int CmdLegicRdbl(const char *Cmd) {
     CLIParserContext *ctx;
     CLIParserInit(&ctx, "hf legic rdbl",
                   "Read data from a LEGIC Prime tag",
-                  "hf legic rdbl -o 0 -l 16           -> reads from byte[0] 16 bytes(system header)\n"
-                  "hf legic rdbl -o 0 -l 4 --iv 55    -> reads from byte[0] 4 bytes with IV 0x55\n"
-                  "hf legic rdbl -o 0 -l 256 --iv 55  -> reads from byte[0] 256 bytes with IV 0x55");
+                  "hf legic rdbl -o 0 -l 16           -> read 16 bytes from offset 0 (system header)\n"
+                  "hf legic rdbl -o 0 -l 4 --iv 55    -> read 4 bytes from offset 0\n"
+                  "hf legic rdbl -o 0 -l 256 --iv 55  -> read 256 bytes from offset 0");
 
     void *argtable[] = {
         arg_param_begin,
-        arg_int1("o", "offset", "<dec>", "offset in data array to start download from"),
-        arg_int1("l", "length", "<dec>", "number of bytes to read"),
+        arg_int0("o", "offset", "<dec>", "offset in data array to start download from"),
+        arg_int0("l", "length", "<dec>", "number of bytes to read"),
         arg_str0(NULL, "iv", "<hex>", "Initialization vector to use. Must be odd and 7bits max"),
         arg_param_end
     };
     CLIExecWithReturn(ctx, Cmd, argtable, false);
 
     int offset = arg_get_int_def(ctx, 1, 0);
-
-    int len = arg_get_int_def(ctx, 2, 0);
+    int len = arg_get_int_def(ctx, 2, 16);
 
     int iv_len = 0;
-    uint8_t iv[1] = {0x01};  // formerly uidcrc
-
+    uint8_t iv[1] = {0x01};
     CLIGetHexWithReturn(ctx, 3, iv, &iv_len);
-
     CLIParserFree(ctx);
 
     // sanity checks
-    if (len + offset >= MAX_LENGTH) {
-        PrintAndLogEx(WARNING, "Out-of-bounds, Cardsize = %d, [offset+len = %d ]", MAX_LENGTH, len + offset);
+    if (len + offset >= LEGIC_PRIME_MIM1024) {
+        PrintAndLogEx(WARNING, "Out-of-bounds, Cardsize = %d, [offset+len = %d ]", LEGIC_PRIME_MIM1024, len + offset);
         return PM3_EOUTOFBOUND;
     }
 
@@ -439,9 +436,9 @@ static int CmdLegicRdbl(const char *Cmd) {
     uint16_t datalen = 0;
     int status = legic_read_mem(offset, len, iv[0], data, &datalen);
     if (status == PM3_SUCCESS) {
-        PrintAndLogEx(NORMAL, " ##  |  0  1  2  3  4  5  6  7  8  9  A  B  C  D  E  F 10 11 12 13 14 15 16 17 18 19 1A 1B 1C 1D 1E 1F");
-        PrintAndLogEx(NORMAL, "-----+------------------------------------------------------------------------------------------------");
-        print_hex_break(data, datalen, 32);
+        PrintAndLogEx(INFO, "## |  0  1  2  3  4  5  6  7  8  9  A  B  C  D  E  F | ascii");
+        PrintAndLogEx(INFO, "---+-------------------------------------------------+-----------------");
+        print_hex_break(data, datalen, 16);
     }
     free(data);
     return status;
@@ -450,17 +447,31 @@ static int CmdLegicRdbl(const char *Cmd) {
 static int CmdLegicSim(const char *Cmd) {
     CLIParserContext *ctx;
     CLIParserInit(&ctx, "hf legic sim",
-                  "Simulates a LEGIC Prime tag. MIM22, MIM256, MIM1024 types can be emulated",
-                  "hf legic sim -t 0   -> Simulate Type MIM22\n"
-                  "hf legic sim -t 1   -> Simulate Type MIM256 (default)\n"
-                  "hf legic sim -t 2   -> Simulate Type MIM1024");
+                  "Simulates a LEGIC Prime tag.\n"
+                  "Following types supported (MIM22, MIM256, MIM1024)",
+                  "hf legic sim --22\n"
+                 );
 
     void *argtable[] = {
         arg_param_begin,
-        arg_int0("t", "type", "<dec>", "Tag type to simulate."),
+        arg_lit0(NULL, "22", "LEGIC Prime MIM22"),
+        arg_lit0(NULL, "256", "LEGIC Prime MIM256 (def)"),
+        arg_lit0(NULL, "1024", "LEGIC Prime MIM1024"),
         arg_param_end
     };
     CLIExecWithReturn(ctx, Cmd, argtable, true);
+    bool m1 = arg_get_lit(ctx, 1);
+    bool m2 = arg_get_lit(ctx, 2);
+    bool m3 = arg_get_lit(ctx, 3);
+    CLIParserFree(ctx);
+
+    // validations
+    if (m1 + m2 + m3 > 1) {
+        PrintAndLogEx(WARNING, "Only specify one LEGIC Prime Type");
+        return PM3_EINVARG;
+    } else if (m1 + m2 + m3 == 0) {
+        m2 = true;
+    }
 
     struct {
         uint8_t tagtype;
@@ -468,14 +479,12 @@ static int CmdLegicSim(const char *Cmd) {
     } PACKED payload;
 
     payload.send_reply = true;
-    payload.tagtype = arg_get_int_def(ctx, 1, 1);
-
-    CLIParserFree(ctx);
-
-    if (payload.tagtype > 2) {
-        PrintAndLogEx(ERR, "Invalid tag type selected.");
-        return PM3_EINVARG;
-    }
+    if (m1)
+        payload.tagtype = 0;
+    else if (m2)
+        payload.tagtype = 1;
+    else if (m3)
+        payload.tagtype = 2;
 
     clearCommandBuffer();
     SendCommandNG(CMD_HF_LEGIC_SIMULATE, (uint8_t *)&payload, sizeof(payload));
@@ -517,15 +526,13 @@ static int CmdLegicWrbl(const char *Cmd) {
     int offset = arg_get_int_def(ctx, 1, 0);
 
     int dlen = 0;
-    uint8_t data[MAX_LENGTH] = {0};
+    uint8_t data[LEGIC_PRIME_MIM1024] = {0};
     CLIGetHexWithReturn(ctx, 2, data, &dlen);
 
     bool autoconfirm = arg_get_lit(ctx, 3);
 
     CLIParserFree(ctx);
 
-    uint32_t IV = 0x55;
-
     // OUT-OF-BOUNDS checks
     // UID 4+1 bytes can't be written to.
     if (offset < 5) {
@@ -553,46 +560,43 @@ static int CmdLegicWrbl(const char *Cmd) {
         PrintAndLogEx(INFO, "#####################################");
         const char *confirm = "Do you really want to continue? y(es)/n(o) : ";
         bool overwrite = false;
-#ifdef HAVE_READLINE
-        char *answer = readline(confirm);
+        char *answer = pm3line_read(confirm);
         overwrite = (answer[0] == 'y' || answer[0] == 'Y');
-#else
-        PrintAndLogEx(NORMAL, "%s" NOLF, confirm);
-        char *answer = NULL;
-        size_t anslen = 0;
-        if (getline(&answer, &anslen, stdin) > 0) {
-            overwrite = (answer[0] == 'y' || answer[0] == 'Y');
-        }
-        PrintAndLogEx(NORMAL, "");
-#endif
-        free(answer);
+        pm3line_free(answer);
         if (overwrite == false) {
             PrintAndLogEx(WARNING, "command cancelled");
             return PM3_EOPABORTED;
         }
     }
 
+    uint32_t IV = 0x55;
     legic_chk_iv(&IV);
 
-    PrintAndLogEx(SUCCESS, "Writing to tag");
+    PrintAndLogEx(SUCCESS, "Writing to tag to offset %i", offset);
+
+    legic_packet_t *payload = calloc(1, sizeof(legic_packet_t) + dlen);
+    payload->offset = (offset & 0xFFFF);
+    payload->iv = (IV & 0x7F);
+    payload->len = dlen;
+    memcpy(payload->data, data, dlen);
 
     PacketResponseNG resp;
     clearCommandBuffer();
-    SendCommandOLD(CMD_HF_LEGIC_WRITER, offset, dlen, IV, data, dlen);
+    SendCommandNG(CMD_HF_LEGIC_WRITER, (uint8_t *)payload, sizeof(legic_packet_t) + dlen);
+    free(payload);
 
     uint8_t timeout = 0;
-    while (!WaitForResponseTimeout(CMD_ACK, &resp, 2000)) {
+    while (WaitForResponseTimeout(CMD_HF_LEGIC_WRITER, &resp, 2000) == false) {
         ++timeout;
         PrintAndLogEx(NORMAL, "." NOLF);
-        if (timeout > 7) {
+        if (timeout > 10) {
             PrintAndLogEx(WARNING, "\ncommand execution time out");
             return PM3_ETIMEOUT;
         }
     }
     PrintAndLogEx(NORMAL, "");
 
-    uint8_t isOK = resp.oldarg[0] & 0xFF;
-    if (!isOK) {
+    if (resp.status != PM3_SUCCESS) {
         PrintAndLogEx(WARNING, "Failed writing tag");
         return PM3_ERFTRANS;
     }
@@ -647,12 +651,18 @@ int legic_read_mem(uint32_t offset, uint32_t len, uint32_t iv, uint8_t *out, uin
 
     legic_chk_iv(&iv);
 
+    legic_packet_t *payload = calloc(1, sizeof(legic_packet_t));
+    payload->offset = (offset & 0xFFFF);
+    payload->iv = iv;
+    payload->len = len;
+
     clearCommandBuffer();
-    SendCommandMIX(CMD_HF_LEGIC_READER, offset, len, iv, NULL, 0);
+    SendCommandNG(CMD_HF_LEGIC_READER, (uint8_t *)payload, sizeof(legic_packet_t));
+    free(payload);
     PacketResponseNG resp;
 
     uint8_t timeout = 0;
-    while (!WaitForResponseTimeout(CMD_ACK, &resp, 1000)) {
+    while (WaitForResponseTimeout(CMD_HF_LEGIC_READER, &resp, 1000) == false) {
         ++timeout;
         PrintAndLogEx(NORMAL,  "." NOLF);
         if (timeout > 14) {
@@ -662,9 +672,8 @@ int legic_read_mem(uint32_t offset, uint32_t len, uint32_t iv, uint8_t *out, uin
     }
     PrintAndLogEx(NORMAL, "");
 
-    uint8_t isOK = resp.oldarg[0] & 0xFF;
-    *outlen = resp.oldarg[1];
-    if (!isOK) {
+    *outlen = resp.data.asDwords[0];
+    if (resp.status != PM3_SUCCESS) {
         PrintAndLogEx(WARNING, "Failed reading tag");
         return PM3_ESOFT;
     }
@@ -673,7 +682,7 @@ int legic_read_mem(uint32_t offset, uint32_t len, uint32_t iv, uint8_t *out, uin
         PrintAndLogEx(WARNING, "Fail, only managed to read %u bytes", *outlen);
 
     // copy data from device
-    if (!GetFromDevice(BIG_BUF_EML, out, *outlen, 0, NULL, 0, NULL, 2500, false)) {
+    if (GetFromDevice(BIG_BUF_EML, out, *outlen, 0, NULL, 0, NULL, 2500, false) == false) {
         PrintAndLogEx(WARNING, "Fail, transfer from device time-out");
         return PM3_ETIMEOUT;
     }
@@ -685,11 +694,11 @@ int legic_print_type(uint32_t tagtype, uint8_t spaces) {
     spc[10] = 0x00;
     char *spacer = spc + (10 - spaces);
 
-    if (tagtype == 22)
+    if (tagtype == LEGIC_PRIME_MIM22)
         PrintAndLogEx(SUCCESS, "%sTYPE: " _YELLOW_("MIM%d card (outdated)"), spacer, tagtype);
-    else if (tagtype == 256)
+    else if (tagtype == LEGIC_PRIME_MIM256)
         PrintAndLogEx(SUCCESS, "%sTYPE: " _YELLOW_("MIM%d card (234 bytes)"), spacer, tagtype);
-    else if (tagtype == 1024)
+    else if (tagtype == LEGIC_PRIME_MIM1024)
         PrintAndLogEx(SUCCESS, "%sTYPE: " _YELLOW_("MIM%d card (1002 bytes)"), spacer, tagtype);
     else
         PrintAndLogEx(INFO, "%sTYPE: " _YELLOW_("Unknown %06x"), spacer, tagtype);
@@ -697,21 +706,22 @@ int legic_print_type(uint32_t tagtype, uint8_t spaces) {
 }
 int legic_get_type(legic_card_select_t *card) {
 
-    if (card == NULL) return PM3_EINVARG;
+    if (card == NULL)
+        return PM3_EINVARG;
 
     clearCommandBuffer();
     SendCommandNG(CMD_HF_LEGIC_INFO, NULL, 0);
     PacketResponseNG resp;
-    if (!WaitForResponseTimeout(CMD_ACK, &resp, 1500))
+    if (WaitForResponseTimeout(CMD_HF_LEGIC_INFO, &resp, 1500) == false)
         return PM3_ETIMEOUT;
 
-    uint8_t isOK = resp.oldarg[0] & 0xFF;
-    if (!isOK)
+    if (resp.status != PM3_SUCCESS)
         return PM3_ESOFT;
 
     memcpy(card, resp.data.asBytes, sizeof(legic_card_select_t));
     return PM3_SUCCESS;
 }
+
 void legic_chk_iv(uint32_t *iv) {
     if ((*iv & 0x7F) != *iv) {
         *iv &= 0x7F;
@@ -723,20 +733,30 @@ void legic_chk_iv(uint32_t *iv) {
         PrintAndLogEx(INFO, "LSB of IV must be SET %u", *iv);
     }
 }
+
 void legic_seteml(uint8_t *src, uint32_t offset, uint32_t numofbytes) {
+
     // fast push mode
     g_conn.block_after_ACK = true;
-    for (size_t i = offset; i < numofbytes; i += PM3_CMD_DATA_SIZE) {
+    for (size_t i = offset; i < numofbytes; i += LEGIC_PACKET_SIZE) {
 
-        size_t len = MIN((numofbytes - i), PM3_CMD_DATA_SIZE);
+        size_t len = MIN((numofbytes - i), LEGIC_PACKET_SIZE);
         if (len == numofbytes - i) {
             // Disable fast mode on last packet
             g_conn.block_after_ACK = false;
         }
+
+        legic_packet_t *payload = calloc(1, sizeof(legic_packet_t) + len);
+        payload->offset = i;
+        payload->len = len;
+        memcpy(payload->data, src + i, len);
+
         clearCommandBuffer();
-        SendCommandOLD(CMD_HF_LEGIC_ESET, i, len, 0, src + i, len);
+        SendCommandNG(CMD_HF_LEGIC_ESET, (uint8_t *)payload, sizeof(legic_packet_t) + len);
+        free(payload);
     }
 }
+
 static int CmdLegicReader(const char *Cmd) {
     CLIParserContext *ctx;
     CLIParserInit(&ctx, "hf legic reader",
@@ -762,10 +782,10 @@ static int CmdLegicReader(const char *Cmd) {
 static int CmdLegicDump(const char *Cmd) {
     CLIParserContext *ctx;
     CLIParserInit(&ctx, "hf legic dump",
-                  "Read all memory from LEGIC Prime MIM22, MIM256, MIM1024 and saves bin/eml/json dump file\n"
-                  "It autodetects card type.",
+                  "Read all memory from LEGIC Prime tags and saves to (bin/eml/json) dump file\n"
+                  "It autodetects card type (MIM22, MIM256, MIM1024)",
                   "hf legic dump             --> use UID as filename\n"
-                  "hf legic dump -f myfile   --> use user specified filename\n"
+                  "hf legic dump -f myfile \n"
                   "hf legic dump --de        --> use UID as filename and deobfuscate data");
 
     void *argtable[] = {
@@ -794,28 +814,33 @@ static int CmdLegicDump(const char *Cmd) {
     legic_print_type(dumplen, 0);
     PrintAndLogEx(SUCCESS, "Reading tag memory %d b...", dumplen);
 
+    legic_packet_t *payload = calloc(1, sizeof(legic_packet_t));
+    payload->offset = 0;
+    payload->iv = 0x55;
+    payload->len = dumplen;
+
     clearCommandBuffer();
-    SendCommandMIX(CMD_HF_LEGIC_READER, 0x00, dumplen, 0x55, NULL, 0);
+    SendCommandNG(CMD_HF_LEGIC_READER, (uint8_t *)payload, sizeof(legic_packet_t));
+    free(payload);
     PacketResponseNG resp;
 
     uint8_t timeout = 0;
-    while (!WaitForResponseTimeout(CMD_ACK, &resp, 2000)) {
+    while (WaitForResponseTimeout(CMD_HF_LEGIC_READER, &resp, 2000) == false) {
         ++timeout;
         PrintAndLogEx(NORMAL, "." NOLF);
-        if (timeout > 7) {
+        if (timeout > 10) {
             PrintAndLogEx(WARNING, "\ncommand execution time out");
             return PM3_ETIMEOUT;
         }
     }
     PrintAndLogEx(NORMAL, "");
 
-    uint8_t isOK = resp.oldarg[0] & 0xFF;
-    if (!isOK) {
+    if (resp.status != PM3_SUCCESS) {
         PrintAndLogEx(WARNING, "Failed dumping tag data");
         return PM3_ERFTRANS;
     }
 
-    uint16_t readlen = resp.oldarg[1];
+    uint16_t readlen = resp.data.asDwords[0];
     uint8_t *data = calloc(readlen, sizeof(uint8_t));
     if (!data) {
         PrintAndLogEx(WARNING, "Fail, cannot allocate memory");
@@ -826,7 +851,7 @@ static int CmdLegicDump(const char *Cmd) {
         PrintAndLogEx(WARNING, "Fail, only managed to read 0x%02X bytes of 0x%02X", readlen, dumplen);
 
     // copy data from device
-    if (!GetFromDevice(BIG_BUF_EML, data, readlen, 0, NULL, 0, NULL, 2500, false)) {
+    if (GetFromDevice(BIG_BUF_EML, data, readlen, 0, NULL, 0, NULL, 2500, false) == false) {
         PrintAndLogEx(WARNING, "Fail, transfer from device time-out");
         free(data);
         return PM3_ETIMEOUT;
@@ -850,9 +875,7 @@ static int CmdLegicDump(const char *Cmd) {
         FillFileNameByUID(filename, data, "-dump", 4);
     }
 
-    saveFile(filename, ".bin", data, readlen);
-    saveFileEML(filename, data, readlen, 8);
-    saveFileJSON(filename, jsfLegic, data, readlen, NULL);
+    pm3_save_dump(filename, data, readlen, jsfLegic, LEGIC_BLOCK_SIZE);
     free(data);
     return PM3_SUCCESS;
 }
@@ -860,7 +883,7 @@ static int CmdLegicDump(const char *Cmd) {
 static int CmdLegicRestore(const char *Cmd) {
     CLIParserContext *ctx;
     CLIParserInit(&ctx, "hf legic restore",
-                  "Reads binary file and it autodetects card type and verifies that the file has the same size\n"
+                  "Reads (bin/eml/json) file and it autodetects card type and verifies that the file has the same size\n"
                   "Then write the data back to card. All bytes except the first 7bytes [UID(4) MCC(1) DCF(2)]",
                   "hf legic restore -f myfile        --> use user specified filename\n"
                   "hf legic restore -f myfile --ob   --> use UID as filename and obfuscate data");
@@ -889,31 +912,26 @@ static int CmdLegicRestore(const char *Cmd) {
 
     legic_print_type(card.cardsize, 0);
 
-    // set up buffer
-    uint8_t *data = calloc(card.cardsize, sizeof(uint8_t));
-    if (!data) {
-        PrintAndLogEx(WARNING, "Fail, cannot allocate memory");
-        return PM3_EMALLOC;
+    // read dump file
+    uint8_t *dump = NULL;
+    size_t bytes_read = 0;
+    int res = pm3_load_dump(filename, (void **)&dump, &bytes_read, LEGIC_PRIME_MIM1024);
+    if (res != PM3_SUCCESS) {
+        return res;
     }
 
-    size_t numofbytes;
-    if (loadFile_safe(filename, ".bin", (void **)&data, &numofbytes) != PM3_SUCCESS) {
-        free(data);
-        PrintAndLogEx(WARNING, "Error, reading file");
-        return PM3_EFILE;
-    }
-
-    if (card.cardsize != numofbytes) {
-        PrintAndLogEx(WARNING, "Fail, filesize and cardsize is not equal. [%u != %zu]", card.cardsize, numofbytes);
-        free(data);
+    // validation
+    if (card.cardsize != bytes_read) {
+        PrintAndLogEx(WARNING, "Fail, filesize and cardsize is not equal. [%u != %zu]", card.cardsize, bytes_read);
+        free(dump);
         return PM3_EFILE;
     }
 
     if (shall_obsfuscate) {
-        if (legic_xor(data, card.cardsize) == false) {
+        if (legic_xor(dump, card.cardsize) == false) {
             PrintAndLogEx(FAILED, "Obsfuscate failed, exiting...");
             PrintAndLogEx(HINT, "Try running command without `--ob` parameter");
-            free(data);
+            free(dump);
             return PM3_EFAILED;
         }
     }
@@ -925,38 +943,46 @@ static int CmdLegicRestore(const char *Cmd) {
 
     // transfer to device
     PacketResponseNG resp;
-    for (size_t i = 7; i < numofbytes; i += PM3_CMD_DATA_SIZE) {
+    // 7 = skip UID bytes and MCC
+    for (size_t i = 7; i < bytes_read; i += LEGIC_PACKET_SIZE) {
 
-        size_t len = MIN((numofbytes - i), PM3_CMD_DATA_SIZE);
-        if (len == numofbytes - i) {
+        size_t len = MIN((bytes_read - i), LEGIC_PACKET_SIZE);
+        if (len == bytes_read - i) {
             // Disable fast mode on last packet
             g_conn.block_after_ACK = false;
         }
+
+        legic_packet_t *payload = calloc(1, sizeof(legic_packet_t) + len);
+        payload->offset = i;
+        payload->iv = 0x55;
+        payload->len = len;
+        memcpy(payload->data, dump + i, len);
+
         clearCommandBuffer();
-        SendCommandOLD(CMD_HF_LEGIC_WRITER, i, len, 0x55, data + i, len);
+        SendCommandNG(CMD_HF_LEGIC_WRITER, (uint8_t *)payload, sizeof(legic_packet_t) + len);
+        free(payload);
 
         uint8_t timeout = 0;
-        while (!WaitForResponseTimeout(CMD_ACK, &resp, 2000)) {
+        while (WaitForResponseTimeout(CMD_HF_LEGIC_WRITER, &resp, 2000) == false) {
             ++timeout;
             PrintAndLogEx(NORMAL, "." NOLF);
-            if (timeout > 7) {
+            if (timeout > 10) {
                 PrintAndLogEx(WARNING, "\ncommand execution time out");
-                free(data);
+                free(dump);
                 return PM3_ETIMEOUT;
             }
         }
         PrintAndLogEx(NORMAL, "");
 
-        uint8_t isOK = resp.oldarg[0] & 0xFF;
-        if (!isOK) {
-            PrintAndLogEx(WARNING, "Failed writing tag [msg = %u]", (uint8_t)(resp.oldarg[1] & 0xFF));
-            free(data);
+        if (resp.status != PM3_SUCCESS) {
+            PrintAndLogEx(WARNING, "Failed writing tag");
+            free(dump);
             return PM3_ERFTRANS;
         }
         PrintAndLogEx(SUCCESS, "Wrote chunk [offset %zu | len %zu | total %zu", i, len, i + len);
     }
 
-    free(data);
+    free(dump);
     PrintAndLogEx(SUCCESS, "Done!");
     return PM3_SUCCESS;
 }
@@ -964,15 +990,14 @@ static int CmdLegicRestore(const char *Cmd) {
 static int CmdLegicELoad(const char *Cmd) {
     CLIParserContext *ctx;
     CLIParserInit(&ctx, "hf legic eload",
-                  "Loads a LEGIC binary dump into emulator memory",
-                  "hf legic eload -f myfile -t 0  -> Simulate Type MIM22\n"
-                  "hf legic eload -f myfile -t 1  -> Simulate Type MIM256 (default)\n"
-                  "hf legic eload -f myfile -t 2  -> Simulate Type MIM1024");
+                  "Loads a LEGIC Prime dump file into emulator memory",
+                  "hf legic eload -f myfile\n"
+                  "hf legic eload -f myfile --obfuscate\n"
+                 );
 
     void *argtable[] = {
         arg_param_begin,
-        arg_str1("f", "file", "<fn>", "Filename to restore"),
-        arg_int0("t", "type", "<dec>", "Tag type to simulate."),
+        arg_str1("f", "file", "<fn>", "Filename to load"),
         arg_lit0(NULL, "obfuscate", "Obfuscate dump data (xor with MCC)"),
         arg_param_end
     };
@@ -982,67 +1007,54 @@ static int CmdLegicELoad(const char *Cmd) {
     char filename[FILE_PATH_SIZE] = {0};
     CLIParamStrToBuf(arg_get_str(ctx, 1), (uint8_t *)filename, FILE_PATH_SIZE, &fnlen);
 
-    size_t numofbytes = 0;
-
-    switch (arg_get_int_def(ctx, 2, 1)) {
-        case 0:
-            numofbytes = 22;
-            break;
-        case 1:
-            numofbytes = 256;
-            break;
-        case 2:
-            numofbytes = 1024;
-            break;
-        default:
-            PrintAndLogEx(ERR, "Unknown tag type");
-            CLIParserFree(ctx);
-            return PM3_EINVARG;
-    }
-
-    bool shall_obsfuscate = arg_get_lit(ctx, 3);
-
+    bool shall_obsfuscate = arg_get_lit(ctx, 2);
     CLIParserFree(ctx);
 
-    // set up buffer
-    uint8_t *data = calloc(numofbytes, sizeof(uint8_t));
-    if (!data) {
-        PrintAndLogEx(WARNING, "Fail, cannot allocate memory");
-        return PM3_EMALLOC;
+    // read dump file
+    uint8_t *dump = NULL;
+    size_t bytes_read = 0;
+    int res = pm3_load_dump(filename, (void **)&dump, &bytes_read, LEGIC_PRIME_MIM1024);
+    if (res != PM3_SUCCESS) {
+        return res;
     }
 
-    if (loadFile_safe(filename, ".bin", (void **)&data, &numofbytes) != PM3_SUCCESS) {
-        free(data);
-        PrintAndLogEx(WARNING, "Error, reading file");
+    // validation
+    if (bytes_read != LEGIC_PRIME_MIM22 &&
+            bytes_read != LEGIC_PRIME_MIM256 &&
+            bytes_read != LEGIC_PRIME_MIM1024) {
+        PrintAndLogEx(ERR, "File content error. Read %zu bytes", bytes_read);
+        free(dump);
         return PM3_EFILE;
     }
 
     if (shall_obsfuscate) {
-        legic_xor(data, numofbytes);
+        legic_xor(dump, bytes_read);
     }
 
     PrintAndLogEx(SUCCESS, "Uploading to emulator memory");
-    legic_seteml(data, 0, numofbytes);
+    legic_seteml(dump, 0, bytes_read);
 
-    free(data);
-    PrintAndLogEx(SUCCESS, "Done");
+    free(dump);
+    PrintAndLogEx(SUCCESS, "Done!");
     return PM3_SUCCESS;
 }
 
 static int CmdLegicESave(const char *Cmd) {
     CLIParserContext *ctx;
     CLIParserInit(&ctx, "hf legic esave",
-                  "Saves bin/eml/json dump file of emulator memory",
-                  "hf legic esave                  --> uses UID as filename\n"
-                  "hf legic esave -f myfile -t 0   --> Type MIM22\n"
-                  "hf legic esave -f myfile -t 1   --> Type MIM256 (default)\n"
-                  "hf legic esave -f myfile -t 2   --> Type MIM1024");
+                  "Saves a (bin/eml/json) dump file of emulator memory",
+                  "hf legic esave                    --> uses UID as filename\n"
+                  "hf legic esave -f myfile --22\n"
+                  "hf legic esave -f myfile --22 --de\n"
+                 );
 
     void *argtable[] = {
         arg_param_begin,
         arg_str0("f", "file", "<fn>", "Filename to save"),
-        arg_int0("t", "type", "<dec>", "Tag type"),
-        arg_lit0(NULL, "deobfuscate", "De-obfuscate dump data (xor with MCC)"),
+        arg_lit0(NULL, "22", "LEGIC Prime MIM22"),
+        arg_lit0(NULL, "256", "LEGIC Prime MIM256 (def)"),
+        arg_lit0(NULL, "1024", "LEGIC Prime MIM1024"),
+        arg_lit0(NULL, "de", "De-obfuscate dump data (xor with MCC)"),
         arg_param_end
     };
     CLIExecWithReturn(ctx, Cmd, argtable, true);
@@ -1051,38 +1063,38 @@ static int CmdLegicESave(const char *Cmd) {
     char filename[FILE_PATH_SIZE] = {0};
     CLIParamStrToBuf(arg_get_str(ctx, 1), (uint8_t *)filename, FILE_PATH_SIZE, &fnlen);
 
-    size_t numofbytes = 0;
+    bool m1 = arg_get_lit(ctx, 2);
+    bool m2 = arg_get_lit(ctx, 3);
+    bool m3 = arg_get_lit(ctx, 4);
+    bool shall_deobsfuscate = arg_get_lit(ctx, 5);
+    CLIParserFree(ctx);
 
-    switch (arg_get_int_def(ctx, 2, 1)) {
-        case 0:
-            numofbytes = 22;
-            break;
-        case 1:
-            numofbytes = 256;
-            break;
-        case 2:
-            numofbytes = 1024;
-            break;
-        default:
-            PrintAndLogEx(ERR, "Unknown tag type");
-            CLIParserFree(ctx);
-            return PM3_EINVARG;
+    // validations
+    if (m1 + m2 + m3 > 1) {
+        PrintAndLogEx(WARNING, "Only specify one LEGIC Prime Type");
+        return PM3_EINVARG;
+    } else if (m1 + m2 + m3 == 0) {
+        m2 = true;
     }
 
-    bool shall_deobsfuscate = arg_get_lit(ctx, 3);
-
-    CLIParserFree(ctx);
+    size_t numofbytes = LEGIC_PRIME_MIM256;
+    if (m1)
+        numofbytes = LEGIC_PRIME_MIM22;
+    else if (m2)
+        numofbytes = LEGIC_PRIME_MIM256;
+    else if (m3)
+        numofbytes = LEGIC_PRIME_MIM1024;
 
     // set up buffer
     uint8_t *data = calloc(numofbytes, sizeof(uint8_t));
-    if (!data) {
+    if (data == NULL) {
         PrintAndLogEx(WARNING, "Fail, cannot allocate memory");
         return PM3_EMALLOC;
     }
 
     // download emulator memory
     PrintAndLogEx(SUCCESS, "Reading emulator memory...");
-    if (!GetFromDevice(BIG_BUF_EML, data, numofbytes, 0, NULL, 0, NULL, 2500, false)) {
+    if (GetFromDevice(BIG_BUF_EML, data, numofbytes, 0, NULL, 0, NULL, 2500, false) == false) {
         PrintAndLogEx(WARNING, "Fail, transfer from device time-out");
         free(data);
         return PM3_ETIMEOUT;
@@ -1099,9 +1111,66 @@ static int CmdLegicESave(const char *Cmd) {
         legic_xor(data, numofbytes);
     }
 
-    saveFile(filename, ".bin", data, numofbytes);
-    saveFileEML(filename, data, numofbytes, 8);
-    saveFileJSON(filename, jsfLegic, data, numofbytes, NULL);
+    pm3_save_dump(filename, data, numofbytes, jsfLegic, LEGIC_BLOCK_SIZE);
+    return PM3_SUCCESS;
+}
+
+static int CmdLegicEView(const char *Cmd) {
+
+    CLIParserContext *ctx;
+    CLIParserInit(&ctx, "hf legic eview",
+                  "It displays emulator memory",
+                  "hf legic eview\n"
+                  "hf legic eview --22\n"
+                 );
+    void *argtable[] = {
+        arg_param_begin,
+        arg_lit0(NULL, "22", "LEGIC Prime MIM22"),
+        arg_lit0(NULL, "256", "LEGIC Prime MIM256 (def)"),
+        arg_lit0(NULL, "1024", "LEGIC Prime MIM1024"),
+        arg_param_end
+    };
+    CLIExecWithReturn(ctx, Cmd, argtable, true);
+
+    bool m1 = arg_get_lit(ctx, 1);
+    bool m2 = arg_get_lit(ctx, 2);
+    bool m3 = arg_get_lit(ctx, 3);
+    CLIParserFree(ctx);
+
+    // validations
+    if (m1 + m2 + m3 > 1) {
+        PrintAndLogEx(WARNING, "Only specify one LEGIC Prime Type");
+        return PM3_EINVARG;
+    } else if (m1 + m2 + m3 == 0) {
+        m2 = true;
+    }
+
+    size_t bytes = LEGIC_PRIME_MIM256;
+    if (m1)
+        bytes = LEGIC_PRIME_MIM22;
+    else if (m2)
+        bytes = LEGIC_PRIME_MIM256;
+    else if (m3)
+        bytes = LEGIC_PRIME_MIM1024;
+
+    uint8_t *dump = calloc(bytes, sizeof(uint8_t));
+    if (dump == NULL) {
+        PrintAndLogEx(WARNING, "Fail, cannot allocate memory");
+        return PM3_EMALLOC;
+    }
+
+    PrintAndLogEx(INFO, "downloading emulator memory");
+    if (GetFromDevice(BIG_BUF_EML, dump, bytes, 0, NULL, 0, NULL, 2500, false) == false) {
+        PrintAndLogEx(WARNING, "Fail, transfer from device time-out");
+        free(dump);
+        return PM3_ETIMEOUT;
+    }
+
+    PrintAndLogEx(NORMAL, "");
+    PrintAndLogEx(INFO, "## |  0  1  2  3  4  5  6  7  8  9  A  B  C  D  E  F | ascii");
+    PrintAndLogEx(INFO, "---+-------------------------------------------------+-----------------");
+    print_hex_break(dump, bytes, 16);
+    free(dump);
     return PM3_SUCCESS;
 }
 
@@ -1141,23 +1210,31 @@ static int CmdLegicWipe(const char *Cmd) {
 
     // transfer to device
     PacketResponseNG resp;
-    for (size_t i = 7; i < card.cardsize; i += PM3_CMD_DATA_SIZE) {
+    for (size_t i = 7; i < card.cardsize; i += LEGIC_PACKET_SIZE) {
 
         PrintAndLogEx(NORMAL, "." NOLF);
 
-        size_t len = MIN((card.cardsize - i), PM3_CMD_DATA_SIZE);
+        size_t len = MIN((card.cardsize - i), LEGIC_PACKET_SIZE);
         if (len == card.cardsize - i) {
             // Disable fast mode on last packet
             g_conn.block_after_ACK = false;
         }
+
+        legic_packet_t *payload = calloc(1, sizeof(legic_packet_t) + len);
+        payload->offset = i;
+        payload->iv = 0x55;
+        payload->len = len;
+        memcpy(payload->data, data + i, len);
+
         clearCommandBuffer();
-        SendCommandOLD(CMD_HF_LEGIC_WRITER, i, len, 0x55, data + i, len);
+        SendCommandNG(CMD_HF_LEGIC_WRITER, (uint8_t *)payload, sizeof(legic_packet_t) + len);
+        free(payload);
 
         uint8_t timeout = 0;
-        while (!WaitForResponseTimeout(CMD_ACK, &resp, 2000)) {
+        while (WaitForResponseTimeout(CMD_HF_LEGIC_WRITER, &resp, 2000) == false) {
             ++timeout;
             PrintAndLogEx(NORMAL, "." NOLF);
-            if (timeout > 7) {
+            if (timeout > 10) {
                 PrintAndLogEx(WARNING, "\ncommand execution time out");
                 free(data);
                 return PM3_ETIMEOUT;
@@ -1165,14 +1242,13 @@ static int CmdLegicWipe(const char *Cmd) {
         }
         PrintAndLogEx(NORMAL, "");
 
-        uint8_t isOK = resp.oldarg[0] & 0xFF;
-        if (!isOK) {
-            PrintAndLogEx(WARNING, "Failed writing tag [msg = %u]", (uint8_t)(resp.oldarg[1] & 0xFF));
+        if (resp.status != PM3_SUCCESS) {
+            PrintAndLogEx(WARNING, "failed writing tag");
             free(data);
             return PM3_ERFTRANS;
         }
     }
-    PrintAndLogEx(SUCCESS, "ok\n");
+    PrintAndLogEx(SUCCESS, "Done!\n");
     free(data);
     return PM3_SUCCESS;
 }
@@ -1181,20 +1257,58 @@ static int CmdLegicList(const char *Cmd) {
     return CmdTraceListAlias(Cmd, "hf legic", "legic");
 }
 
+static int CmdLegicView(const char *Cmd) {
+    CLIParserContext *ctx;
+    CLIParserInit(&ctx, "hf legic view",
+                  "Print a LEGIC Prime dump file (bin/eml/json)",
+                  "hf legic view -f hf-legic-01020304-dump.bin"
+                 );
+    void *argtable[] = {
+        arg_param_begin,
+        arg_str1("f", "file", "<fn>", "Filename of dump"),
+        arg_param_end
+    };
+    CLIExecWithReturn(ctx, Cmd, argtable, false);
+    int fnlen = 0;
+    char filename[FILE_PATH_SIZE];
+    CLIParamStrToBuf(arg_get_str(ctx, 1), (uint8_t *)filename, FILE_PATH_SIZE, &fnlen);
+    CLIParserFree(ctx);
+
+    // read dump file
+    uint8_t *dump = NULL;
+    size_t bytes_read = 0;
+    int res = pm3_load_dump(filename, (void **)&dump, &bytes_read, LEGIC_PRIME_MIM1024);
+    if (res != PM3_SUCCESS) {
+        return res;
+    }
+
+    PrintAndLogEx(NORMAL, "");
+    PrintAndLogEx(INFO, "## |  0  1  2  3  4  5  6  7  8  9  A  B  C  D  E  F | ascii");
+    PrintAndLogEx(INFO, "---+-------------------------------------------------+-----------------");
+    print_hex_break(dump, bytes_read, 16);
+    free(dump);
+    return PM3_SUCCESS;
+}
+
 static command_t CommandTable[] =  {
+    {"-----------", CmdHelp,      AlwaysAvailable, "--------------------- " _CYAN_("operations") " ---------------------"},
     {"help",    CmdHelp,          AlwaysAvailable, "This help"},
-    {"list",    CmdLegicList,     AlwaysAvailable,    "List LEGIC history"},
-    {"reader",  CmdLegicReader,   IfPm3Legicrf,    "LEGIC Prime Reader UID and tag info"},
-    {"info",    CmdLegicInfo,     IfPm3Legicrf,    "Display deobfuscated and decoded LEGIC Prime tag data"},
     {"dump",    CmdLegicDump,     IfPm3Legicrf,    "Dump LEGIC Prime tag to binary file"},
-    {"restore", CmdLegicRestore,  IfPm3Legicrf,    "Restore a dump file onto a LEGIC Prime tag"},
+    {"info",    CmdLegicInfo,     IfPm3Legicrf,    "Display deobfuscated and decoded LEGIC Prime tag data"},
+    {"list",    CmdLegicList,     AlwaysAvailable, "List LEGIC history"},
     {"rdbl",    CmdLegicRdbl,     IfPm3Legicrf,    "Read bytes from a LEGIC Prime tag"},
-    {"sim",     CmdLegicSim,      IfPm3Legicrf,    "Start tag simulator"},
-    {"wrbl",    CmdLegicWrbl,     IfPm3Legicrf,    "Write data to a LEGIC Prime tag"},
-    {"crc",     CmdLegicCalcCrc,  AlwaysAvailable, "Calculate Legic CRC over given bytes"},
-    {"eload",   CmdLegicELoad,    AlwaysAvailable,    "Load binary dump to emulator memory"},
-    {"esave",   CmdLegicESave,    AlwaysAvailable,    "Save emulator memory to binary file"},
+    {"reader",  CmdLegicReader,   IfPm3Legicrf,    "LEGIC Prime Reader UID and tag info"},
+    {"restore", CmdLegicRestore,  IfPm3Legicrf,    "Restore a dump file onto a LEGIC Prime tag"},
     {"wipe",    CmdLegicWipe,     IfPm3Legicrf,    "Wipe a LEGIC Prime tag"},
+    {"wrbl",    CmdLegicWrbl,     IfPm3Legicrf,    "Write data to a LEGIC Prime tag"},
+    {"-----------", CmdHelp,      AlwaysAvailable, "--------------------- " _CYAN_("simulation") " ---------------------"},
+    {"sim",     CmdLegicSim,      IfPm3Legicrf,    "Start tag simulator"},
+    {"eload",   CmdLegicELoad,    IfPm3Legicrf,    "Load binary dump to emulator memory"},
+    {"esave",   CmdLegicESave,    IfPm3Legicrf,    "Save emulator memory to binary file"},
+    {"eview",   CmdLegicEView,    IfPm3Legicrf,    "View emulator memory"},
+    {"-----------", CmdHelp,      AlwaysAvailable, "--------------------- " _CYAN_("utils") " ---------------------"},
+    {"crc",     CmdLegicCalcCrc,  AlwaysAvailable, "Calculate Legic CRC over given bytes"},
+    {"view",    CmdLegicView,     AlwaysAvailable, "Display content from tag dump file"},
     {NULL, NULL, NULL, NULL}
 };
 
