@@ -55,7 +55,7 @@ void ClearAuthData(void) {
 
 static int gs_ntag_i2c_state = 0;
 static int gs_mfuc_state = 0;
-static uint8_t gs_mfuc_authdata[3][16] = {0};
+static uint8_t gs_mfuc_authdata[3][16] = {{0}};
 static uint8_t *gs_mfuc_key = NULL;
 
 /**
@@ -174,16 +174,39 @@ uint8_t iclass_CRC_check(bool isResponse, uint8_t *d, uint8_t n) {
 
 int applyIso14443a(char *exp, size_t size, uint8_t *cmd, uint8_t cmdsize, bool is_response) {
 
-    if (! is_response) {
+    if (is_response == false) {
         if ((gs_ntag_i2c_state == 1) && (cmdsize == 6) && (memcmp(cmd + 1, "\x00\x00\x00", 3) == 0)) {
             snprintf(exp, size, "SECTOR(%d)", cmd[0]);
             gs_ntag_i2c_state = 0;
-            return 1;
+            return PM3_SUCCESS;
         }
 
-        if (cmdsize > 10 && (memcmp(cmd, "\x6a\x02\xC8\x01\x00\x03\x00\x02\x79", 9) == 0)) {
-            snprintf(exp, size, "ECP");
-            return 1;
+        if (cmdsize >= 7 && cmd[0] == ECP_HEADER) {
+            // Byte 0 is a header
+            // Byte 1 indicates format version
+            // Version 0x01 format is 7 bytes long (including crc)
+            // Version 0x02 format is at least 7 bytes long (including crc). First 4 bits of byte 2 define extra payload length
+            if (cmd[1] == 0x01 && cmdsize == 7) {
+                snprintf(exp, size, "ECP1");
+                return PM3_SUCCESS;
+            } else if (cmd[1] == 0x02 && cmdsize == (cmd[2] & 0x0f) + 7) {
+                // Byte 3 is the reader type
+                switch (cmd[3]) {
+                    case 0x01:
+                        snprintf(exp, size, "ECP2 (Transit)");
+                        break;
+                    case 0x02:
+                        snprintf(exp, size, "ECP2 (Access)");
+                        break;
+                    case 0x03:
+                        snprintf(exp, size, "ECP2 (Identity)");
+                        break;
+                    default:
+                        snprintf(exp, size, "ECP2");
+                        break;
+                }
+                return PM3_SUCCESS;
+            }
         }
 
         gs_ntag_i2c_state = 0;
@@ -263,7 +286,7 @@ int applyIso14443a(char *exp, size_t size, uint8_t *cmd, uint8_t cmdsize, bool i
                 break;
             case MIFARE_CMD_RESTORE:
 
-                if (cmdsize == 4)
+                if (cmdsize == 4) {
                     // cmd0 == 0xC2 and cmd1 == 0xFF
                     // high probability its SELECT SECTOR COMMAND:
                     if (cmd[1] == 0xFF) {
@@ -271,8 +294,11 @@ int applyIso14443a(char *exp, size_t size, uint8_t *cmd, uint8_t cmdsize, bool i
                         gs_ntag_i2c_state = 1;
                     } else {
                         snprintf(exp, size, "RESTORE(%d)", cmd[1]);
-                    } else
-                    return 0;
+                    }
+                } else {
+                    return PM3_ESOFT;
+                }
+
                 break;
             case MIFARE_CMD_TRANSFER:
                 snprintf(exp, size, "TRANSFER(%d)", cmd[1]);
@@ -292,6 +318,18 @@ int applyIso14443a(char *exp, size_t size, uint8_t *cmd, uint8_t cmdsize, bool i
                 snprintf(exp, size, "AUTH-B(%d)", cmd[1]);
                 break;
             }
+            case MIFARE_MAGIC_GDM_AUTH_KEYA: {
+                if (cmdsize > 3) {
+                    snprintf(exp, size, "MAGIC AUTH-A(%d)", cmd[1]);
+                    MifareAuthState = masNt;
+                }
+                break;
+            }
+            case MIFARE_MAGIC_GDM_AUTH_KEYB: {
+                MifareAuthState = masNt;
+                snprintf(exp, size, "MAGIC AUTH-B(%d)", cmd[1]);
+                break;
+            }
             case MIFARE_MAGICWUPC1:
                 snprintf(exp, size, "MAGIC WUPC1");
                 break;
@@ -312,18 +350,18 @@ int applyIso14443a(char *exp, size_t size, uint8_t *cmd, uint8_t cmdsize, bool i
 // buffer too small to print the full key,
 // so we give just a hint that one of the default keys was used
 //                        snprintf(exp, size, "AUTH-2 KEY: " _YELLOW_("%s"), sprint_hex(gs_mfuc_key, 16));
-                        snprintf(exp, size, "AUTH-2 KEY: " _YELLOW_("%02x%02x%02x%02x..."), gs_mfuc_key[0], gs_mfuc_key[1], gs_mfuc_key[2], gs_mfuc_key[3]);
+                        snprintf(exp, size, "AUTH-2 KEY: " _GREEN_("%02X%02X%02X%02X..."), gs_mfuc_key[0], gs_mfuc_key[1], gs_mfuc_key[2], gs_mfuc_key[3]);
                     } else {
                         snprintf(exp, size, "AUTH-2");
                     }
                     gs_mfuc_state = 3;
                 } else {
-                    return 0;
+                    return PM3_ESOFT;
                 }
                 break;
             case MIFARE_ULEV1_AUTH:
                 if (cmdsize == 7)
-                    snprintf(exp, size, "PWD-AUTH KEY: " _YELLOW_("0x%02x%02x%02x%02x"), cmd[1], cmd[2], cmd[3], cmd[4]);
+                    snprintf(exp, size, "PWD-AUTH KEY: " _GREEN_("0x%02X%02X%02X%02X"), cmd[1], cmd[2], cmd[3], cmd[4]);
                 else
                     snprintf(exp, size, "PWD-AUTH");
                 break;
@@ -386,7 +424,7 @@ int applyIso14443a(char *exp, size_t size, uint8_t *cmd, uint8_t cmdsize, bool i
 
                 break;
             default:
-                return 0;
+                return PM3_ESOFT;
         }
     } else {
         if (gs_mfuc_state == 1) {
@@ -405,14 +443,14 @@ int applyIso14443a(char *exp, size_t size, uint8_t *cmd, uint8_t cmdsize, bool i
                 if (trace_mfuc_try_default_3des_keys(&gs_mfuc_key, gs_mfuc_state, gs_mfuc_authdata) == PM3_SUCCESS) {
                     snprintf(exp, size, "AUTH-2 ANSW OK");
                     gs_mfuc_state = 0;
-                    return 1;
+                    return PM3_SUCCESS;
                 }
             }
             gs_mfuc_state = 0;
         }
-        return 0;
+        return PM3_ESOFT;
     }
-    return 1;
+    return PM3_SUCCESS;
 }
 
 void annotateIso14443a(char *exp, size_t size, uint8_t *cmd, uint8_t cmdsize, bool is_response) {
@@ -453,10 +491,17 @@ void annotateIclass(char *exp, size_t size, uint8_t *cmd, uint8_t cmdsize, bool
                 curr_state = PICO_NONE;
                 break;
             case ICLASS_CMD_CHECK:
-                snprintf(exp, size, "CHECK");
                 curr_state = PICO_AUTH_MACS;
                 memcpy(rmac, cmd + 1, 4);
                 memcpy(tmac, cmd + 5, 4);
+
+                uint8_t key[8];
+                if (check_known_default(csn, epurse, rmac, tmac, key)) {
+                    snprintf(exp, size, "CHECK ( %s )", sprint_hex_inrow(key, 8));
+                } else {
+                    snprintf(exp, size, "CHECK");
+                }
+
                 break;
             case ICLASS_CMD_READ4:
                 snprintf(exp, size, "READ4(%d)", cmd[1]);
@@ -505,11 +550,7 @@ void annotateIclass(char *exp, size_t size, uint8_t *cmd, uint8_t cmdsize, bool
         } else if (curr_state == PICO_AUTH_EPURSE) {
             memcpy(epurse, cmd, 8);
         } else if (curr_state == PICO_AUTH_MACS) {
-
-            uint8_t key[8];
-            if (check_known_default(csn, epurse, rmac, tmac, key)) {
-                snprintf(exp, size, "( " _GREEN_("%s") " )", sprint_hex_inrow(key, 8));
-            }
+            snprintf(exp, size, _GREEN_("CHECK SUCCESS"));
             curr_state = PICO_NONE;
         }
     }
@@ -550,7 +591,11 @@ void annotateIso15693(char *exp, size_t size, uint8_t *cmd, uint8_t cmdsize) {
                 snprintf(exp, size, "LOCKBLOCK");
                 return;
             case ISO15693_READ_MULTI_BLOCK:
-                snprintf(exp, size, "READ_MULTI_BLOCK");
+                if (cmdsize == 6) {
+                    snprintf(exp, size, "READ_MULTI_BLOCK(%d-%d)", cmd[2], (cmd[2] + cmd[3]));
+                } else {
+                    snprintf(exp, size, "READ_MULTI_BLOCK(%d-%d)", cmd[10], (cmd[10] + cmd[11]));
+                }
                 return;
             case ISO15693_WRITE_MULTI_BLOCK:
                 snprintf(exp, size, "WRITE_MULTI_BLOCK");
@@ -829,7 +874,7 @@ void annotateIso7816(char *exp, size_t size, uint8_t *cmd, uint8_t cmdsize) {
 void annotateMfDesfire(char *exp, size_t size, uint8_t *cmd, uint8_t cmdsize) {
 
     // it's basically a ISO14443a tag, so try annotation from there
-    if (applyIso14443a(exp, size, cmd, cmdsize, false) == 0) {
+    if (applyIso14443a(exp, size, cmd, cmdsize, false) != PM3_SUCCESS) {
 
         // S-block 11xxx010
         if ((cmd[0] & 0xC0) && (cmdsize == 3)) {
@@ -866,18 +911,40 @@ void annotateMfDesfire(char *exp, size_t size, uint8_t *cmd, uint8_t cmdsize) {
             for (uint8_t i = 0; i < 2; i++, pos++) {
                 bool found_annotation = true;
 
+                uint8_t *data = cmd + pos + 1;
+                // if the byte prior to the command is 90 the command is wrapped, so data starts 3 bytes later
+                if (i > 0 && cmd[pos - 1] == 0x90) {
+                    data += 3;
+                }
+                uint8_t data_size = 0;
+                if (cmdsize > (data - cmd)) {
+                    data_size = cmdsize - (data - cmd);
+                }
+
                 switch (cmd[pos]) {
                     case MFDES_CREATE_APPLICATION:
-                        snprintf(exp, size, "CREATE APPLICATION");
+                        if (data_size >= 3) {
+                            snprintf(exp, size, "CREATE APPLICATION (appId %06x)", MemLeToUint3byte(data));
+                        } else {
+                            snprintf(exp, size, "CREATE APPLICATION");
+                        }
                         break;
                     case MFDES_DELETE_APPLICATION:
-                        snprintf(exp, size, "DELETE APPLICATION");
+                        if (data_size >= 3) {
+                            snprintf(exp, size, "DELETE APPLICATION (appId %06x)", MemLeToUint3byte(data));
+                        } else {
+                            snprintf(exp, size, "DELETE APPLICATION");
+                        }
                         break;
                     case MFDES_GET_APPLICATION_IDS:
                         snprintf(exp, size, "GET APPLICATION IDS");
                         break;
                     case MFDES_SELECT_APPLICATION:
-                        snprintf(exp, size, "SELECT APPLICATION");
+                        if (data_size >= 3) {
+                            snprintf(exp, size, "SELECT APPLICATION (appId %06x)", MemLeToUint3byte(data));
+                        } else {
+                            snprintf(exp, size, "SELECT APPLICATION");
+                        }
                         break;
                     case MFDES_FORMAT_PICC:
                         snprintf(exp, size, "FORMAT PICC");
@@ -886,31 +953,67 @@ void annotateMfDesfire(char *exp, size_t size, uint8_t *cmd, uint8_t cmdsize) {
                         snprintf(exp, size, "GET VERSION");
                         break;
                     case MFDES_READ_DATA:
-                        snprintf(exp, size, "READ DATA");
+                        if (data_size >= 7) {
+                            snprintf(exp, size, "READ DATA (fileId %02x, offset %u, len %u)", data[0], MemLeToUint3byte(data + 1), MemLeToUint3byte(data + 4));
+                        } else {
+                            snprintf(exp, size, "READ DATA");
+                        }
                         break;
                     case MFDES_WRITE_DATA:
-                        snprintf(exp, size, "WRITE DATA");
+                        if (data_size >= 7) {
+                            snprintf(exp, size, "WRITE DATA (fileId %02x, offset %u, len %u)", data[0], MemLeToUint3byte(data + 1), MemLeToUint3byte(data + 4));
+                        } else {
+                            snprintf(exp, size, "WRITE DATA");
+                        }
                         break;
                     case MFDES_GET_VALUE:
-                        snprintf(exp, size, "GET VALUE");
+                        if (data_size >= 1) {
+                            snprintf(exp, size, "GET VALUE (fileId %02x)", data[0]);
+                        } else {
+                            snprintf(exp, size, "GET VALUE");
+                        }
                         break;
                     case MFDES_CREDIT:
-                        snprintf(exp, size, "CREDIT");
+                        if (data_size >= 1) {
+                            snprintf(exp, size, "CREDIT (fileId %02x)", data[0]);
+                        } else {
+                            snprintf(exp, size, "CREDIT");
+                        }
                         break;
                     case MFDES_DEBIT:
-                        snprintf(exp, size, "DEBIT");
+                        if (data_size >= 1) {
+                            snprintf(exp, size, "DEBIT (fileId %02x)", data[0]);
+                        } else {
+                            snprintf(exp, size, "DEBIT");
+                        }
                         break;
                     case MFDES_LIMITED_CREDIT:
-                        snprintf(exp, size, "LIMITED CREDIT");
+                        if (data_size >= 1) {
+                            snprintf(exp, size, "LIMITED CREDIT (fileId %02x)", data[0]);
+                        } else {
+                            snprintf(exp, size, "LIMITED CREDIT");
+                        }
                         break;
                     case MFDES_WRITE_RECORD:
-                        snprintf(exp, size, "WRITE RECORD");
+                        if (data_size >= 7) {
+                            snprintf(exp, size, "WRITE RECORD (fileId %02x, offset %u, len %u)", data[0], MemLeToUint3byte(data + 1), MemLeToUint3byte(data + 4));
+                        } else {
+                            snprintf(exp, size, "WRITE RECORD");
+                        }
                         break;
                     case MFDES_READ_RECORDS:
-                        snprintf(exp, size, "READ RECORDS");
+                        if (data_size >= 7) {
+                            snprintf(exp, size, "READ RECORDS (fileId %02x, offset %u, len %u)", data[0], MemLeToUint3byte(data + 1), MemLeToUint3byte(data + 4));
+                        } else {
+                            snprintf(exp, size, "READ RECORDS");
+                        }
                         break;
                     case MFDES_CLEAR_RECORD_FILE:
-                        snprintf(exp, size, "CLEAR RECORD FILE");
+                        if (data_size >= 1) {
+                            snprintf(exp, size, "CLEAR RECORD FILE (fileId %02x)", data[0]);
+                        } else {
+                            snprintf(exp, size, "CLEAR RECORD FILE");
+                        }
                         break;
                     case MFDES_COMMIT_TRANSACTION:
                         snprintf(exp, size, "COMMIT TRANSACTION");
@@ -931,57 +1034,87 @@ void annotateMfDesfire(char *exp, size_t size, uint8_t *cmd, uint8_t cmdsize) {
                         snprintf(exp, size, "GET ISOFILE IDS");
                         break;
                     case MFDES_GET_FILE_SETTINGS:
-                        snprintf(exp, size, "GET FILE SETTINGS");
+                        if (data_size >= 1) {
+                            snprintf(exp, size, "GET FILE SETTINGS (fileId %02x)", data[0]);
+                        } else {
+                            snprintf(exp, size, "GET FILE SETTINGS");
+                        }
                         break;
                     case MFDES_CHANGE_FILE_SETTINGS:
-                        snprintf(exp, size, "CHANGE FILE SETTINGS");
+                        if (data_size >= 1) {
+                            snprintf(exp, size, "CHANGE FILE SETTINGS (fileId %02x)", data[0]);
+                        } else {
+                            snprintf(exp, size, "CHANGE FILE SETTINGS");
+                        }
                         break;
                     case MFDES_CREATE_STD_DATA_FILE:
-                        snprintf(exp, size, "CREATE STD DATA FILE");
+                        if (data_size >= 1) {
+                            snprintf(exp, size, "CREATE STD DATA FILE (fileId %02x)", data[0]);
+                        } else {
+                            snprintf(exp, size, "CREATE STD DATA FILE");
+                        }
                         break;
                     case MFDES_CREATE_BACKUP_DATA_FILE:
-                        snprintf(exp, size, "CREATE BACKUP DATA FILE");
+                        if (data_size >= 1) {
+                            snprintf(exp, size, "CREATE BACKUP DATA FILE (fileId %02x)", data[0]);
+                        } else {
+                            snprintf(exp, size, "CREATE BACKUP DATA FILE");
+                        }
                         break;
                     case MFDES_CREATE_VALUE_FILE:
-                        snprintf(exp, size, "CREATE VALUE FILE");
+                        if (data_size >= 1) {
+                            snprintf(exp, size, "CREATE VALUE FILE (fileId %02x)", data[0]);
+                        } else {
+                            snprintf(exp, size, "CREATE VALUE FILE");
+                        }
                         break;
                     case MFDES_CREATE_LINEAR_RECORD_FILE:
-                        snprintf(exp, size, "CREATE LINEAR RECORD FILE");
+                        if (data_size >= 1) {
+                            snprintf(exp, size, "CREATE LINEAR RECORD FILE (fileId %02x)", data[0]);
+                        } else {
+                            snprintf(exp, size, "CREATE LINEAR RECORD FILE");
+                        }
                         break;
                     case MFDES_CREATE_CYCLIC_RECORD_FILE:
-                        snprintf(exp, size, "CREATE CYCLIC RECORD FILE");
+                        if (data_size >= 1) {
+                            snprintf(exp, size, "CREATE CYCLIC RECORD FILE (fileId %02x)", data[0]);
+                        } else {
+                            snprintf(exp, size, "CREATE CYCLIC RECORD FILE");
+                        }
                         break;
                     case MFDES_CREATE_TRANS_MAC_FILE:
-                        snprintf(exp, size, "CREATE TRANSACTION MAC FILE");
+                        if (data_size >= 1) {
+                            snprintf(exp, size, "CREATE TRANSACTION MAC FILE (fileId %02x)", data[0]);
+                        } else {
+                            snprintf(exp, size, "CREATE TRANSACTION MAC FILE");
+                        }
                         break;
                     case MFDES_DELETE_FILE:
-                        snprintf(exp, size, "DELETE FILE");
+                        if (data_size >= 1) {
+                            snprintf(exp, size, "DELETE FILE (fileId %02x)", data[0]);
+                        } else {
+                            snprintf(exp, size, "DELETE FILE");
+                        }
                         break;
                     case MFDES_AUTHENTICATE:
-                        if (cmdsize > 6) {
-                            //Assume wrapped
-                            snprintf(exp, size, "AUTH NATIVE (keyNo %d)", cmd[pos + 4]);
+                        if (data_size >= 1) {
+                            snprintf(exp, size, "AUTH NATIVE (keyNo %u)", data[0]);
                         } else {
-                            //Assume unwrapped
-                            snprintf(exp, size, "AUTH NATIVE (keyNo %d)", cmd[pos + 1]);
+                            snprintf(exp, size, "AUTH NATIVE");
                         }
                         break;  // AUTHENTICATE_NATIVE
                     case MFDES_AUTHENTICATE_ISO:
-                        if (cmdsize > 6) {
-                            //Assume wrapped
-                            snprintf(exp, size, "AUTH ISO (keyNo %d)", cmd[pos + 4]);
+                        if (data_size >= 1) {
+                            snprintf(exp, size, "AUTH ISO (keyNo %u)", data[0]);
                         } else {
-                            //Assume unwrapped
-                            snprintf(exp, size, "AUTH ISO (keyNo %d)", cmd[pos + 1]);
+                            snprintf(exp, size, "AUTH ISO");
                         }
                         break;  // AUTHENTICATE_STANDARD
                     case MFDES_AUTHENTICATE_AES:
-                        if (cmdsize > 6) {
-                            //Assume wrapped
-                            snprintf(exp, size, "AUTH AES (keyNo %d)", cmd[pos + 4]);
+                        if (data_size >= 1) {
+                            snprintf(exp, size, "AUTH AES (keyNo %u)", data[0]);
                         } else {
-                            //Assume unwrapped
-                            snprintf(exp, size, "AUTH AES (keyNo %d)", cmd[pos + 1]);
+                            snprintf(exp, size, "AUTH AES");
                         }
                         break;
                     case MFDES_AUTHENTICATE_EV2F:
@@ -997,10 +1130,18 @@ void annotateMfDesfire(char *exp, size_t size, uint8_t *cmd, uint8_t cmdsize) {
                         snprintf(exp, size, "GET KEY SETTINGS");
                         break;
                     case MFDES_CHANGE_KEY:
-                        snprintf(exp, size, "CHANGE KEY");
+                        if (data_size >= 1) {
+                            snprintf(exp, size, "CHANGE KEY (keyNo %u)", data[0]);
+                        } else {
+                            snprintf(exp, size, "CHANGE KEY");
+                        }
                         break;
                     case MFDES_GET_KEY_VERSION:
-                        snprintf(exp, size, "GET KEY VERSION");
+                        if (data_size >= 1) {
+                            snprintf(exp, size, "GET KEY VERSION (keyNo %u)", data[0]);
+                        } else {
+                            snprintf(exp, size, "GET KEY VERSION");
+                        }
                         break;
                     case MFDES_ADDITIONAL_FRAME:
                         snprintf(exp, size, "AUTH FRAME / NEXT FRAME");
@@ -1173,6 +1314,22 @@ void annotateCryptoRF(char *exp, size_t size, uint8_t *cmd, uint8_t cmdsize) {
     }
 }
 
+void annotateSeos(char *exp, size_t size, uint8_t *cmd, uint8_t cmdsize) {
+
+    // it's basically a ISO14443a tag, so try annotation from there
+    if (applyIso14443a(exp, size, cmd, cmdsize, false) != PM3_SUCCESS) {
+
+//        switch (cmd[0]) {
+//            default:
+//                break;
+//        };
+
+        // apply ISO7816 annotations?
+//        if (annotateIso7816(exp, size, cmd, cmdsize) == 0) {
+//        }
+        // apply SEOS annotations?
+    }
+}
 
 // LEGIC
 // 1 = read
@@ -1425,6 +1582,10 @@ void annotateMifare(char *exp, size_t size, uint8_t *cmd, uint8_t cmdsize,
 
     // get UID
     if (MifareAuthState == masNone) {
+        if (isResponse && cmdsize == 5) {
+            ClearAuthData();
+            AuthData.uid = bytes_to_num(&cmd[0], 4);
+        }
         if (cmdsize == 9 && cmd[0] == ISO14443A_CMD_ANTICOLL_OR_SELECT && cmd[1] == 0x70) {
             ClearAuthData();
             AuthData.uid = bytes_to_num(&cmd[2], 4);
@@ -1498,10 +1659,11 @@ static void mf_get_paritybinstr(char *s, uint32_t val, uint8_t par) {
     num_to_bytes(val, sizeof(uint32_t), foo);
     for (uint8_t i = 0; i < 4; i++) {
         if (oddparity8(foo[i]) != ((par >> (7 - (i & 0x0007))) & 0x01))
-            sprintf(s++, "1");
+            *(s++) = '1';
         else
-            sprintf(s++, "0");
+            *(s++) = '0';
     }
+    s[0] = '\0';
 }
 
 bool DecodeMifareData(uint8_t *cmd, uint8_t cmdsize, uint8_t *parity, bool isResponse, uint8_t *mfData, size_t *mfDataLen, const uint64_t *dicKeys, uint32_t dicKeysCount) {
