@@ -333,10 +333,10 @@ static int CmdAWIDClone(const char *Cmd) {
     CLIParserContext *ctx;
     CLIParserInit(&ctx, "lf awid clone",
                   "clone a AWID Prox tag to a T55x7, Q5/T5555 or EM4305/4469 tag",
-                  "lf awid clone --fmt 26 --fc 123 --cn 1337\n"
-                  "lf awid clone --fmt 50 --fc 2001 --cn 13371337\n"
-                  "lf awid clone --q5 --fmt 26 --fc 123 --cn 1337   -> encode for Q5/T5555 tag\n"
-                  "lf awid clone --em --fmt 26 --fc 123 --cn 1337   -> encode for EM4305/4469"
+                  "lf awid clone --fmt 26 --fc 123 --cn 1337       -> encode for T55x7 tag\n"
+                  "lf awid clone --fmt 50 --fc 2001 --cn 13371337  -> encode long fmt for T55x7 tag\n"
+                  "lf awid clone --fmt 26 --fc 123 --cn 1337 --q5  -> encode for Q5/T5555 tag\n"
+                  "lf awid clone --fmt 26 --fc 123 --cn 1337 --em  -> encode for EM4305/4469"
                  );
 
     void *argtable[] = {
@@ -373,6 +373,7 @@ static int CmdAWIDClone(const char *Cmd) {
 
     // EM4305
     if (em) {
+        PrintAndLogEx(WARNING, "Beware some EM4305 tags don't support FSK and datarate = RF/50, check your tag copy!");
         blocks[0] = EM4305_AWID_CONFIG_BLOCK;
         snprintf(cardtype, sizeof(cardtype), "EM4305/4469");
     }
@@ -391,6 +392,14 @@ static int CmdAWIDClone(const char *Cmd) {
     blocks[2] = bytebits_to_byte(bits + 32, 32);
     blocks[3] = bytebits_to_byte(bits + 64, 32);
 
+    // EM4305
+    if (em) {
+        // invert FSK data
+        for (uint8_t i = 1; i < ARRAYLEN(blocks) ; i++) {
+            blocks[i] = blocks[i] ^ 0xFFFFFFFF;
+        }
+    }
+
     free(bits);
 
     PrintAndLogEx(INFO, "Preparing to clone AWID %u to " _YELLOW_("%s") " with FC: " _GREEN_("%u") " CN: " _GREEN_("%u"), fmtlen, cardtype, fc, cn);
@@ -478,7 +487,7 @@ static int CmdAWIDBrute(const char *Cmd) {
         arg_u64_1(NULL, "fc", "<dec>", "8|16bit value facility code"),
         arg_u64_0(NULL, "cn", "<dec>", "optional -  card number to start with, max 65535"),
         arg_u64_0(NULL, "delay", "<dec>", "optional - delay betweens attempts in ms. Default 1000ms"),
-        arg_lit0("v", "verbose", "verbose logging, show all tries"),
+        arg_lit0("v", "verbose", "verbose output"),
         arg_param_end
     };
     CLIExecWithReturn(ctx, Cmd, argtable, false);
