@@ -41,6 +41,7 @@
 #include "generator.h"
 #include "cliparser.h"
 #include "cmdhw.h"
+#include "util.h"
 
 //////////////// 4205 / 4305 commands
 
@@ -692,24 +693,25 @@ int CmdEM4x05Dump(const char *Cmd) {
     } else {
     }
 
-    if (success == PM3_SUCCESS) { // all ok save dump to file
-        // saveFileEML will add .eml extension to filename
-        // saveFile (binary) passes in the .bin extension.
+    // all ok save dump to file
+    if (success == PM3_SUCCESS) {
+
         if (strcmp(filename, "") == 0) {
 
             if (card_type == EM_4369) {
-                sprintf(filename, "lf-4369-%08X-dump", BSWAP_32(data[1]));
+                snprintf(filename, sizeof(filename), "lf-4369-%08X-dump", BSWAP_32(data[1]));
             } else if (card_type == EM_4469) {
-                sprintf(filename, "lf-4469-%08X-dump", BSWAP_32(data[1]));
+                snprintf(filename, sizeof(filename), "lf-4469-%08X-dump", BSWAP_32(data[1]));
             } else {
-                sprintf(filename, "lf-4x05-%08X-dump", BSWAP_32(data[1]));
+                snprintf(filename, sizeof(filename), "lf-4x05-%08X-dump", BSWAP_32(data[1]));
             }
 
         }
         PrintAndLogEx(NORMAL, "");
-        saveFileJSON(filename, (card_type == EM_4369 || card_type == EM_4469) ? jsfEM4x69 : jsfEM4x05, (uint8_t *)data, 16 * sizeof(uint32_t), NULL);
-        saveFileEML(filename, (uint8_t *)data, 16 * sizeof(uint32_t), sizeof(uint32_t));
-        saveFile(filename, ".bin", data, sizeof(data));
+        if (card_type == EM_4369 || card_type == EM_4469)
+            pm3_save_dump(filename, (uint8_t *)data, sizeof(data), jsfEM4x69, 4);
+        else
+            pm3_save_dump(filename, (uint8_t *)data, sizeof(data), jsfEM4x05, 4);
     }
     PrintAndLogEx(NORMAL, "");
     return success;
@@ -871,7 +873,7 @@ int CmdEM4x05Wipe(const char *Cmd) {
         arg_lit0(NULL, "4205", "target chip type EM 4205"),
         arg_lit0(NULL, "4305", "target chip type EM 4305 (default)"),
         arg_lit0(NULL, "4369", "target chip type EM 4369"),
-        arg_lit0(NULL, "4369", "target chip type EM 4469"),
+        arg_lit0(NULL, "4469", "target chip type EM 4469"),
         arg_str0("p", "pwd", "<hex>", "optional - password, 4 bytes hex"),
         arg_param_end
     };
@@ -1681,13 +1683,13 @@ int CmdEM4x05Unlock(const char *Cmd) {
         if (tries >= 5 && n == 0 && soon != late) {
 
             if (soon > late) {
-                PrintAndLogEx(INFO, "Tried %d times, soon:%i late:%i        => " _CYAN_("adjust +1 us >> %.0lf us"), tries, soon, late, start);
                 start++;
                 end++;
+                PrintAndLogEx(INFO, "Tried %d times, soon:%i late:%i        => " _CYAN_("adjust +1 us >> %.0lf us"), tries, soon, late, start);
             } else {
-                PrintAndLogEx(INFO, "Tried %d times, soon:%i late:%i        => " _CYAN_("adjust -1 us >> %.0lf us"), tries, soon, late, start);
                 start--;
                 end--;
+                PrintAndLogEx(INFO, "Tried %d times, soon:%i late:%i        => " _CYAN_("adjust -1 us >> %.0lf us"), tries, soon, late, start);
             }
             tries = 0;
             soon = 0;
@@ -1739,7 +1741,7 @@ int CmdEM4x05Unlock(const char *Cmd) {
             if (my_auto) {
                 start += n;
                 PrintAndLogEx(INFO, "                                    => " _CYAN_("adjust +%.0lf us >> %.0lf us"), n, start);
-                n /= 2;
+                n = (int)(n / 2);
             } else {
                 soon++;
             }
@@ -1793,7 +1795,7 @@ int CmdEM4x05Unlock(const char *Cmd) {
                     end = start;
                     start -= n;
                     PrintAndLogEx(INFO, "                                    => " _CYAN_("adjust -%.0lf us >> %.0lf us"), n, start);
-                    n /= 2;
+                    n = (int)(n / 2);
                 } else {
                     late++;
                 }
@@ -1992,11 +1994,10 @@ int CmdEM4x05Sniff(const char *Cmd) {
     bool fwd = arg_get_lit(ctx, 2);
     CLIParserFree(ctx);
 
-    char cmdText[100];
+    const char *cmdText;
     char dataText[100];
     char blkAddr[4];
-    char bits[80];
-    int i, bitidx;
+    int i;
     int ZeroWidth;    // 32-42 "1" is 32
     int CycleWidth;
     size_t pulseSamples;
@@ -2017,13 +2018,15 @@ int CmdEM4x05Sniff(const char *Cmd) {
     PrintAndLogEx(SUCCESS, "offset | Command     |   Data   | blk | raw");
     PrintAndLogEx(SUCCESS, "-------+-------------+----------+-----+------------------------------------------------------------");
 
+    smartbuf bits = { 0 };
+    bits.ptr = malloc(EM4X05_BITS_BUFSIZE);
+    bits.size = EM4X05_BITS_BUFSIZE;
+    bits.idx = 0;
     size_t idx = 0;
     // loop though sample buffer
     while (idx < g_GraphTraceLen) {
-        bool eop = false;
         bool haveData = false;
         bool pwd = false;
-        uint32_t tmpValue;
 
         idx = em4x05_Sniff_GetNextBitStart(idx, g_GraphTraceLen, g_GraphBuffer, &pulseSamples);
         size_t pktOffset = idx;
@@ -2036,9 +2039,10 @@ int CmdEM4x05Sniff(const char *Cmd) {
 
             if (ZeroWidth <= 50) {
                 pktOffset -= ZeroWidth;
-                memset(bits, 0x00, sizeof(bits));
-                bitidx = 0;
+                memset(bits.ptr, 0, bits.size);
+                bits.idx = 0;
 
+                bool eop = false;
                 while ((idx < g_GraphTraceLen) && !eop) {
                     CycleWidth = idx;
                     idx = em4x05_Sniff_GetNextBitStart(idx, g_GraphTraceLen, g_GraphBuffer, &pulseSamples);
@@ -2046,8 +2050,8 @@ int CmdEM4x05Sniff(const char *Cmd) {
                     CycleWidth = idx - CycleWidth;
                     if ((CycleWidth > 300) || (CycleWidth < (ZeroWidth - 5))) { // to long or too short
                         eop = true;
-                        bits[bitidx++] = '0';   // Append last zero from the last bit find
-                        cmdText[0] = 0;
+                        sb_append_char(&bits, '0');   // Append last zero from the last bit find
+                        cmdText = "";
 
                         // EM4305 command lengths
                         // Login        0011 <pwd>          => 4 +     45 => 49
@@ -2058,76 +2062,77 @@ int CmdEM4x05Sniff(const char *Cmd) {
                         // -> disable 1010 11111111 0 11111111 0 11111111 0 11111111 0 00000000 0
 
                         // Check to see if we got the leading 0
-                        if (((strncmp(bits, "00011", 5) == 0) && (bitidx == 50)) ||
-                                ((strncmp(bits, "00101", 5) == 0) && (bitidx == 57)) ||
-                                ((strncmp(bits, "01001", 5) == 0) && (bitidx == 12)) ||
-                                ((strncmp(bits, "01100", 5) == 0) && (bitidx == 50)) ||
-                                ((strncmp(bits, "01010", 5) == 0) && (bitidx == 50))) {
-                            memmove(bits, &bits[1], bitidx - 1);
-                            bitidx--;
+                        if (((strncmp(bits.ptr, "00011", 5) == 0) && (bits.idx == 50)) ||
+                                ((strncmp(bits.ptr, "00101", 5) == 0) && (bits.idx == 57)) ||
+                                ((strncmp(bits.ptr, "01001", 5) == 0) && (bits.idx == 12)) ||
+                                ((strncmp(bits.ptr, "01100", 5) == 0) && (bits.idx == 50)) ||
+                                ((strncmp(bits.ptr, "01010", 5) == 0) && (bits.idx == 50))) {
+                            memmove(bits.ptr, &bits.ptr[1], bits.idx - 1);
+                            bits.idx--;
                             PrintAndLogEx(INFO, "Trim leading 0");
                         }
-                        bits[bitidx] = 0;
+                        sb_append_char(&bits, 0);
+                        bits.idx--;
 
                         // logon
-                        if ((strncmp(bits, "0011", 4) == 0) && (bitidx == 49)) {
+                        if ((strncmp(bits.ptr, "0011", 4) == 0) && (bits.idx == 49)) {
                             haveData = true;
                             pwd = true;
-                            sprintf(cmdText, "Logon");
-                            sprintf(blkAddr, "   ");
-                            tmpValue = em4x05_Sniff_GetBlock(&bits[4], fwd);
-                            sprintf(dataText, "%08X", tmpValue);
+                            cmdText = "Logon";
+                            strncpy(blkAddr, "   ",  sizeof(blkAddr));
+                            uint32_t tmpValue = em4x05_Sniff_GetBlock(&bits.ptr[4], fwd);
+                            snprintf(dataText, sizeof(dataText), "%08X", tmpValue);
                         }
 
                         // write
-                        if ((strncmp(bits, "0101", 4) == 0) && (bitidx == 56)) {
+                        if ((strncmp(bits.ptr, "0101", 4) == 0) && (bits.idx == 56)) {
                             haveData = true;
-                            sprintf(cmdText, "Write");
-                            tmpValue = (bits[4] - '0') + ((bits[5] - '0') << 1) + ((bits[6] - '0') << 2)  + ((bits[7] - '0') << 3);
-                            sprintf(blkAddr, "%u", tmpValue);
+                            cmdText = "Write";
+                            uint32_t tmpValue = (bits.ptr[4] - '0') + ((bits.ptr[5] - '0') << 1) + ((bits.ptr[6] - '0') << 2)  + ((bits.ptr[7] - '0') << 3);
+                            snprintf(blkAddr, sizeof(blkAddr), "%u", tmpValue);
                             if (tmpValue == 2) {
                                 pwd = true;
                             }
-                            tmpValue = em4x05_Sniff_GetBlock(&bits[11], fwd);
-                            sprintf(dataText, "%08X", tmpValue);
+                            tmpValue = em4x05_Sniff_GetBlock(&bits.ptr[11], fwd);
+                            snprintf(dataText, sizeof(dataText), "%08X", tmpValue);
                         }
 
                         // read
-                        if ((strncmp(bits, "1001", 4) == 0) && (bitidx == 11)) {
+                        if ((strncmp(bits.ptr, "1001", 4) == 0) && (bits.idx == 11)) {
                             haveData = true;
                             pwd = false;
-                            sprintf(cmdText, "Read");
-                            tmpValue = (bits[4] - '0') + ((bits[5] - '0') << 1) + ((bits[6] - '0') << 2)  + ((bits[7] - '0') << 3);
-                            sprintf(blkAddr, "%u", tmpValue);
-                            sprintf(dataText, " ");
+                            cmdText = "Read";
+                            uint32_t tmpValue = (bits.ptr[4] - '0') + ((bits.ptr[5] - '0') << 1) + ((bits.ptr[6] - '0') << 2)  + ((bits.ptr[7] - '0') << 3);
+                            snprintf(blkAddr, sizeof(blkAddr), "%u", tmpValue);
+                            strncpy(dataText, " ", sizeof(dataText));
                         }
 
                         // protect
-                        if ((strncmp(bits, "1100", 4) == 0) && (bitidx == 49)) {
+                        if ((strncmp(bits.ptr, "1100", 4) == 0) && (bits.idx == 49)) {
                             haveData = true;
                             pwd = false;
-                            sprintf(cmdText, "Protect");
-                            sprintf(blkAddr, " ");
-                            tmpValue = em4x05_Sniff_GetBlock(&bits[11], fwd);
-                            sprintf(dataText, "%08X", tmpValue);
+                            cmdText = "Protect";
+                            strncpy(blkAddr, " ",  sizeof(blkAddr));
+                            uint32_t tmpValue = em4x05_Sniff_GetBlock(&bits.ptr[11], fwd);
+                            snprintf(dataText, sizeof(dataText), "%08X", tmpValue);
                         }
 
                         // disable
-                        if ((strncmp(bits, "1010", 4) == 0) && (bitidx == 49)) {
+                        if ((strncmp(bits.ptr, "1010", 4) == 0) && (bits.idx == 49)) {
                             haveData = true;
                             pwd = false;
-                            sprintf(cmdText, "Disable");
-                            sprintf(blkAddr, " ");
-                            tmpValue = em4x05_Sniff_GetBlock(&bits[11], fwd);
-                            sprintf(dataText, "%08X", tmpValue);
+                            cmdText = "Disable";
+                            strncpy(blkAddr, " ",  sizeof(blkAddr));
+                            uint32_t tmpValue = em4x05_Sniff_GetBlock(&bits.ptr[11], fwd);
+                            snprintf(dataText, sizeof(dataText), "%08X", tmpValue);
                         }
 
                         //  bits[bitidx] = 0;
                     } else {
                         i = (CycleWidth - ZeroWidth) / 28;
-                        bits[bitidx++] = '0';
+                        sb_append_char(&bits, '0');
                         for (int ii = 0; ii < i; ii++) {
-                            bits[bitidx++] = '1';
+                            sb_append_char(&bits, '1');
                         }
                     }
                 }
@@ -2138,11 +2143,13 @@ int CmdEM4x05Sniff(const char *Cmd) {
         // Print results
         if (haveData) { //&& (minWidth > 1) && (maxWidth > minWidth)){
             if (pwd)
-                PrintAndLogEx(SUCCESS, "%6zu | %-10s  | " _YELLOW_("%8s")" | " _YELLOW_("%3s")" | %s", pktOffset, cmdText, dataText, blkAddr, bits);
+                PrintAndLogEx(SUCCESS, "%6zu | %-10s  | " _YELLOW_("%8s")" | " _YELLOW_("%3s")" | %s", pktOffset, cmdText, dataText, blkAddr, bits.ptr);
             else
-                PrintAndLogEx(SUCCESS, "%6zu | %-10s  | " _GREEN_("%8s")" | " _GREEN_("%3s")" | %s", pktOffset, cmdText, dataText, blkAddr, bits);
+                PrintAndLogEx(SUCCESS, "%6zu | %-10s  | " _GREEN_("%8s")" | " _GREEN_("%3s")" | %s", pktOffset, cmdText, dataText, blkAddr, bits.ptr);
         }
     }
+    free(bits.ptr);
+    bits.ptr = NULL;
 
     // footer
     PrintAndLogEx(SUCCESS, "---------------------------------------------------------------------------------------------------");
