@@ -81,6 +81,7 @@ static int sendTry(uint8_t format_idx, wiegand_card_t *card, uint32_t delay, boo
     }
 
     lf_hidsim_t payload;
+    payload.Q5 = false;
     payload.hi2 = packed.Top;
     payload.hi = packed.Mid;
     payload.lo = packed.Bot;
@@ -322,11 +323,11 @@ static int CmdHIDClone(const char *Cmd) {
     CLIParserInit(&ctx, "lf hid clone",
                   "clone a HID Prox tag to a T55x7, Q5/T5555 or EM4305/4469 tag.\n"
                   "Tag must be on the antenna when issuing this command.",
-                  "lf hid clone -r 2006ec0c86                      -> write raw value (HID 10301 26 bit)\n"
-                  "lf hid clone -r 2e0ec00c87                      -> write raw value (HID Corporate 35 bit)\n"
-                  "lf hid clone -r 01f0760643c3                    -> write raw value (HID P10001 40 bit)\n"
-                  "lf hid clone -r 01400076000c86                  -> write raw value (HID Corporate 48 bit)\n"
-                  "lf hid clone -w H10301 --fc 118 --cn 1603       -> write raw value (HID 10301 26 bit)\n"
+                  "lf hid clone -r 2006ec0c86                      -> write raw value for T55x7 tag (HID 10301 26 bit)\n"
+                  "lf hid clone -r 2e0ec00c87                      -> write raw value for T55x7 tag (HID Corporate 35 bit)\n"
+                  "lf hid clone -r 01f0760643c3                    -> write raw value for T55x7 tag (HID P10001 40 bit)\n"
+                  "lf hid clone -r 01400076000c86                  -> write raw value for T55x7 tag (HID Corporate 48 bit)\n"
+                  "lf hid clone -w H10301 --fc 118 --cn 1603       -> HID 10301 26 bit, encode for T55x7 tag\n"
                   "lf hid clone -w H10301 --fc 118 --cn 1603 --q5  -> HID 10301 26 bit, encode for Q5/T5555 tag\n"
                   "lf hid clone -w H10301 --fc 118 --cn 1603 --em  -> HID 10301 26 bit, encode for EM4305/4469"
                  );
@@ -419,6 +420,7 @@ static int CmdHIDClone(const char *Cmd) {
 
     // EM4305
     if (em) {
+        PrintAndLogEx(WARNING, "Beware some EM4305 tags don't support FSK and datarate = RF/50, check your tag copy!");
         snprintf(cardtype, sizeof(cardtype), "EM4305/4469");
     }
 
@@ -482,7 +484,7 @@ static int CmdHIDBrute(const char *Cmd) {
 
     void *argtable[] = {
         arg_param_begin,
-        arg_lit0("v", "verbose",             "verbose logging, show all tries"),
+        arg_lit0("v", "verbose",             "verbose output"),
         arg_str1("w", "wiegand", "<format>", "see " _YELLOW_("`wiegand list`") " for available formats"),
         arg_u64_0(NULL, "fc",     "<dec>",    "facility code"),
         arg_u64_0(NULL, "cn",     "<dec>",    "card number to start with"),
@@ -530,27 +532,30 @@ static int CmdHIDBrute(const char *Cmd) {
     CLIParserFree(ctx);
 
     if (verbose) {
-        PrintAndLogEx(INFO, "Wiegand format#.. %i", format_idx);
-        PrintAndLogEx(INFO, "OEM#............. %u", cn_hi.OEM);
-        PrintAndLogEx(INFO, "ISSUE#........... %u", cn_hi.IssueLevel);
-        PrintAndLogEx(INFO, "Facility#........ %u", cn_hi.FacilityCode);
-        PrintAndLogEx(INFO, "Card#............ %" PRIu64, cn_hi.CardNumber);
+        PrintAndLogEx(INFO, "Wiegand format... %i", format_idx);
+        PrintAndLogEx(INFO, "OEM.............. %u", cn_hi.OEM);
+        PrintAndLogEx(INFO, "ISSUE............ %u", cn_hi.IssueLevel);
+        PrintAndLogEx(INFO, "Facility code.... %u", cn_hi.FacilityCode);
+        PrintAndLogEx(INFO, "Card number...... %" PRIu64, cn_hi.CardNumber);
+        PrintAndLogEx(INFO, "Delay............ " _YELLOW_("%d"), delay);
         switch (direction) {
             case 0:
-                PrintAndLogEx(INFO, "Brute-forcing direction: " _YELLOW_("BOTH") " delay " _YELLOW_("%d"), delay);
+                PrintAndLogEx(INFO, "Direction........ " _YELLOW_("BOTH"));
                 break;
             case 1:
-                PrintAndLogEx(INFO, "Brute-forcing direction: " _YELLOW_("UP") " delay " _YELLOW_("%d"), delay);
+                PrintAndLogEx(INFO, "Direction........ " _YELLOW_("UP"));
                 break;
             case 2:
-                PrintAndLogEx(INFO, "Brute-forcing direction: " _YELLOW_("DOWN") " delay " _YELLOW_("%d"), delay);
+                PrintAndLogEx(INFO, "Direction........ " _YELLOW_("DOWN"));
                 break;
             default:
                 break;
         }
     }
+    PrintAndLogEx(NORMAL, "");
     PrintAndLogEx(INFO, "Started brute-forcing HID Prox reader");
     PrintAndLogEx(INFO, "Press " _GREEN_("<Enter>") " or pm3-button to abort simulation");
+    PrintAndLogEx(NORMAL, "");
     // copy values to low.
     cn_low = cn_hi;
 
@@ -561,7 +566,7 @@ static int CmdHIDBrute(const char *Cmd) {
     fin_hi = fin_low = false;
     do {
 
-        if (!g_session.pm3_present) {
+        if (g_session.pm3_present == false) {
             PrintAndLogEx(WARNING, "Device offline\n");
             return PM3_ENODATA;
         }
@@ -574,10 +579,10 @@ static int CmdHIDBrute(const char *Cmd) {
         // do one up
         if (direction != 2) {
             if (cn_hi.CardNumber < 0xFFFF) {
-                cn_hi.CardNumber++;
                 if (sendTry(format_idx, &cn_hi, delay, verbose) != PM3_SUCCESS) {
                     return PM3_ESOFT;
                 }
+                cn_hi.CardNumber++;
             } else {
                 fin_hi = true;
             }
