@@ -156,11 +156,12 @@ static int CmdLFHitagList(const char *Cmd) {
         int j;
         for (j = 0; j < len; j++) {
 
+            int offset = j * 4;
             //if((parityBits >> (len - j - 1)) & 0x01) {
             if (isResponse && (oddparity8(frame[j]) != ((parityBits >> (len - j - 1)) & 0x01))) {
-                sprintf(line + (j * 4), "%02x!  ", frame[j]);
+                snprintf(line + offset, sizeof(line) - offset, "%02x!  ", frame[j]);
             } else {
-                sprintf(line + (j * 4), "%02x   ", frame[j]);
+                snprintf(line + offset, sizeof(line) - offset, "%02x   ", frame[j]);
             }
         }
 
@@ -248,45 +249,18 @@ static int CmdLFHitagEload(const char *Cmd) {
         return PM3_EINVARG;
     }
 
-    DumpFileType_t dftype = getfiletype(filename);
-    size_t dumplen = 0;
+    // read dump file
     uint8_t *dump = NULL;
-    int res = 0;
-    switch (dftype) {
-        case BIN: {
-            res = loadFile_safe(filename, ".bin", (void **)&dump, &dumplen);
-            break;
-        }
-        case EML: {
-            res = loadFileEML_safe(filename, (void **)&dump, &dumplen);
-            break;
-        }
-        case JSON: {
-            dumplen = 4 * 64;
-            dump = calloc(dumplen, sizeof(uint8_t));
-            if (dump == NULL) {
-                PrintAndLogEx(ERR, "error, cannot allocate memory");
-                return PM3_EMALLOC;
-            }
-            res = loadFileJSON(filename, (void *)dump, dumplen, &dumplen, NULL);
-            break;
-        }
-        case DICTIONARY: {
-            PrintAndLogEx(ERR, "error, only BIN/JSON/EML formats allowed");
-            return PM3_EINVARG;
-        }
-    }
-
+    size_t bytes_read = (4 * 64);
+    int res = pm3_load_dump(filename, (void **)&dump, &bytes_read, (4 * 64));
     if (res != PM3_SUCCESS) {
-        PrintAndLogEx(ERR, "error, something went wrong when loading file");
-        free(dump);
-        return PM3_EFILE;
+        return res;
     }
 
     // check dump len..
-    if (dumplen == 48 ||  dumplen == 4 * 64) {
+    if (bytes_read == 48 ||  bytes_read == 4 * 64) {
 
-        lf_hitag_t *payload =  calloc(1, sizeof(lf_hitag_t) + dumplen);
+        lf_hitag_t *payload =  calloc(1, sizeof(lf_hitag_t) + bytes_read);
 
         if (use_ht1)
             payload->type = 1;
@@ -297,14 +271,14 @@ static int CmdLFHitagEload(const char *Cmd) {
         if (use_htm)
             payload->type = 4;
 
-        payload->len = dumplen;
-        memcpy(payload->data, dump, dumplen);
+        payload->len = bytes_read;
+        memcpy(payload->data, dump, bytes_read);
 
         clearCommandBuffer();
-        SendCommandNG(CMD_LF_HITAG_ELOAD, (uint8_t *)payload, 3 + dumplen);
+        SendCommandNG(CMD_LF_HITAG_ELOAD, (uint8_t *)payload, 3 + bytes_read);
         free(payload);
     } else {
-        PrintAndLogEx(ERR, "error, wrong dump file size. got %zu", dumplen);
+        PrintAndLogEx(ERR, "error, wrong dump file size. got %zu", bytes_read);
     }
 
     free(dump);
@@ -607,24 +581,24 @@ static int CmdLFHitagReader(const char *Cmd) {
     if (s01) {
         cmd = CMD_LF_HITAGS_READ;
         htf = RHTSF_CHALLENGE;
-        memcpy(htd.auth.NrAr, nrar, sizeof(nrar));
+        memcpy(htd.auth.NrAr, nrar, sizeof(htd.auth.NrAr));
     }
     if (s02) {
         cmd = CMD_LF_HITAGS_READ;
         htf = RHTSF_KEY;
-        memcpy(htd.crypto.key, key, sizeof(key));
+        memcpy(htd.crypto.key, key, sizeof(htd.crypto.key));
     }
     if (h21) {
         htf = RHT2F_PASSWORD;
-        memcpy(htd.pwd.password, key, 4);
+        memcpy(htd.pwd.password, key, sizeof(htd.pwd.password));
     }
     if (h22) {
         htf = RHT2F_AUTHENTICATE;
-        memcpy(htd.auth.NrAr, nrar, sizeof(nrar));
+        memcpy(htd.auth.NrAr, nrar, sizeof(htd.auth.NrAr));
     }
     if (h23) {
         htf = RHT2F_CRYPTO;
-        memcpy(htd.crypto.key, key, sizeof(key));
+        memcpy(htd.crypto.key, key, sizeof(htd.crypto.key));
     }
     if (h25) {
         htf = RHT2F_TEST_AUTH_ATTEMPTS;
@@ -653,6 +627,9 @@ static int CmdLFHitagReader(const char *Cmd) {
 
         // block3, 1 byte
         printHitag2Configuration(data[4 * 3]);
+
+        // print data
+        print_hex_break(data, 48, 4);
     }
     return PM3_SUCCESS;
 }
@@ -793,22 +770,22 @@ static int CmdLFHitagWriter(const char *Cmd) {
 
     if (s03) {
         htf = WHTSF_CHALLENGE;
-        memcpy(htd.auth.NrAr, nrar, sizeof(nrar));
+        memcpy(htd.auth.NrAr, nrar, sizeof(htd.auth.NrAr));
         memcpy(htd.auth.data, data, sizeof(data));
     }
     if (s04) {
         htf = WHTSF_KEY;
-        memcpy(htd.crypto.key, key, sizeof(key));
+        memcpy(htd.crypto.key, key, sizeof(htd.crypto.key));
         memcpy(htd.crypto.data, data, sizeof(data));
     }
     if (h24) {
         htf = WHT2F_CRYPTO;
-        memcpy(htd.pwd.password, key, 4);
+        memcpy(htd.crypto.key, key, sizeof(htd.crypto.key));
         memcpy(htd.crypto.data, data, sizeof(data));
     }
     if (h27) {
         htf = WHT2F_PASSWORD;
-        memcpy(htd.pwd.password, key, 4);
+        memcpy(htd.pwd.password, key, sizeof(htd.pwd.password));
         memcpy(htd.crypto.data, data, sizeof(data));
     }
 
@@ -846,17 +823,14 @@ static int CmdLFHitag2Dump(const char *Cmd) {
         arg_param_end
     };
     CLIExecWithReturn(ctx, Cmd, argtable, false);
-    uint8_t filename[FILE_PATH_SIZE] = {0};
+
     int fnlen = 0;
-    int res = CLIParamHexToBuf(arg_get_str(ctx, 1), filename, sizeof(filename), &fnlen);
-    if (res != 0) {
-        CLIParserFree(ctx);
-        return PM3_EINVARG;
-    }
+    char filename[FILE_PATH_SIZE] = {0};
+    CLIParamStrToBuf(arg_get_str(ctx, 1), (uint8_t *)filename, FILE_PATH_SIZE, &fnlen);
 
     uint8_t key[6];
     int keylen = 0;
-    res = CLIParamHexToBuf(arg_get_str(ctx, 2), key, sizeof(key), &keylen);
+    int res = CLIParamHexToBuf(arg_get_str(ctx, 2), key, sizeof(key), &keylen);
     if (res != 0) {
         CLIParserFree(ctx);
         return PM3_EINVARG;
@@ -870,25 +844,49 @@ static int CmdLFHitag2Dump(const char *Cmd) {
         return PM3_EINVARG;
     }
 
-    PrintAndLogEx(WARNING, "to be implemented...");
+    hitag_function htf;
+    hitag_data htd;
+    memset(&htd, 0, sizeof(htd));
 
-    /*
-        PrintAndLogEx(SUCCESS, "Dumping tag memory...");
+    if (keylen == 6) {
+        htf = RHT2F_CRYPTO;
+        memcpy(htd.crypto.key, key, sizeof(htd.crypto.key));
+        PrintAndLogEx(INFO, "Authenticating in crypto mode");
+    } else {
+        htf = RHT2F_PASSWORD;
+        memcpy(htd.pwd.password, key, sizeof(htd.pwd.password));
+        PrintAndLogEx(INFO, "Authenticating in password mode");
+    }
 
-        clearCommandBuffer();
-        //SendCommandNG(CMD_LF_HITAG_DUMP, &htd, sizeof(htd));
-        PacketResponseNG resp;
-        uint8_t *data = resp.data.asBytes;
-        if (fnlen < 1) {
-            char *fptr = filename;
-            fptr += sprintf(fptr, "lf-hitag-");
-            FillFileNameByUID(fptr, data, "-dump", 4);
-        }
+    uint16_t cmd = CMD_LF_HITAG_READER;
+    clearCommandBuffer();
+    SendCommandMIX(cmd, htf, 0, 0, &htd, sizeof(htd));
+    PacketResponseNG resp;
 
-        saveFile(filename, ".bin", data, 48);
-        saveFileEML(filename, data, 48, 4);
-        saveFileJSON(filename, jsfHitag, data, 48, NULL);
-    */
+    if (WaitForResponseTimeout(CMD_ACK, &resp, 2000) == false) {
+        PrintAndLogEx(WARNING, "timeout while waiting for reply.");
+        return PM3_ETIMEOUT;
+    }
+    if (resp.oldarg[0] == false) {
+        PrintAndLogEx(DEBUG, "DEBUG: Error - hitag failed");
+        return PM3_ESOFT;
+    }
+    uint8_t *data = resp.data.asBytes;
+
+    if (data == NULL)
+        return PM3_ESOFT;
+
+    PrintAndLogEx(SUCCESS, "Dumping tag memory...");
+
+    if (fnlen < 1) {
+        char *fptr = filename;
+        fptr += snprintf(filename, sizeof(filename), "lf-hitag-");
+        FillFileNameByUID(fptr, data, "-dump", 4);
+    }
+
+    print_hex_break(data, 48, 4);
+
+    pm3_save_dump(filename, data, 48, jsfHitag, 4);
     return PM3_SUCCESS;
 }
 
@@ -947,7 +945,6 @@ void annotateHitag2(char *exp, size_t size, const uint8_t *cmd, uint8_t cmdsize,
 
 }
 
-
 void annotateHitagS(char *exp, size_t size, const uint8_t *cmd, uint8_t cmdsize, bool is_response) {
 }
 
