@@ -37,7 +37,7 @@ static int CmdHelp(const char *Cmd);
 int demodMotorola(bool verbose) {
     (void) verbose; // unused so far
     //PSK1
-    if (PSKDemod(32, 1, 100, true) != PM3_SUCCESS) {
+    if (PSKDemod(32, 1, 100, false) != PM3_SUCCESS) {
         PrintAndLogEx(DEBUG, "DEBUG: Error - Motorola: PSK Demod failed");
         return PM3_ESOFT;
     }
@@ -194,9 +194,9 @@ static int CmdMotorolaClone(const char *Cmd) {
     CLIParserInit(&ctx, "lf motorola clone",
                   "clone Motorola UID to a T55x7, Q5/T5555 or EM4305/4469 tag.\n"
                   "defaults to 64 bit format",
-                  "lf motorola clone --raw a0000000a0002021\n"
-                  "lf motorola clone --q5 --raw a0000000a0002021   -> encode for Q5/T5555 tag\n"
-                  "lf motorola clone --em --raw a0000000a0002021   -> encode for EM4305/4469"
+                  "lf motorola clone --raw a0000000a0002021       -> encode for T55x7 tag\n"
+                  "lf motorola clone --raw a0000000a0002021 --q5  -> encode for Q5/T5555 tag\n"
+                  "lf motorola clone --raw a0000000a0002021 --em  -> encode for EM4305/4469"
                  );
 
     void *argtable[] = {
