@@ -39,25 +39,24 @@ void print_type4_cc_info(uint8_t *d, uint8_t n) {
 
     PrintAndLogEx(NORMAL, "");
     PrintAndLogEx(SUCCESS, "------------ " _CYAN_("Capability Container file") " ------------");
-    PrintAndLogEx(SUCCESS, " len      %u bytes (" _GREEN_("0x%02X") ")", d[1], d[1]);
-    PrintAndLogEx(SUCCESS, " version  %s (" _GREEN_("0x%02X") ")", (d[2] == 0x20) ? "v2.0" : "v1.0", d[2]);
-
+    PrintAndLogEx(SUCCESS, " Version... %s ( " _GREEN_("0x%02X") " )", (d[2] == 0x20) ? "v2.0" : "v1.0", d[2]);
+    PrintAndLogEx(SUCCESS, " Len....... %u bytes ( " _GREEN_("0x%02X") " )", d[1], d[1]);
     uint16_t maxr = (d[3] << 8 | d[4]);
-    PrintAndLogEx(SUCCESS, " max bytes read  %u bytes ( 0x%04X )", maxr, maxr);
+    PrintAndLogEx(SUCCESS, " Max bytes read  %u bytes ( 0x%04X )", maxr, maxr);
     uint16_t maxw = (d[5] << 8 | d[6]);
-    PrintAndLogEx(SUCCESS, " max bytes write %u bytes ( 0x%04X )", maxw, maxw);
+    PrintAndLogEx(SUCCESS, " Max bytes write %u bytes ( 0x%04X )", maxw, maxw);
     PrintAndLogEx(NORMAL, "");
-    PrintAndLogEx(SUCCESS, " NDEF file control TLV  {");
-    PrintAndLogEx(SUCCESS, "    (t) type of file  ( %02X )", d[7]);
-    PrintAndLogEx(SUCCESS, "    (v)               ( %02X )", d[8]);
-    PrintAndLogEx(SUCCESS, "    file id           ( %02X%02X )", d[9], d[10]);
+    PrintAndLogEx(SUCCESS, " NDEF file control TLV");
+    PrintAndLogEx(SUCCESS, "    (t) type of file.... %02X", d[7]);
+    PrintAndLogEx(SUCCESS, "    (v) ................ %02X", d[8]);
+    PrintAndLogEx(SUCCESS, "    file id............. %02X%02X", d[9], d[10]);
 
     uint16_t maxndef = (d[11] << 8 | d[12]);
-    PrintAndLogEx(SUCCESS, "    max NDEF filesize   %u bytes ( 0x%04X )", maxndef, maxndef);
-    PrintAndLogEx(SUCCESS, "    ----- " _CYAN_("access rights") " -------");
+    PrintAndLogEx(SUCCESS, "    Max NDEF filesize... %u bytes ( 0x%04X )", maxndef, maxndef);
+    PrintAndLogEx(SUCCESS, "    " _CYAN_("Access rights"));
     PrintAndLogEx(SUCCESS, "    read   ( %02X ) protection: %s", d[13], ((d[13] & 0x80) == 0x80) ? _RED_("enabled") : _GREEN_("disabled"));
     PrintAndLogEx(SUCCESS, "    write  ( %02X ) protection: %s", d[14], ((d[14] & 0x80) == 0x80) ? _RED_("enabled") : _GREEN_("disabled"));
-    PrintAndLogEx(SUCCESS, " }");
+    PrintAndLogEx(SUCCESS, "");
     PrintAndLogEx(SUCCESS, "----------------- " _CYAN_("raw") " -----------------");
     PrintAndLogEx(SUCCESS, "%s", sprint_hex_inrow(d, n));
     PrintAndLogEx(NORMAL, "");
@@ -104,44 +103,27 @@ static int CmdNfcDecode(const char *Cmd) {
     int res = PM3_SUCCESS;
     if (fnlen != 0) {
 
+        // read dump file
         uint8_t *dump = NULL;
         size_t bytes_read = 4096;
-        DumpFileType_t dftype = getfiletype(filename);
-        switch (dftype) {
-            case BIN: {
-                res = loadFile_safe(filename, ".bin", (void **)&dump, &bytes_read);
-                break;
-            }
-            case EML: {
-                res = loadFileEML_safe(filename, (void **)&dump, &bytes_read);
-                break;
-            }
-            case JSON:
-            case DICTIONARY: {
-                free(dump);
-                PrintAndLogEx(ERR, "Error: Only BIN/EML formats allowed");
-                return PM3_EINVARG;
-            }
+        res = pm3_load_dump(filename, (void **)&dump, &bytes_read, 4096);
+        if (res != PM3_SUCCESS || dump == NULL) {
+            return res;
         }
 
-        if (res != PM3_SUCCESS) {
-            PrintAndLogEx(ERR, "error, something went wrong when loading file");
-            free(dump);
-            return PM3_EFILE;
-        }
-
-
         res = NDEFDecodeAndPrint(dump, bytes_read, verbose);
         if (res != PM3_SUCCESS) {
             PrintAndLogEx(INFO, "Trying to parse NDEF records w/o NDEF header");
-            res = NDEFRecordsDecodeAndPrint(dump, bytes_read);
+            res = NDEFRecordsDecodeAndPrint(dump, bytes_read, verbose);
         }
+
         free(dump);
+
     } else {
         res = NDEFDecodeAndPrint(data, datalen, verbose);
         if (res != PM3_SUCCESS) {
             PrintAndLogEx(INFO, "Trying to parse NDEF records w/o NDEF header");
-            res = NDEFRecordsDecodeAndPrint(data, datalen);
+            res = NDEFRecordsDecodeAndPrint(data, datalen, verbose);
         }
     }
     return res;
@@ -156,6 +138,7 @@ static int CmdNFCType1Help(const char *Cmd);
 static command_t CommandNFCType1Table[] = {
 
     {"--------",    CmdNFCType1Help,  AlwaysAvailable, "-------------- " _CYAN_("NFC Forum Tag Type 1") " ---------------"},
+//    {"format",     CmdNFCType1Format,  IfPm3Iso14443a,  "format ISO-14443-a tag as NFC Tag"},
     {"read",        CmdNFCType1Read,  IfPm3Iso14443a,  "read NFC Forum Tag Type 1"},
 //    {"write",        CmdNFCType1Write, IfPm3Iso14443a, "write NFC Forum Tag Type 1"},
     {"--------",    CmdNFCType1Help,  AlwaysAvailable, "--------------------- " _CYAN_("General") " ---------------------"},
@@ -183,6 +166,7 @@ static int CmdNFCType2Help(const char *Cmd);
 static command_t CommandNFCType2Table[] = {
 
     {"--------",    CmdNFCType2Help,  AlwaysAvailable, "-------------- " _CYAN_("NFC Forum Tag Type 2") " ---------------"},
+//    {"format",     CmdNFCType2Format,  IfPm3Iso14443a,  "format ISO-14443-a tag as NFC Tag"},
     {"read",        CmdNFCType2Read,  IfPm3Iso14443a,  "read NFC Forum Tag Type 2"},
 //    {"write",        CmdNFCType2Write, IfPm3Iso14443a, "write NFC Forum Tag Type 2"},
     {"--------",    CmdNFCType2Help,  AlwaysAvailable, "--------------------- " _CYAN_("General") " ---------------------"},
@@ -211,8 +195,9 @@ static int CmdNFCType3Help(const char *Cmd);
 static command_t CommandNFCType3Table[] = {
 
     {"--------",    CmdNFCType3Help,  AlwaysAvailable, "-------------- " _CYAN_("NFC Forum Tag Type 3") " ---------------"},
+//    {"format",        CmdNFCType3Format,  IfPm3Felica, "format FeliCa tag as NFC Tag"},
     {"read",        CmdNFCType3Read,  IfPm3Felica, "read NFC Forum Tag Type 3"},
-    {"write",       CmdNFCType3Write, IfPm3Felica, "write NFC Forum Tag Type 3"},
+//    {"write",       CmdNFCType3Write, IfPm3Felica, "write NFC Forum Tag Type 3"},
     {"--------",    CmdNFCType3Help,  AlwaysAvailable, "--------------------- " _CYAN_("General") " ---------------------"},
     {"help",        CmdNFCType3Help,  AlwaysAvailable, "This help"},
     {NULL, NULL, NULL, NULL}
@@ -238,19 +223,28 @@ static int CmdNFCST25TARead(const char *Cmd) {
     return CmdHFST25TANdefRead(Cmd);
 }
 
+static int CmdNFCType4AFormat(const char *Cmd) {
+    return CmdHF14ANdefFormat(Cmd);
+}
+
+static int CmdNFCType4AWrite(const char *Cmd) {
+    return CmdHF14ANdefWrite(Cmd);
+}
+
 static int CmdNFCType4AHelp(const char *Cmd);
 
 static command_t CommandNFCType4ATable[] = {
 
-    {"--------",     CmdNFCType4AHelp,  AlwaysAvailable, "--------- " _CYAN_("NFC Forum Tag Type 4 ISO14443A") " ----------"},
-    {"read",         CmdNFCType4ARead,  IfPm3Iso14443a,  "read NFC Forum Tag Type 4 A"},
-//    {"write",        CmdNFCType4AWrite, IfPm3Iso14443a,  "write NFC Forum Tag Type 4 A"},
+    {"--------",     CmdNFCType4AHelp,   AlwaysAvailable, "--------- " _CYAN_("NFC Forum Tag Type 4 ISO14443A") " ----------"},
+    {"format",       CmdNFCType4AFormat, IfPm3Iso14443a,  "format ISO-14443-a tag as NFC Tag"},
+    {"read",         CmdNFCType4ARead,   IfPm3Iso14443a,  "read NFC Forum Tag Type 4 A"},
+    {"write",        CmdNFCType4AWrite,  IfPm3Iso14443a,  "write NFC Forum Tag Type 4 A"},
 //    {"mfdesread",    CmdNFCMFDESRead,   IfPm3Iso14443a,  "read NDEF from MIFARE DESfire"}, // hf mfdes ndefread
 //    {"mfdesformat",  CmdNFCMFDESFormat, IfPm3Iso14443a,  "format MIFARE DESfire as NFC Forum Tag Type 4"},
-    {"st25taread",   CmdNFCST25TARead,  IfPm3Iso14443a,  "read ST25TA as NFC Forum Tag Type 4"},
+    {"st25taread",   CmdNFCST25TARead,   IfPm3Iso14443a,  "read ST25TA as NFC Forum Tag Type 4"},
 
-    {"--------",     CmdNFCType4AHelp,  AlwaysAvailable, "--------------------- " _CYAN_("General") " ---------------------"},
-    {"help",         CmdNFCType4AHelp,  AlwaysAvailable, "This help"},
+    {"--------",     CmdNFCType4AHelp,   AlwaysAvailable, "--------------------- " _CYAN_("General") " ---------------------"},
+    {"help",         CmdNFCType4AHelp,   AlwaysAvailable, "This help"},
     {NULL, NULL, NULL, NULL}
 };
 
@@ -274,6 +268,7 @@ static int CmdNFCType4BHelp(const char *Cmd);
 static command_t CommandNFCType4BTable[] = {
 
     {"--------",    CmdNFCType4BHelp,  AlwaysAvailable, "--------- " _CYAN_("NFC Forum Tag Type 4 ISO14443B") " -------------"},
+//    {"format",     CmdNFCType4BFormat,  IfPm3Iso14443b,  "format ISO-14443-b tag as NFC Tag"},
     {"read",        CmdNFCType4BRead,  IfPm3Iso14443b,  "read NFC Forum Tag Type 4 B"},
 //    {"write",       CmdNFCType4BWrite, IfPm3Iso14443b,  "write NFC Forum Tag Type 4 B"},
     {"--------",    CmdNFCType4BHelp,  AlwaysAvailable, "--------------------- " _CYAN_("General") " ---------------------"},
@@ -302,8 +297,9 @@ static int CmdNFCType5Help(const char *Cmd);
 static command_t CommandNFCType5Table[] = {
 
     {"--------",    CmdNFCType5Help,  AlwaysAvailable, "-------------- " _CYAN_("NFC Forum Tag Type 5") " ---------------"},
+//    {"format",     CmdNFCType5Format,  IfPm3Iso15693,  "format ISO-15693 tag as NFC Tag"},
     {"read",        CmdNFCType5Read,  IfPm3Iso15693,   "read NFC Forum Tag Type 5"},
-    {"write",       CmdNFCType5Write, IfPm3Iso15693,   "write NFC Forum Tag Type 5"},
+//    {"write",       CmdNFCType5Write, IfPm3Iso15693,   "write NFC Forum Tag Type 5"},
     {"--------",    CmdNFCType5Help,  AlwaysAvailable, "--------------------- " _CYAN_("General") " ---------------------"},
     {"help",        CmdNFCType5Help,  AlwaysAvailable, "This help"},
     {NULL, NULL, NULL, NULL}
@@ -325,6 +321,15 @@ static int CmdNFCMFCRead(const char *Cmd) {
     return CmdHFMFNDEFRead(Cmd);
 }
 
+static int CmdNFCMFCFormat(const char *Cmd) {
+    return CmdHFMFNDEFFormat(Cmd);
+}
+
+static int CmdNFCMFCWrite(const char *Cmd) {
+    return CmdHFMFNDEFWrite(Cmd);
+}
+
+
 static int CmdNFCMFPRead(const char *Cmd) {
     return CmdHFMFPNDEFRead(Cmd);
 }
@@ -334,9 +339,9 @@ static int CmdNFCMFHelp(const char *Cmd);
 static command_t CommandMFTable[] = {
 
     {"--------",    CmdNFCMFHelp,     AlwaysAvailable, "--------- " _CYAN_("NFC Type MIFARE Classic/Plus Tag") " --------"},
+    {"cformat",     CmdNFCMFCFormat,  IfPm3Iso14443a,  "format MIFARE Classic Tag as NFC Tag"},
     {"cread",       CmdNFCMFCRead,    IfPm3Iso14443a,  "read NFC Type MIFARE Classic Tag"},
-//    {"cwrite",       CmdNFCMFCWrite,  IfPm3Iso14443a,  "write NFC Type MIFARE Classic Tag"},
-//    {"cformat",      CmdNFCMFCFormat, IfPm3Iso14443a,  "format MIFARE Classic Tag as NFC Tag"},
+    {"cwrite",      CmdNFCMFCWrite,  IfPm3Iso14443a,   "write NFC Type MIFARE Classic Tag"},
     {"pread",       CmdNFCMFPRead,    IfPm3Iso14443a,  "read NFC Type MIFARE Plus Tag"},
     {"--------",    CmdNFCMFHelp,     AlwaysAvailable, "--------------------- " _CYAN_("General") " ---------------------"},
     {"help",        CmdNFCMFHelp,     AlwaysAvailable, "This help"},
@@ -405,6 +410,7 @@ static command_t CommandTable[] = {
     {"--------",    CmdHelp,          AlwaysAvailable, "--------------------- " _CYAN_("General") " ---------------------"},
     {"help",        CmdHelp,          AlwaysAvailable, "This help"},
     {"decode",      CmdNfcDecode,     AlwaysAvailable, "Decode NDEF records"},
+//    {"encode",      CmdNfcEncode,     AlwaysAvailable, "Encode NDEF records"},
     {NULL, NULL, NULL, NULL}
 };
 
