@@ -35,7 +35,7 @@ static int CmdHelp(const char *Cmd);
 
 // trace pointer
 static uint8_t *gs_trace;
-static long gs_traceLen = 0;
+static uint16_t gs_traceLen = 0;
 
 static bool is_last_record(uint16_t tracepos, uint16_t traceLen) {
     return ((tracepos + TRACELOG_HDR_LEN) >= traceLen);
@@ -79,6 +79,334 @@ static bool merge_topaz_reader_frames(uint32_t timestamp, uint32_t *duration, ui
 
     return true;
 }
+static uint8_t calc_pos(const uint8_t *d) {
+    // PCB [CID] [NAD] [INF] CRC CRC
+    uint8_t pos = 1;
+    if ((d[0] & 0x08) == 0x08)  // cid byte following
+        pos++;
+
+    if ((d[0] & 0x04) == 0x04)  // nad byte following
+        pos++;
+
+    return pos;
+}
+
+static uint8_t extract_uid[10] = {0};
+static uint8_t extract_uidlen = 0;
+static uint8_t extract_epurse[8] = {0};
+
+#define SKIP_TO_NEXT(a)  (TRACELOG_HDR_LEN + (a)->data_len + TRACELOG_PARITY_LEN((a)))
+
+static uint16_t extractChall_ev2(uint16_t tracepos, uint8_t *trace, uint8_t cmdpos, uint8_t long_jmp) {
+    tracelog_hdr_t *next_hdr = (tracelog_hdr_t *)(trace + tracepos);
+    if (next_hdr->data_len != 21) {
+        return 0;
+    }
+
+    tracepos += TRACELOG_HDR_LEN + next_hdr->data_len + TRACELOG_PARITY_LEN(next_hdr);
+
+    PrintAndLogEx(INFO, "1499999999 %s " NOLF, sprint_hex_inrow(next_hdr->frame + 1, 16));
+
+    next_hdr = (tracelog_hdr_t *)(trace + tracepos);
+    tracepos += TRACELOG_HDR_LEN + next_hdr->data_len + TRACELOG_PARITY_LEN(next_hdr);
+
+    if (next_hdr->frame[cmdpos] == MFDES_ADDITIONAL_FRAME) {
+        PrintAndLogEx(NORMAL, "%s", sprint_hex_inrow(next_hdr->frame + cmdpos + long_jmp, 32));
+    } else {
+        PrintAndLogEx(NORMAL, "");
+    }
+    return tracepos;
+}
+
+static uint16_t extractChallenges(uint16_t tracepos, uint16_t traceLen, uint8_t *trace) {
+
+    // sanity check
+    if (is_last_record(tracepos, traceLen)) {
+        return traceLen;
+    }
+
+    tracelog_hdr_t *hdr = (tracelog_hdr_t *)(trace + tracepos);
+    uint16_t data_len = hdr->data_len;
+    uint8_t *frame = hdr->frame;
+
+    // sanity check tracking position is less then available trace size
+    if (tracepos + TRACELOG_HDR_LEN + data_len + TRACELOG_PARITY_LEN(hdr) > traceLen) {
+        PrintAndLogEx(DEBUG, "trace pos offset %"PRIu64 " larger than reported tracelen %u",
+                      tracepos + TRACELOG_HDR_LEN + data_len + TRACELOG_PARITY_LEN(hdr),
+                      traceLen
+                     );
+        return traceLen;
+    }
+
+    // set trace position
+    tracepos += SKIP_TO_NEXT(hdr);
+
+    // sanity check data frame length
+    if (data_len == 0) {
+        return tracepos;
+    }
+
+    // extract MFC
+    switch (frame[0]) {
+        case MIFARE_AUTH_KEYA: {
+            if (data_len > 3) {
+            }
+            break;
+        }
+        case MIFARE_AUTH_KEYB: {
+            if (data_len > 3) {
+            }
+            break;
+        }
+    }
+
+    // extract MFU-C
+    switch (frame[0]) {
+        case MIFARE_ULC_AUTH_1: {
+            if (data_len != 4) {
+                break;
+            }
+
+            // time to skip to next
+            tracelog_hdr_t *next_hdr = (tracelog_hdr_t *)(trace + tracepos);
+            tracepos += SKIP_TO_NEXT(next_hdr);
+            if (next_hdr->data_len != 11) {
+                break;
+            }
+
+            if (next_hdr->frame[0] != MIFARE_ULC_AUTH_2) {
+                break;
+            }
+
+            PrintAndLogEx(INFO, "MFU-C AUTH");
+            PrintAndLogEx(INFO, "3DES %s " NOLF, sprint_hex_inrow(next_hdr->frame + 1, 8));
+
+            next_hdr = (tracelog_hdr_t *)(trace + tracepos);
+            tracepos += SKIP_TO_NEXT(next_hdr);
+
+            if (next_hdr->frame[0] == MIFARE_ULC_AUTH_2 && next_hdr->data_len == 19) {
+                PrintAndLogEx(NORMAL, "%s", sprint_hex_inrow(next_hdr->frame + 1, 16));
+            }
+
+            return tracepos;
+        }
+    }
+
+    // extract iCLASS
+    // --csn 9655a400f8ff12e0 --epurse f0ffffffffffffff --macs 0000000089cb984b
+
+    if (hdr->isResponse == false)  {
+
+        uint8_t c = frame[0] & 0x0F;
+        switch (c) {
+            case ICLASS_CMD_SELECT: {
+
+                tracelog_hdr_t *next_hdr = (tracelog_hdr_t *)(trace + tracepos);
+                tracepos += SKIP_TO_NEXT(next_hdr);
+                if (next_hdr->data_len != 10) {
+                    break;
+                }
+                memcpy(extract_uid, next_hdr->frame, 8);
+                extract_uidlen = 8;
+                break;
+            }
+            case ICLASS_CMD_READCHECK: {
+
+                // get epurse
+                if (frame[1] == 2 && data_len == 2) {
+                    tracelog_hdr_t *next_hdr = (tracelog_hdr_t *)(trace + tracepos);
+                    tracepos += SKIP_TO_NEXT(next_hdr);
+                    if (next_hdr->data_len < 8) {
+                        break;
+                    }
+                    memcpy(extract_epurse, next_hdr->frame, 8);
+                }
+                break;
+            }
+            case ICLASS_CMD_CHECK: {
+                // get macs
+                if (data_len == 9) {
+                    if (extract_uidlen == 8) {
+                        PrintAndLogEx(INFO, "hf iclass lookup --csn %s " NOLF, sprint_hex_inrow(extract_uid, extract_uidlen));
+                        PrintAndLogEx(NORMAL, "--epurse %s " NOLF, sprint_hex_inrow(extract_epurse, 8));
+                        PrintAndLogEx(NORMAL, "--macs %s " NOLF, sprint_hex_inrow(frame + 1, 8));
+                        PrintAndLogEx(NORMAL, "-f iclass_default_keys.dic");
+                        return tracepos;
+                    }
+                }
+                break;
+            }
+        }
+    }
+
+    // extract UID
+    switch (frame[0]) {
+        case ISO14443A_CMD_ANTICOLL_OR_SELECT: {
+            // 93 20 = Anticollision (usage: 9320 - answer: 4bytes UID+1byte UID-bytes-xor)
+            // 93 50 = Bit oriented anti-collision (usage: 9350+ up to 5bytes, 9350 answer - up to 5bytes UID+BCC)
+            // 93 70 = Select (usage: 9370+5bytes 9370 answer - answer: 1byte SAK)
+            if (frame[1] == 0x70) {
+                if (frame[2] == 0x88) {
+                    memcpy(extract_uid, frame + 3, 3);
+                    extract_uidlen = 3;
+                } else {
+                    memcpy(extract_uid, frame + 2, 4);
+                    extract_uidlen = 4;
+                    PrintAndLogEx(INFO, "UID... " _YELLOW_("%s"), sprint_hex_inrow(extract_uid, extract_uidlen));
+                }
+            }
+            break;
+        }
+        case ISO14443A_CMD_ANTICOLL_OR_SELECT_2: {
+            // 95 20 = Anticollision of cascade level2
+            // 95 50 = Bit oriented anti-collision level2
+            // 95 70 = Select of cascade level2
+            if (frame[1] == 0x70) {
+                if (frame[2] == 0x88) {
+                    memcpy(extract_uid + extract_uidlen, frame + 3, 3);
+                    extract_uidlen += 3;
+                } else {
+                    memcpy(extract_uid + extract_uidlen, frame + 2, 4);
+                    extract_uidlen += 4;
+                    PrintAndLogEx(INFO, "UID... " _YELLOW_("%s"), sprint_hex_inrow(extract_uid, extract_uidlen));
+                }
+            }
+            break;
+        }
+        case ISO14443A_CMD_ANTICOLL_OR_SELECT_3: {
+            // 97 20 = Anticollision of cascade level3
+            // 97 50 = Bit oriented anti-collision level3
+            // 97 70 = Select of cascade level3
+            if (frame[1] == 0x70) {
+                memcpy(extract_uid + extract_uidlen, frame + 2, 4);
+                extract_uidlen += 4;
+                PrintAndLogEx(INFO, "UID... " _YELLOW_("%s"), sprint_hex_inrow(extract_uid, extract_uidlen));
+            }
+            break;
+        }
+    }
+
+    // extract DESFIRE
+    if ((frame[0] & 0xC0) != 0x00) {
+        return tracepos;
+    }
+
+    if (hdr->isResponse) {
+        return tracepos;
+    }
+
+    // PCB [CID] [NAD] [INF] CRC CRC
+    uint8_t pos = calc_pos(frame);
+    uint8_t long_jmp = (data_len > 6) ? 4 : 1;
+
+    for (uint8_t i = 0; i < 2; i++, pos++) {
+
+        switch (frame[pos]) {
+
+            case MFDES_AUTHENTICATE: {
+
+                // Assume wrapped or unwrapped
+                PrintAndLogEx(INFO, "AUTH NATIVE (keyNo %d)", frame[pos + long_jmp]);
+                if (next_record_is_response(tracepos, trace) == false) {
+                    break;
+                }
+
+                tracelog_hdr_t *next_hdr = (tracelog_hdr_t *)(trace + tracepos);
+                if (next_hdr->data_len < 7) {
+                    break;
+                }
+                tracepos += TRACELOG_HDR_LEN + next_hdr->data_len + TRACELOG_PARITY_LEN(next_hdr);
+
+                PrintAndLogEx(INFO, "DES 1499999999 %s " NOLF, sprint_hex_inrow(next_hdr->frame + 1, 8));
+
+                next_hdr = (tracelog_hdr_t *)(trace + tracepos);
+                tracepos += TRACELOG_HDR_LEN + next_hdr->data_len + TRACELOG_PARITY_LEN(next_hdr);
+
+                if (next_hdr->frame[pos] == MFDES_ADDITIONAL_FRAME) {
+                    PrintAndLogEx(NORMAL, "%s", sprint_hex_inrow(next_hdr->frame + pos + long_jmp, 16));
+                } else {
+                    PrintAndLogEx(NORMAL, "");
+                }
+                return tracepos;   // AUTHENTICATE_NATIVE
+            }
+            case MFDES_AUTHENTICATE_ISO: {
+                // Assume wrapped or unwrapped
+                PrintAndLogEx(INFO, "AUTH ISO (keyNo %d)", frame[pos + long_jmp]);
+                if (next_record_is_response(tracepos, trace) == false) {
+                    break;
+                }
+
+                tracelog_hdr_t *next_hdr = (tracelog_hdr_t *)(trace + tracepos);
+                tracepos += TRACELOG_HDR_LEN + next_hdr->data_len + TRACELOG_PARITY_LEN(next_hdr);
+                if (next_hdr->data_len < 7) {
+                    break;
+                }
+
+                uint8_t tdea = 8;
+                if (next_hdr->data_len > 20) {
+                    tdea = 16;
+                    PrintAndLogEx(INFO, "3TDEA 1499999999 %s " NOLF, sprint_hex_inrow(next_hdr->frame + 1, tdea));
+                } else {
+                    PrintAndLogEx(INFO, "2TDEA 1499999999 %s " NOLF, sprint_hex_inrow(next_hdr->frame + 1, tdea));
+                }
+
+                next_hdr = (tracelog_hdr_t *)(trace + tracepos);
+                tracepos += TRACELOG_HDR_LEN + next_hdr->data_len + TRACELOG_PARITY_LEN(next_hdr);
+
+                if (next_hdr->frame[pos] == MFDES_ADDITIONAL_FRAME) {
+                    PrintAndLogEx(NORMAL, "%s", sprint_hex_inrow(next_hdr->frame + pos + long_jmp, (tdea << 1)));
+                } else {
+                    PrintAndLogEx(NORMAL, "");
+                }
+                return tracepos;  // AUTHENTICATE_STANDARD
+            }
+            case MFDES_AUTHENTICATE_AES: {
+                // Assume wrapped or unwrapped
+                PrintAndLogEx(INFO, "AUTH AES (keyNo %d)", frame[pos + long_jmp]);
+                if (next_record_is_response(tracepos, trace)) {
+                    break;
+                }
+
+                tracelog_hdr_t *next_hdr = (tracelog_hdr_t *)(trace + tracepos);
+                tracepos += TRACELOG_HDR_LEN + next_hdr->data_len + TRACELOG_PARITY_LEN(next_hdr);
+                if (next_hdr->data_len < 7) {
+                    break;
+                }
+
+                PrintAndLogEx(INFO, "AES 1499999999 %s " NOLF, sprint_hex_inrow(next_hdr->frame + 1, 8));
+
+                next_hdr = (tracelog_hdr_t *)(trace + tracepos);
+                tracepos += TRACELOG_HDR_LEN + next_hdr->data_len + TRACELOG_PARITY_LEN(next_hdr);
+
+                if (next_hdr->frame[pos] == MFDES_ADDITIONAL_FRAME) {
+                    PrintAndLogEx(NORMAL, "%s", sprint_hex_inrow(next_hdr->frame + pos + long_jmp, 16));
+                } else {
+                    PrintAndLogEx(NORMAL, "");
+                }
+                return tracepos;
+            }
+            case MFDES_AUTHENTICATE_EV2F: {
+                PrintAndLogEx(INFO, "AUTH EV2 First");
+                uint16_t tmp = extractChall_ev2(tracepos, trace, pos, long_jmp);
+                if (tmp == 0)
+                    break;
+                else
+                    return tmp;
+
+            }
+            case MFDES_AUTHENTICATE_EV2NF: {
+                PrintAndLogEx(INFO, "AUTH EV2 Non First");
+                uint16_t tmp = extractChall_ev2(tracepos, trace, pos, long_jmp);
+                if (tmp == 0)
+                    break;
+                else
+                    return tmp;
+            }
+        }
+    }
+
+    return tracepos;
+}
 
 static uint16_t printHexLine(uint16_t tracepos, uint16_t traceLen, uint8_t *trace, uint8_t protocol) {
     // sanity check
@@ -108,18 +436,23 @@ static uint16_t printHexLine(uint16_t tracepos, uint16_t traceLen, uint8_t *trac
              * we use format timestamp, newline, offset (0x000000), pseudo header, data
              * `text2pcap -t "%S." -l 264 -n <input-text-file> <output-pcapng-file>`
              */
-            char line[(hdr->data_len * 3) + 1];
-            char *ptr = &line[0];
+            int line_len = (hdr->data_len * 3) + 1;
+            char line[line_len];
+            char *ptr = line;
 
             for (int i = 0; i < hdr->data_len ; i++) {
-                ptr += sprintf(ptr, "%02x ", hdr->frame[i]);
+                ptr += snprintf(ptr, line_len, "%02x ", hdr->frame[i]);
+                line_len -= 3;
+                if (line_len <= 0) {
+                    break;
+                }
             }
 
             char data_len_str[5];
             char temp_str1[3] = {0};
             char temp_str2[3] = {0};
 
-            sprintf(data_len_str, "%04x", hdr->data_len);
+            snprintf(data_len_str, sizeof(data_len_str), "%04x", hdr->data_len);
             memmove(temp_str1, data_len_str, 2);
             memmove(temp_str2, data_len_str + 2, 2);
 
@@ -150,20 +483,19 @@ static uint16_t printTraceLine(uint16_t tracepos, uint16_t traceLen, uint8_t *tr
     }
 
     uint32_t end_of_transmission_timestamp = 0;
-    uint32_t duration;
-    uint16_t data_len;
     uint8_t topaz_reader_command[9];
     char explanation[40] = {0};
-    uint8_t mfData[32] = {0};
-    size_t mfDataLen = 0;
     tracelog_hdr_t *first_hdr = (tracelog_hdr_t *)(trace);
     tracelog_hdr_t *hdr = (tracelog_hdr_t *)(trace + tracepos);
 
-    duration = hdr->duration;
-    data_len = hdr->data_len;
+    uint32_t duration = hdr->duration;
+    uint16_t data_len = hdr->data_len;
 
     if (tracepos + TRACELOG_HDR_LEN + data_len + TRACELOG_PARITY_LEN(hdr) > traceLen) {
-        PrintAndLogEx(DEBUG, "trace pos offset %"PRIu64 " larger than reported tracelen %u", tracepos + TRACELOG_HDR_LEN + data_len + TRACELOG_PARITY_LEN(hdr), traceLen);
+        PrintAndLogEx(DEBUG, "trace pos offset %"PRIu64 " larger than reported tracelen %u",
+                      tracepos + TRACELOG_HDR_LEN + data_len + TRACELOG_PARITY_LEN(hdr),
+                      traceLen
+                     );
         return traceLen;
     }
 
@@ -206,6 +538,7 @@ static uint16_t printTraceLine(uint16_t tracepos, uint16_t traceLen, uint8_t *tr
             case ISO_14443A:
             case MFDES:
             case LTO:
+            case SEOS:
                 crcStatus = iso14443A_CRC_check(hdr->isResponse, frame, data_len);
                 break;
             case ISO_7816_4:
@@ -238,15 +571,15 @@ static uint16_t printTraceLine(uint16_t tracepos, uint16_t traceLen, uint8_t *tr
     //2 Not crc-command
 
     //--- Draw the data column
-    char line[18][120] = {{0}};
+    char line[18][160] = {{0}};
 
     if (data_len == 0) {
         if (protocol == ICLASS && duration == 2048) {
-            sprintf(line[0], "<SOF>");
+            snprintf(line[0], sizeof(line[0]), "<SOF>");
         } else if (protocol == ISO_15693 && duration == 512) {
-            sprintf(line[0], "<EOF>");
+            snprintf(line[0], sizeof(line[0]), "<EOF>");
         } else {
-            sprintf(line[0], "<empty trace - possible error>");
+            snprintf(line[0], sizeof(line[0]), "<empty trace - possible error>");
         }
     }
     uint8_t partialbytebuff = 0;
@@ -265,7 +598,7 @@ static uint16_t printTraceLine(uint16_t tracepos, uint16_t traceLen, uint8_t *tr
                 && protocol != FELICA
                 && protocol != LTO
                 && protocol != PROTO_CRYPTORF
-                && (hdr->isResponse || protocol == ISO_14443A || protocol == PROTO_MIFARE)
+                && (hdr->isResponse || protocol == ISO_14443A || protocol == PROTO_MIFARE || protocol == SEOS)
                 && (oddparity8(frame[j]) != ((parityBits >> (7 - (j & 0x0007))) & 0x01))) {
 
             snprintf(line[j / 18] + ((j % 18) * 4), 120, "%02x! ", frame[j]);
@@ -298,7 +631,9 @@ static uint16_t printTraceLine(uint16_t tracepos, uint16_t traceLen, uint8_t *tr
 
     }
 
-    if (markCRCBytes) {
+
+    uint8_t crc_format_string_offset = 0;
+    if (markCRCBytes && data_len > 2) {
         //CRC-command
         if (((protocol == PROTO_HITAG1) || (protocol == PROTO_HITAGS)) && (data_len > 1)) {
             // Note that UID REQUEST response has no CRC, but we don't know
@@ -306,19 +641,49 @@ static uint16_t printTraceLine(uint16_t tracepos, uint16_t traceLen, uint8_t *tr
             char *pos1 = line[(data_len - 1) / 18] + (((data_len - 1) % 18) * 4) + offset - 1;
             (*pos1) = '[';
             char *pos2 = line[(data_len) / 18] + (((data_len) % 18) * 4) + offset - 2;
-            sprintf(pos2, "%c", ']');
+            (*pos2) = ']';
+            (*(pos2 + 1)) = '\0';
         } else {
+
             if (crcStatus == 0 || crcStatus == 1) {
-                char *pos1 = line[(data_len - 2) / 18] + (((data_len - 2) % 18) * 4) - 1;
-                (*pos1) = '[';
-                char *pos2 = line[(data_len) / 18] + (((data_len) % 18) * 4) - 1;
-                sprintf(pos2, "%c", ']');
+
+                char *pos1 = line[(data_len - 2) / 18];
+                pos1 += (((data_len - 2) % 18) * 4) - 1;
+
+                (*(pos1 + 6 + 1)) = '\0';
+
+                char *cb_str = str_dup(pos1 + 1);
+
+                if (hdr->isResponse) {
+                    if (g_session.supports_colors) {
+                        if (crcStatus == 0) {
+                            snprintf(pos1, 24, " " _RED_("%s") " ", cb_str);
+                        } else {
+                            snprintf(pos1, 24, " " _GREEN_("%s") " ", cb_str);
+                        }
+                        crc_format_string_offset = 9;
+                    } else {
+                        snprintf(pos1, 9, "[%s]", cb_str);
+                    }
+                } else {
+                    if (g_session.supports_colors) {
+                        if (crcStatus == 0) {
+                            snprintf(pos1, 24, AEND " " _RED_("%s") " ", cb_str);
+                        } else {
+                            snprintf(pos1, 24, AEND " " _GREEN_("%s") " ", cb_str);
+                        }
+                        crc_format_string_offset = 13;
+                    } else {
+                        snprintf(pos1, 9, "[%s]", cb_str);
+                    }
+                }
+                free(cb_str);
             }
         }
     }
 
     // Draw the CRC column
-    const char *crcstrings[] = { "!crc", " ok ", "    ", "A ok", "B ok" };
+    const char *crcstrings[] = { _RED_(" !! "), _GREEN_(" ok "), "    ", _GREEN_("A ok"), _GREEN_("B ok") };
     const char *crc = crcstrings[crcStatus];
 
     // mark short bytes (less than 8 Bit + Parity)
@@ -418,6 +783,9 @@ static uint16_t printTraceLine(uint16_t tracepos, uint16_t traceLen, uint8_t *tr
             case PROTO_CRYPTORF:
                 annotateCryptoRF(explanation, sizeof(explanation), frame, data_len);
                 break;
+            case SEOS:
+                annotateSeos(explanation, sizeof(explanation), frame, data_len);
+                break;
             default:
                 break;
         }
@@ -437,17 +805,19 @@ static uint16_t printTraceLine(uint16_t tracepos, uint16_t traceLen, uint8_t *tr
             if (hdr->isResponse) {
                 // tag row
                 if (use_us) {
-                    PrintAndLogEx(NORMAL, " %10.1f | %10.1f | Tag |%-72s | %s| %s",
+                    PrintAndLogEx(NORMAL, " %10.1f | %10.1f | Tag |%-*s | %s| %s",
                                   (float)time1 / 13.56,
                                   (float)time2 / 13.56,
+                                  72 + crc_format_string_offset,
                                   line[j],
                                   (j == num_lines - 1) ? crc : "    ",
                                   (j == num_lines - 1) ? explanation : ""
                                  );
                 } else {
-                    PrintAndLogEx(NORMAL, " %10u | %10u | Tag |%-72s | %s| %s",
-                                  (hdr->timestamp - first_hdr->timestamp),
-                                  (end_of_transmission_timestamp - first_hdr->timestamp),
+                    PrintAndLogEx(NORMAL, " %10u | %10u | Tag |%-*s | %s| %s",
+                                  time1,
+                                  time2,
+                                  72 + crc_format_string_offset,
                                   line[j],
                                   (j == num_lines - 1) ? crc : "    ",
                                   (j == num_lines - 1) ? explanation : ""
@@ -457,18 +827,20 @@ static uint16_t printTraceLine(uint16_t tracepos, uint16_t traceLen, uint8_t *tr
                 // reader row
                 if (use_us) {
                     PrintAndLogEx(NORMAL,
-                                  _YELLOW_(" %10.1f") " | " _YELLOW_("%10.1f") " | " _YELLOW_("Rdr") " |" _YELLOW_("%-72s")" | " _YELLOW_("%s") "| " _YELLOW_("%s"),
+                                  _YELLOW_(" %10.1f") " | " _YELLOW_("%10.1f") " | " _YELLOW_("Rdr") " |" _YELLOW_("%-*s")" | " _YELLOW_("%s") "| " _YELLOW_("%s"),
                                   (float)time1 / 13.56,
                                   (float)time2 / 13.56,
+                                  72 + crc_format_string_offset,
                                   line[j],
                                   (j == num_lines - 1) ? crc : "    ",
                                   (j == num_lines - 1) ? explanation : ""
                                  );
                 } else {
                     PrintAndLogEx(NORMAL,
-                                  _YELLOW_(" %10u") " | " _YELLOW_("%10u") " | " _YELLOW_("Rdr") " |" _YELLOW_("%-72s")" | " _YELLOW_("%s") "| " _YELLOW_("%s"),
-                                  (hdr->timestamp - first_hdr->timestamp),
-                                  (end_of_transmission_timestamp - first_hdr->timestamp),
+                                  _YELLOW_(" %10u") " | " _YELLOW_("%10u") " | " _YELLOW_("Rdr") " |" _YELLOW_("%-*s")" | " _YELLOW_("%s") "| " _YELLOW_("%s"),
+                                  time1,
+                                  time2,
+                                  72 + crc_format_string_offset,
                                   line[j],
                                   (j == num_lines - 1) ? crc : "    ",
                                   (j == num_lines - 1) ? explanation : ""
@@ -479,13 +851,15 @@ static uint16_t printTraceLine(uint16_t tracepos, uint16_t traceLen, uint8_t *tr
 
         } else {
             if (hdr->isResponse) {
-                PrintAndLogEx(NORMAL, "            |            |     |%-72s | %s| %s",
+                PrintAndLogEx(NORMAL, "            |            |     |%-*s | %s| %s",
+                              72 + crc_format_string_offset,
                               line[j],
                               (j == num_lines - 1) ? crc : "    ",
                               (j == num_lines - 1) ? explanation : ""
                              );
             } else {
-                PrintAndLogEx(NORMAL, "            |            |     |" _YELLOW_("%-72s")" | " _YELLOW_("%s") "| " _YELLOW_("%s"),
+                PrintAndLogEx(NORMAL, "            |            |     |" _YELLOW_("%-*s")" | " _YELLOW_("%s") "| " _YELLOW_("%s"),
+                              72 + crc_format_string_offset,
                               line[j],
                               (j == num_lines - 1) ? crc : "    ",
                               (j == num_lines - 1) ? explanation : ""
@@ -495,13 +869,16 @@ static uint16_t printTraceLine(uint16_t tracepos, uint16_t traceLen, uint8_t *tr
     }
 
     if (protocol == PROTO_MIFARE) {
+        uint8_t mfData[32] = {0};
+        size_t mfDataLen = 0;
         if (DecodeMifareData(frame, data_len, parityBytes, hdr->isResponse, mfData, &mfDataLen, mfDicKeys, mfDicKeysCount)) {
             memset(explanation, 0x00, sizeof(explanation));
             annotateIso14443a(explanation, sizeof(explanation), mfData, mfDataLen, hdr->isResponse);
             uint8_t crcc = iso14443A_CRC_check(hdr->isResponse, mfData, mfDataLen);
-            PrintAndLogEx(NORMAL, "            |            |  *  |%-72s | %-4s| %s",
+            PrintAndLogEx(NORMAL, "            |            |  *  |%-*s | %-4s| %s",
+                          72 + crc_format_string_offset,
                           sprint_hex_inrow_spaces(mfData, mfDataLen, 2),
-                          (crcc == 0 ? "!crc" : (crcc == 1 ? " ok " : "    ")),
+                          (crcc == 0 ? _RED_(" !! ") : (crcc == 1 ? _GREEN_(" ok ") : "    ")),
                           explanation);
         }
     }
@@ -514,11 +891,26 @@ static uint16_t printTraceLine(uint16_t tracepos, uint16_t traceLen, uint8_t *tr
 
         tracelog_hdr_t *next_hdr = (tracelog_hdr_t *)(trace + tracepos);
 
-        PrintAndLogEx(NORMAL, " %10u | %10u | %s |fdt (Frame Delay Time): " _YELLOW_("%d"),
-                      (end_of_transmission_timestamp - first_hdr->timestamp),
-                      (next_hdr->timestamp - first_hdr->timestamp),
-                      "   ",
-                      (next_hdr->timestamp - end_of_transmission_timestamp));
+        uint32_t time1 = end_of_transmission_timestamp - first_hdr->timestamp;
+        uint32_t time2 = next_hdr->timestamp - first_hdr->timestamp;
+        if (prev_eot) {
+            time1 = 0;
+            time2 = next_hdr->timestamp - end_of_transmission_timestamp;
+        }
+
+        if (use_us) {
+            PrintAndLogEx(NORMAL, " %10.1f | %10.1f | %s |fdt (Frame Delay Time): " _YELLOW_("%.1f"),
+                          (float)time1 / 13.56,
+                          (float)time2 / 13.56,
+                          "   ",
+                          (float)(next_hdr->timestamp - end_of_transmission_timestamp) / 13.56);
+        } else {
+            PrintAndLogEx(NORMAL, " %10u | %10u | %s |fdt (Frame Delay Time): " _YELLOW_("%d"),
+                          time1,
+                          time2,
+                          "   ",
+                          (next_hdr->timestamp - end_of_transmission_timestamp));
+        }
     }
 
     return tracepos;
@@ -587,6 +979,50 @@ static int SanityOfflineCheck( bool useTraceBuffer ){
 }
 */
 
+static int CmdTraceExtract(const char *Cmd) {
+    CLIParserContext *ctx;
+    CLIParserInit(&ctx, "trace extract",
+                  "Extracts protocol authentication challenges from trace buffer\n",
+                  "trace extract\n"
+                  "trace extract -1\n"
+                 );
+
+    void *argtable[] = {
+        arg_param_begin,
+        arg_lit0("1", "buffer", "use data from trace buffer"),
+        arg_param_end
+    };
+    CLIExecWithReturn(ctx, Cmd, argtable, true);
+    bool use_buffer = arg_get_lit(ctx, 1);
+    CLIParserFree(ctx);
+
+    clearCommandBuffer();
+
+    if (use_buffer == false) {
+        download_trace();
+    } else if (gs_traceLen == 0) {
+        PrintAndLogEx(FAILED, "You requested a trace list in offline mode but there is no trace.");
+        PrintAndLogEx(FAILED, "Consider using " _YELLOW_("`trace load`") " or removing parameter " _YELLOW_("`-1`"));
+        return PM3_EINVARG;
+    }
+
+    PrintAndLogEx(SUCCESS, "Recorded activity (trace len = " _YELLOW_("%u") " bytes)", gs_traceLen);
+    if (gs_traceLen == 0) {
+        return PM3_SUCCESS;
+    }
+
+    uint16_t tracepos = 0;
+
+    while (tracepos < gs_traceLen) {
+        tracepos = extractChallenges(tracepos, gs_traceLen, gs_trace);
+
+        if (kbd_enter_pressed())
+            break;
+    }
+
+    return PM3_SUCCESS;
+}
+
 static int CmdTraceLoad(const char *Cmd) {
 
     CLIParserContext *ctx;
@@ -621,7 +1057,7 @@ static int CmdTraceLoad(const char *Cmd) {
 
     gs_traceLen = (long)len;
 
-    PrintAndLogEx(SUCCESS, "Recorded Activity (TraceLen = " _YELLOW_("%lu") " bytes)", gs_traceLen);
+    PrintAndLogEx(SUCCESS, "Recorded Activity (TraceLen = " _YELLOW_("%u") " bytes)", gs_traceLen);
     PrintAndLogEx(HINT, "try " _YELLOW_("`trace list -1 -t ...`") " to view trace.  Remember the " _YELLOW_("`-1`") " param");
     return PM3_SUCCESS;
 }
@@ -669,8 +1105,8 @@ int CmdTraceListAlias(const char *Cmd, const char *alias, const char *protocol)
              protocol);
     char example[200] = {0};
     snprintf(example, sizeof(example) - 1,
-             "%s list -f              -> show frame delay times\n"
-             "%s list -1              -> use trace buffer ",
+             "%s list --frame      -> show frame delay times\n"
+             "%s list -1           -> use trace buffer ",
              alias, alias);
     char fullalias[100] = {0};
     snprintf(fullalias, sizeof(fullalias) - 1, "%s list", alias);
@@ -679,20 +1115,20 @@ int CmdTraceListAlias(const char *Cmd, const char *alias, const char *protocol)
     void *argtable[] = {
         arg_param_begin,
         arg_lit0("1", "buffer", "use data from trace buffer"),
-        arg_lit0("f", NULL, "show frame delay times"),
+        arg_lit0(NULL, "frame", "show frame delay times"),
         arg_lit0("c", NULL, "mark CRC bytes"),
         arg_lit0("r", NULL, "show relative times (gap and duration)"),
         arg_lit0("u", NULL, "display times in microseconds instead of clock cycles"),
         arg_lit0("x", NULL, "show hexdump to convert to pcap(ng)\n"
                  "                                   or to import into Wireshark using encapsulation type \"ISO 14443\""),
-        arg_str0(NULL, "dict", "<file>", "use dictionary keys file"),
+        arg_str0("f", "file", "<fn>", "filename of dictionary"),
         arg_param_end
     };
     CLIExecWithReturn(ctx, Cmd, argtable, true);
     CLIParserFree(ctx);
 
     char args[128] = {0};
-    snprintf(args, sizeof(args), "-t %s ", protocol);
+    snprintf(args, sizeof(args), "-c -t %s ", protocol);
     strncat(args, Cmd, sizeof(args) - strlen(args) - 1);
     return CmdTraceList(args);
 }
@@ -703,38 +1139,41 @@ int CmdTraceList(const char *Cmd) {
                   "Annotate trace buffer with selected protocol data\n"
                   "You can load a trace from file (see `trace load -h`) or it be downloaded from device by default\n",
                   "trace list -t raw      -> just show raw data without annotations\n"
+                  "\n"
                   "trace list -t 14a      -> interpret as " _YELLOW_("ISO14443-A") "\n"
-                  "trace list -t thinfilm -> interpret as " _YELLOW_("Thinfilm") "\n"
-                  "trace list -t topaz    -> interpret as " _YELLOW_("Topaz") "\n"
-                  "trace list -t mf       -> interpret as " _YELLOW_("MIFARE Classic") " and decrypt crypto1 stream\n"
-                  "trace list -t des      -> interpret as " _YELLOW_("MIFARE DESFire") "\n"
                   "trace list -t 14b      -> interpret as " _YELLOW_("ISO14443-B") "\n"
-                  "trace list -t 7816     -> interpret as " _YELLOW_("ISO7816-4") "\n"
                   "trace list -t 15       -> interpret as " _YELLOW_("ISO15693") "\n"
-                  "trace list -t iclass   -> interpret as " _YELLOW_("iCLASS") "\n"
-                  "trace list -t legic    -> interpret as " _YELLOW_("LEGIC") "\n"
+                  "trace list -t 7816     -> interpret as " _YELLOW_("ISO7816-4") "\n"
+                  "trace list -t cryptorf -> interpret as " _YELLOW_("CryptoRF") "\n\n"
+                  "trace list -t des      -> interpret as " _YELLOW_("MIFARE DESFire") "\n"
                   "trace list -t felica   -> interpret as " _YELLOW_("ISO18092 / FeliCa") "\n"
                   "trace list -t hitag1   -> interpret as " _YELLOW_("Hitag1") "\n"
                   "trace list -t hitag2   -> interpret as " _YELLOW_("Hitag2") "\n"
                   "trace list -t hitags   -> interpret as " _YELLOW_("HitagS") "\n"
+                  "trace list -t iclass   -> interpret as " _YELLOW_("iCLASS") "\n"
+                  "trace list -t legic    -> interpret as " _YELLOW_("LEGIC") "\n"
                   "trace list -t lto      -> interpret as " _YELLOW_("LTO-CM") "\n"
-                  "trace list -t cryptorf -> interpret as " _YELLOW_("CryptoRF") "\n\n"
-                  "trace list -t mf --dict <mfc_default_keys>    -> use dictionary keys file\n"
-                  "trace list -t 14a -f                          -> show frame delay times\n"
-                  "trace list -t 14a -1                          -> use trace buffer "
+                  "trace list -t mf       -> interpret as " _YELLOW_("MIFARE Classic") " and decrypt crypto1 stream\n"
+                  "trace list -t seos     -> interpret as " _YELLOW_("SEOS") "\n"
+                  "trace list -t thinfilm -> interpret as " _YELLOW_("Thinfilm") "\n"
+                  "trace list -t topaz    -> interpret as " _YELLOW_("Topaz") "\n"
+                  "\n"
+                  "trace list -t mf -f mfc_default_keys.dic     -> use default dictionary file\n"
+                  "trace list -t 14a --frame                    -> show frame delay times\n"
+                  "trace list -t 14a -1                         -> use trace buffer "
                  );
 
     void *argtable[] = {
         arg_param_begin,
         arg_lit0("1", "buffer", "use data from trace buffer"),
-        arg_lit0("f", NULL, "show frame delay times"),
+        arg_lit0(NULL, "frame", "show frame delay times"),
         arg_lit0("c", NULL, "mark CRC bytes"),
         arg_lit0("r", NULL, "show relative times (gap and duration)"),
         arg_lit0("u", NULL, "display times in microseconds instead of clock cycles"),
         arg_lit0("x", NULL, "show hexdump to convert to pcap(ng)\n"
                  "                                   or to import into Wireshark using encapsulation type \"ISO 14443\""),
         arg_str0("t", "type", NULL, "protocol to annotate the trace"),
-        arg_str0(NULL, "dict", "<fn>", "use dictionary keys file"),
+        arg_str0("f", "file", "<fn>", "filename of dictionary"),
         arg_param_end
     };
     CLIExecWithReturn(ctx, Cmd, argtable, true);
@@ -766,23 +1205,24 @@ int CmdTraceList(const char *Cmd) {
     uint8_t protocol = -1;
 
     // validate type of output
-    if (strcmp(type,      "iclass") == 0)   protocol = ICLASS;
-    else if (strcmp(type, "14a") == 0)      protocol = ISO_14443A;
+    if (strcmp(type, "14a") == 0)      protocol = ISO_14443A;
     else if (strcmp(type, "14b") == 0)      protocol = ISO_14443B;
-    else if (strcmp(type, "topaz") == 0)    protocol = TOPAZ;
-    else if (strcmp(type, "7816") == 0)     protocol = ISO_7816_4;
-    else if (strcmp(type, "des") == 0)      protocol = MFDES;
-    else if (strcmp(type, "legic") == 0)    protocol = LEGIC;
     else if (strcmp(type, "15") == 0)       protocol = ISO_15693;
+    else if (strcmp(type, "7816") == 0)     protocol = ISO_7816_4;
+    else if (strcmp(type, "cryptorf") == 0) protocol = PROTO_CRYPTORF;
+    else if (strcmp(type, "des") == 0)      protocol = MFDES;
     else if (strcmp(type, "felica") == 0)   protocol = FELICA;
-    else if (strcmp(type, "mf") == 0)       protocol = PROTO_MIFARE;
     else if (strcmp(type, "hitag1") == 0)   protocol = PROTO_HITAG1;
     else if (strcmp(type, "hitag2") == 0)   protocol = PROTO_HITAG2;
     else if (strcmp(type, "hitags") == 0)   protocol = PROTO_HITAGS;
-    else if (strcmp(type, "thinfilm") == 0) protocol = THINFILM;
+    else if (strcmp(type, "iclass") == 0)   protocol = ICLASS;
+    else if (strcmp(type, "legic") == 0)    protocol = LEGIC;
     else if (strcmp(type, "lto") == 0)      protocol = LTO;
-    else if (strcmp(type, "cryptorf") == 0) protocol = PROTO_CRYPTORF;
+    else if (strcmp(type, "mf") == 0)       protocol = PROTO_MIFARE;
     else if (strcmp(type, "raw") == 0)      protocol = -1;
+    else if (strcmp(type, "seos") == 0)     protocol = SEOS;
+    else if (strcmp(type, "thinfilm") == 0) protocol = THINFILM;
+    else if (strcmp(type, "topaz") == 0)    protocol = TOPAZ;
     else if (strcmp(type, "") == 0)         protocol = -1;
     else {
         PrintAndLogEx(FAILED, "Unknown protocol \"%s\"", type);
@@ -792,11 +1232,12 @@ int CmdTraceList(const char *Cmd) {
     if (use_buffer == false) {
         download_trace();
     } else if (gs_traceLen == 0) {
-        PrintAndLogEx(FAILED, "You requested a trace list in offline mode but there is no trace, consider using 'trace load' or removing parameter '-1'");
+        PrintAndLogEx(FAILED, "You requested a trace list in offline mode but there is no trace.");
+        PrintAndLogEx(FAILED, "Consider using " _YELLOW_("`trace load`") " or removing parameter " _YELLOW_("`-1`"));
         return PM3_EINVARG;
     }
 
-    PrintAndLogEx(SUCCESS, "Recorded activity (trace len = " _YELLOW_("%lu") " bytes)", gs_traceLen);
+    PrintAndLogEx(SUCCESS, "Recorded activity (trace len = " _YELLOW_("%u") " bytes)", gs_traceLen);
     if (gs_traceLen == 0) {
         return PM3_SUCCESS;
     }
@@ -931,6 +1372,7 @@ int CmdTraceList(const char *Cmd) {
 
 static command_t CommandTable[] = {
     {"help",    CmdHelp,          AlwaysAvailable, "This help"},
+    {"extract", CmdTraceExtract,  AlwaysAvailable, "Extract authentication challenges found in trace"},
     {"list",    CmdTraceList,     AlwaysAvailable, "List protocol data in trace buffer"},
     {"load",    CmdTraceLoad,     AlwaysAvailable, "Load trace from file"},
     {"save",    CmdTraceSave,     AlwaysAvailable, "Save trace buffer to file"},
