@@ -17,11 +17,9 @@
 //-----------------------------------------------------------------------------
 
 #include "cmdemv.h"
-
 #include <string.h>
-
-#include "comms.h" // DropField
-#include "cmdsmartcard.h" // smart_select
+#include "comms.h"          // DropField
+#include "cmdsmartcard.h"   // smart_select
 #include "cmdtrace.h"
 #include "emvjson.h"
 #include "test/cryptotest.h"
@@ -35,6 +33,7 @@
 #include "ui.h"
 #include "emv_tags.h"
 #include "fileutils.h"
+#include "protocols.h"      // ISO7816 APDU return codes
 
 static int CmdHelp(const char *Cmd);
 
@@ -87,11 +86,11 @@ static int CmdEMVSelect(const char *Cmd) {
 
     void *argtable[] = {
         arg_param_begin,
-        arg_lit0("sS",  "select",  "activate field and select card"),
-        arg_lit0("kK",  "keep",    "keep field for next command"),
-        arg_lit0("aA",  "apdu",    "show APDU requests and responses"),
+        arg_lit0("sS",  "select",  "Activate field and select card"),
+        arg_lit0("kK",  "keep",    "Keep field for next command"),
+        arg_lit0("aA",  "apdu",    "Show APDU requests and responses"),
         arg_lit0("tT",  "tlv",     "TLV decode results"),
-        arg_lit0("wW",  "wired",   "Send data via contact (iso7816) interface. Contactless interface set by default."),
+        arg_lit0("wW",  "wired",   "Send data via contact (iso7816) interface. (def: Contactless interface)"),
         arg_str1(NULL, NULL, "<hex>", "Applet AID"),
         arg_param_end
     };
@@ -138,11 +137,11 @@ static int CmdEMVSearch(const char *Cmd) {
 
     void *argtable[] = {
         arg_param_begin,
-        arg_lit0("sS",  "select",  "activate field and select card"),
-        arg_lit0("kK",  "keep",    "keep field ON for next command"),
-        arg_lit0("aA",  "apdu",    "show APDU reqests and responses"),
+        arg_lit0("sS",  "select",  "Activate field and select card"),
+        arg_lit0("kK",  "keep",    "Keep field ON for next command"),
+        arg_lit0("aA",  "apdu",    "Show APDU requests and responses"),
         arg_lit0("tT",  "tlv",     "TLV decode results of selected applets"),
-        arg_lit0("wW",  "wired",   "Send data via contact (iso7816) interface. Contactless interface set by default."),
+        arg_lit0("wW",  "wired",   "Send data via contact (iso7816) interface. (def: Contactless interface)"),
         arg_param_end
     };
     CLIExecWithReturn(ctx, Cmd, argtable, true);
@@ -151,9 +150,12 @@ static int CmdEMVSearch(const char *Cmd) {
     bool leaveSignalON = arg_get_lit(ctx, 2);
     bool APDULogging = arg_get_lit(ctx, 3);
     bool decodeTLV = arg_get_lit(ctx, 4);
+
     Iso7816CommandChannel channel = CC_CONTACTLESS;
-    if (arg_get_lit(ctx, 5))
+    if (arg_get_lit(ctx, 5)) {
         channel = CC_CONTACT;
+    }
+
     PrintChannel(channel);
     CLIParserFree(ctx);
 
@@ -170,7 +172,7 @@ static int CmdEMVSearch(const char *Cmd) {
     PrintAndLogEx(SUCCESS, "Search completed.");
 
     // print list here
-    if (!decodeTLV) {
+    if (decodeTLV == false) {
         TLVPrintAIDlistFromSelectTLV(t);
     }
 
@@ -189,13 +191,13 @@ static int CmdEMVPPSE(const char *Cmd) {
 
     void *argtable[] = {
         arg_param_begin,
-        arg_lit0("sS",  "select",  "activate field and select card"),
-        arg_lit0("kK",  "keep",    "keep field ON for next command"),
-        arg_lit0("1",   "pse",     "pse (1PAY.SYS.DDF01) mode"),
-        arg_lit0("2",   "ppse",    "ppse (2PAY.SYS.DDF01) mode (default mode)"),
-        arg_lit0("aA",  "apdu",    "show APDU reqests and responses"),
+        arg_lit0("sS",  "select",  "Activate field and select card"),
+        arg_lit0("kK",  "keep",    "Keep field ON for next command"),
+        arg_lit0("1",   "pse",     "PSE (1PAY.SYS.DDF01) mode"),
+        arg_lit0("2",   "ppse",    "PPSE (2PAY.SYS.DDF01) mode (def)"),
+        arg_lit0("aA",  "apdu",    "Show APDU requests and responses"),
         arg_lit0("tT",  "tlv",     "TLV decode results of selected applets"),
-        arg_lit0("wW",  "wired",   "Send data via contact (iso7816) interface. Contactless interface set by default."),
+        arg_lit0("wW",  "wired",   "Send data via contact (iso7816) interface. (def: Contactless interface)"),
         arg_param_end
     };
     CLIExecWithReturn(ctx, Cmd, argtable, true);
@@ -249,12 +251,12 @@ static int CmdEMVGPO(const char *Cmd) {
 
     void *argtable[] = {
         arg_param_begin,
-        arg_lit0("kK",  "keep",    "keep field ON for next command"),
-        arg_lit0("pP",  "params",  "load parameters from `emv_defparams.json` file for PDOLdata making from PDOL and parameters"),
-        arg_lit0("mM",  "make",    "make PDOLdata from PDOL (tag 9F38) and parameters (by default uses default parameters)"),
-        arg_lit0("aA",  "apdu",    "show APDU reqests and responses"),
+        arg_lit0("kK",  "keep",    "Keep field ON for next command"),
+        arg_lit0("pP",  "params",  "Load parameters from `emv_defparams.json` file for PDOLdata making from PDOL and parameters"),
+        arg_lit0("mM",  "make",    "Make PDOLdata from PDOL (tag 9F38) and parameters (def: uses default parameters)"),
+        arg_lit0("aA",  "apdu",    "Show APDU requests and responses"),
         arg_lit0("tT",  "tlv",     "TLV decode results of selected applets"),
-        arg_lit0("wW",  "wired",   "Send data via contact (iso7816) interface. Contactless interface set by default."),
+        arg_lit0("wW",  "wired",   "Send data via contact (iso7816) interface. (def: Contactless interface)"),
         arg_strx0(NULL,  NULL,     "<hex>", "PDOLdata/PDOL"),
         arg_param_end
     };
@@ -358,10 +360,10 @@ static int CmdEMVReadRecord(const char *Cmd) {
 
     void *argtable[] = {
         arg_param_begin,
-        arg_lit0("kK",  "keep",    "keep field ON for next command"),
-        arg_lit0("aA",  "apdu",    "show APDU reqests and responses"),
+        arg_lit0("kK",  "keep",    "Keep field ON for next command"),
+        arg_lit0("aA",  "apdu",    "Show APDU requests and responses"),
         arg_lit0("tT",  "tlv",     "TLV decode results of selected applets"),
-        arg_lit0("wW",  "wired",   "Send data via contact (iso7816) interface. Contactless interface set by default."),
+        arg_lit0("wW",  "wired",   "Send data via contact (iso7816) interface. (def: Contactless interface)"),
         arg_strx1(NULL,  NULL,     "<hex>", "<SFI 1 byte><SFIrecord 1 byte"),
         arg_param_end
     };
@@ -418,14 +420,14 @@ static int CmdEMVAC(const char *Cmd) {
 
     void *argtable[] = {
         arg_param_begin,
-        arg_lit0("kK",  "keep",     "keep field ON for next command"),
-        arg_lit0("cC",  "cda",      "executes CDA transaction. Needs to get SDAD in results."),
+        arg_lit0("kK",  "keep",     "Keep field ON for next command"),
+        arg_lit0("cC",  "cda",      "Executes CDA transaction. Needs to get SDAD in results."),
         arg_str0("dD",  "decision", "<aac|tc|arqc>", "Terminal decision. aac - declined, tc - approved, arqc - online authorisation requested"),
-        arg_lit0("pP",  "params",   "load parameters from `emv_defparams.json` file for CDOLdata making from CDOL and parameters"),
-        arg_lit0("mM",  "make",     "make CDOLdata from CDOL (tag 8C and 8D) and parameters (by default uses default parameters)"),
-        arg_lit0("aA",  "apdu",     "show APDU reqests and responses"),
+        arg_lit0("pP",  "params",   "Load parameters from `emv_defparams.json` file for CDOLdata making from CDOL and parameters"),
+        arg_lit0("mM",  "make",     "Make CDOLdata from CDOL (tag 8C and 8D) and parameters (def: use default parameters)"),
+        arg_lit0("aA",  "apdu",     "Show APDU requests and responses"),
         arg_lit0("tT",  "tlv",      "TLV decode results of selected applets"),
-        arg_lit0("wW",  "wired",   "Send data via contact (iso7816) interface. Contactless interface set by default."),
+        arg_lit0("wW",  "wired",    "Send data via contact (iso7816) interface. (def: Contactless interface)"),
         arg_strx1(NULL,  NULL,      "<hex>", "CDOLdata/CDOL"),
         arg_param_end
     };
@@ -540,9 +542,9 @@ static int CmdEMVGenerateChallenge(const char *Cmd) {
 
     void *argtable[] = {
         arg_param_begin,
-        arg_lit0("kK",  "keep",    "keep field ON for next command"),
-        arg_lit0("aA",  "apdu",    "show APDU reqests and responses"),
-        arg_lit0("wW",  "wired",   "Send data via contact (iso7816) interface. Contactless interface set by default."),
+        arg_lit0("kK",  "keep",    "Keep field ON for next command"),
+        arg_lit0("aA",  "apdu",    "Show APDU requests and responses"),
+        arg_lit0("wW",  "wired",   "Send data via contact (iso7816) interface. (def: Contactless interface)"),
         arg_param_end
     };
     CLIExecWithReturn(ctx, Cmd, argtable, true);
@@ -592,12 +594,12 @@ static int CmdEMVInternalAuthenticate(const char *Cmd) {
 
     void *argtable[] = {
         arg_param_begin,
-        arg_lit0("kK",  "keep",    "keep field ON for next command"),
-        arg_lit0("pP",  "params",  "load parameters from `emv_defparams.json` file for DDOLdata making from DDOL and parameters"),
-        arg_lit0("mM",  "make",    "make DDOLdata from DDOL (tag 9F49) and parameters (by default uses default parameters)"),
-        arg_lit0("aA",  "apdu",    "show APDU reqests and responses"),
+        arg_lit0("kK",  "keep",    "Keep field ON for next command"),
+        arg_lit0("pP",  "params",  "Load parameters from `emv_defparams.json` file for DDOLdata making from DDOL and parameters"),
+        arg_lit0("mM",  "make",    "Make DDOLdata from DDOL (tag 9F49) and parameters (def: use default parameters)"),
+        arg_lit0("aA",  "apdu",    "Show APDU requests and responses"),
         arg_lit0("tT",  "tlv",     "TLV decode results of selected applets"),
-        arg_lit0("wW",  "wired",   "Send data via contact (iso7816) interface. Contactless interface set by default."),
+        arg_lit0("wW",  "wired",   "Send data via contact (iso7816) interface. (def: Contactless interface)"),
         arg_strx1(NULL,  NULL,     "<hex>", "DDOLdata/DDOL"),
         arg_param_end
     };
@@ -819,17 +821,17 @@ static int CmdEMVExec(const char *Cmd) {
 
     void *argtable[] = {
         arg_param_begin,
-        arg_lit0("sS",  "select",   "activate field and select card."),
-        arg_lit0("aA",  "apdu",     "show APDU reqests and responses."),
-        arg_lit0("tT",  "tlv",      "TLV decode results."),
-        arg_lit0("jJ",  "jload",    "Load transaction parameters from `emv_defparams.json` file."),
-        arg_lit0("fF",  "forceaid", "Force search AID. Search AID instead of execute PPSE."),
+        arg_lit0("sS",  "select",   "Activate field and select card"),
+        arg_lit0("aA",  "apdu",     "Show APDU requests and responses"),
+        arg_lit0("tT",  "tlv",      "TLV decode results"),
+        arg_lit0("jJ",  "jload",    "Load transaction parameters from `emv_defparams.json` file"),
+        arg_lit0("fF",  "forceaid", "Force search AID. Search AID instead of execute PPSE"),
         arg_rem("By default:",      "Transaction type - MSD"),
-        arg_lit0("vV",  "qvsdc",    "Transaction type - qVSDC or M/Chip."),
-        arg_lit0("cC",  "qvsdccda", "Transaction type - qVSDC or M/Chip plus CDA (SDAD generation)."),
-        arg_lit0("xX",  "vsdc",     "Transaction type - VSDC. For test only. Not a standard behavior."),
-        arg_lit0("gG",  "acgpo",    "VISA. generate AC from GPO."),
-        arg_lit0("wW",  "wired",   "Send data via contact (iso7816) interface. Contactless interface set by default."),
+        arg_lit0("vV",  "qvsdc",    "Transaction type - qVSDC or M/Chip"),
+        arg_lit0("cC",  "qvsdccda", "Transaction type - qVSDC or M/Chip plus CDA (SDAD generation)"),
+        arg_lit0("xX",  "vsdc",     "Transaction type - VSDC. For test only. Not a standard behavior"),
+        arg_lit0("gG",  "acgpo",    "VISA. generate AC from GPO"),
+        arg_lit0("wW",  "wired",    "Send data via contact (iso7816) interface. (def: Contactless interface)"),
         arg_param_end
     };
     CLIExecWithReturn(ctx, Cmd, argtable, true);
@@ -841,17 +843,23 @@ static int CmdEMVExec(const char *Cmd) {
     bool forceSearch = arg_get_lit(ctx, 5);
 
     enum TransactionType TrType = TT_MSD;
+
     if (arg_get_lit(ctx, 7))
         TrType = TT_QVSDCMCHIP;
+
     if (arg_get_lit(ctx, 8))
         TrType = TT_CDA;
+
     if (arg_get_lit(ctx, 9))
         TrType = TT_VSDC;
 
     bool GenACGPO = arg_get_lit(ctx, 10);
+
     Iso7816CommandChannel channel = CC_CONTACTLESS;
-    if (arg_get_lit(ctx, 11))
+    if (arg_get_lit(ctx, 11)) {
         channel = CC_CONTACT;
+    }
+
     PrintChannel(channel);
     uint8_t psenum = (channel == CC_CONTACT) ? 1 : 2;
     CLIParserFree(ctx);
@@ -1439,17 +1447,17 @@ static int CmdEMVScan(const char *Cmd) {
 
     void *argtable[] = {
         arg_param_begin,
-        arg_lit0("aA",  "apdu",     "show APDU reqests and responses."),
-        arg_lit0("tT",  "tlv",      "TLV decode results."),
+        arg_lit0("aA",  "apdu",     "Show APDU requests and responses"),
+        arg_lit0("tT",  "tlv",      "TLV decode results"),
         arg_lit0("eE",  "extract",  "Extract TLV elements and fill Application Data"),
-        arg_lit0("jJ",  "jload",    "Load transaction parameters from `emv_defparams.json` file."),
+        arg_lit0("jJ",  "jload",    "Load transaction parameters from `emv_defparams.json` file"),
         arg_rem("By default:",      "Transaction type - MSD"),
-        arg_lit0("vV",  "qvsdc",    "Transaction type - qVSDC or M/Chip."),
-        arg_lit0("cC",  "qvsdccda", "Transaction type - qVSDC or M/Chip plus CDA (SDAD generation)."),
-        arg_lit0("xX",  "vsdc",     "Transaction type - VSDC. For test only. Not a standard behavior."),
-        arg_lit0("gG",  "acgpo",    "VISA. generate AC from GPO."),
+        arg_lit0("vV",  "qvsdc",    "Transaction type - qVSDC or M/Chip"),
+        arg_lit0("cC",  "qvsdccda", "Transaction type - qVSDC or M/Chip plus CDA (SDAD generation)"),
+        arg_lit0("xX",  "vsdc",     "Transaction type - VSDC. For test only. Not a standard behavior"),
+        arg_lit0("gG",  "acgpo",    "VISA. generate AC from GPO"),
         arg_lit0("mM",  "merge",    "Merge output file with card's data. (warning: the file may be corrupted!)"),
-        arg_lit0("wW",  "wired",    "Send data via contact (iso7816) interface. Contactless interface set by default."),
+        arg_lit0("wW",  "wired",    "Send data via contact (iso7816) interface. (def: Contactless interface)"),
         arg_str1(NULL,  NULL,       "<fn>", "JSON output filename"),
         arg_param_end
     };
@@ -1555,7 +1563,7 @@ static int CmdEMVScan(const char *Cmd) {
     PrintAndLogEx(INFO, "PPSE");
     res = EMVSelectPSE(channel, true, true, 2, buf, sizeof(buf), &len, &sw);
 
-    if (!res && sw == 0x9000) {
+    if (!res && sw == ISO7816_OK) {
         if (decodeTLV)
             TLVPrintFromBuffer(buf, len);
 
@@ -1830,8 +1838,8 @@ static int CmdEMVTest(const char *Cmd) {
 
     void *argtable[] = {
         arg_param_begin,
-        arg_lit0("i", "ignore", "ignore timing tests for VM"),
-        arg_lit0("l", "long", "run long tests too"),
+        arg_lit0("i", "ignore", "Ignore timing tests for VM"),
+        arg_lit0("l", "long",   "Run long tests too"),
         arg_param_end
     };
     CLIExecWithReturn(ctx, Cmd, argtable, true);
@@ -1862,9 +1870,9 @@ static int CmdEMVRoca(const char *Cmd) {
 
     void *argtable[] = {
         arg_param_begin,
-        arg_lit0("tT",  "selftest",   "self test"),
-        arg_lit0("aA",  "apdu",    "show APDU reqests and responses"),
-        arg_lit0("wW",  "wired",   "Send data via contact (iso7816) interface. Contactless interface set by default"),
+        arg_lit0("tT",  "selftest", "Self test"),
+        arg_lit0("aA",  "apdu",     "Show APDU requests and responses"),
+        arg_lit0("wW",  "wired",    "Send data via contact (iso7816) interface. (def: Contactless interface)"),
         arg_param_end
     };
     CLIExecWithReturn(ctx, Cmd, argtable, true);
@@ -2115,17 +2123,17 @@ out:
 
 static command_t CommandTable[] =  {
     {"help",        CmdHelp,                        AlwaysAvailable, "This help"},
-    {"exec",        CmdEMVExec,                     IfPm3Iso14443,   "Executes EMV contactless transaction."},
-    {"pse",         CmdEMVPPSE,                     IfPm3Iso14443,   "Execute PPSE. It selects 2PAY.SYS.DDF01 or 1PAY.SYS.DDF01 directory."},
-    {"search",      CmdEMVSearch,                   IfPm3Iso14443,   "Try to select all applets from applets list and print installed applets."},
-    {"select",      CmdEMVSelect,                   IfPm3Iso14443,   "Select applet."},
-    {"gpo",         CmdEMVGPO,                      IfPm3Iso14443,   "Execute GetProcessingOptions."},
-    {"readrec",     CmdEMVReadRecord,               IfPm3Iso14443,   "Read files from card."},
-    {"genac",       CmdEMVAC,                       IfPm3Iso14443,   "Generate ApplicationCryptogram."},
-    {"challenge",   CmdEMVGenerateChallenge,        IfPm3Iso14443,   "Generate challenge."},
-    {"intauth",     CmdEMVInternalAuthenticate,     IfPm3Iso14443,   "Internal authentication."},
-    {"scan",        CmdEMVScan,                     IfPm3Iso14443,   "Scan EMV card and save it contents to json file for emulator."},
-    {"test",        CmdEMVTest,                     AlwaysAvailable, "Crypto logic test."},
+    {"exec",        CmdEMVExec,                     IfPm3Iso14443,   "Executes EMV contactless transaction"},
+    {"pse",         CmdEMVPPSE,                     IfPm3Iso14443,   "Execute PPSE. It selects 2PAY.SYS.DDF01 or 1PAY.SYS.DDF01 directory"},
+    {"search",      CmdEMVSearch,                   IfPm3Iso14443,   "Try to select all applets from applets list and print installed applets"},
+    {"select",      CmdEMVSelect,                   IfPm3Iso14443,   "Select applet"},
+    {"gpo",         CmdEMVGPO,                      IfPm3Iso14443,   "Execute GetProcessingOptions"},
+    {"readrec",     CmdEMVReadRecord,               IfPm3Iso14443,   "Read files from card"},
+    {"genac",       CmdEMVAC,                       IfPm3Iso14443,   "Generate ApplicationCryptogram"},
+    {"challenge",   CmdEMVGenerateChallenge,        IfPm3Iso14443,   "Generate challenge"},
+    {"intauth",     CmdEMVInternalAuthenticate,     IfPm3Iso14443,   "Internal authentication"},
+    {"scan",        CmdEMVScan,                     IfPm3Iso14443,   "Scan EMV card and save it contents to json file for emulator"},
+    {"test",        CmdEMVTest,                     AlwaysAvailable, "Crypto logic test"},
     /*
     {"getrng",      CmdEMVGetrng,                   IfPm3Iso14443,   "get random number from terminal"},
     {"eload",       CmdEmvELoad,                    IfPm3Iso14443,   "load EMV tag into device"},
@@ -2133,7 +2141,7 @@ static command_t CommandTable[] =  {
     {"sim",         CmdEmvSim,                      IfPm3Iso14443,   "simulate EMV tag"},
     {"clone",       CmdEmvClone,                    IfPm3Iso14443,   "clone an EMV tag"},
     */
-    {"list",        CmdEMVList,                     AlwaysAvailable,   "List ISO7816 history"},
+    {"list",        CmdEMVList,                     AlwaysAvailable, "List ISO7816 history"},
     {"roca",        CmdEMVRoca,                     IfPm3Iso14443,   "Extract public keys and run ROCA test"},
     {NULL, NULL, NULL, NULL}
 };
