@@ -467,6 +467,7 @@ static void emv_tag_dump_bitmask(const struct tlv *tlv, const struct emv_tag *ta
         unsigned char val = tlv->value[byte - 1];
         PrintAndLogEx(INFO, "%*s" NOLF, (level * 4), " ");
         PrintAndLogEx(NORMAL, "    Byte %u (%02x)", byte, val);
+
         for (bit = 8; bit > 0; bit--, val <<= 1) {
             if (val & 0x80) {
                 PrintAndLogEx(INFO, "%*s" NOLF, (level * 4), " ");
@@ -490,8 +491,7 @@ static void emv_tag_dump_dol(const struct tlv *tlv, const struct emv_tag *tag, i
         const struct emv_tag *doltag;
 
         if (!tlv_parse_tl(&buf, &left, &doltlv)) {
-            PrintAndLogEx(INFO, "%*s" NOLF, (level * 4), " ");
-            PrintAndLogEx(NORMAL, "Invalid Tag-Len");
+            PrintAndLogEx(INFO, "%*sInvalid Tag-Len", (level * 4), " ");
             continue;
         }
 
@@ -503,7 +503,7 @@ static void emv_tag_dump_dol(const struct tlv *tlv, const struct emv_tag *tag, i
 }
 
 static void emv_tag_dump_string(const struct tlv *tlv, const struct emv_tag *tag, int level) {
-    PrintAndLogEx(NORMAL, "    String value '%s'", sprint_hex_inrow(tlv->value, tlv->len));
+    PrintAndLogEx(NORMAL, "    String value '" _YELLOW_("%s")"'", sprint_hex_inrow(tlv->value, tlv->len));
 }
 
 static unsigned long emv_value_numeric(const struct tlv *tlv, unsigned start, unsigned end) {
@@ -538,12 +538,12 @@ static unsigned long emv_value_numeric(const struct tlv *tlv, unsigned start, un
 
 static void emv_tag_dump_numeric(const struct tlv *tlv, const struct emv_tag *tag, int level) {
     PrintAndLogEx(INFO, "%*s" NOLF, (level * 4), " ");
-    PrintAndLogEx(NORMAL, "    Numeric value %lu", emv_value_numeric(tlv, 0, tlv->len * 2));
+    PrintAndLogEx(NORMAL, "    Numeric value " _YELLOW_("%lu"), emv_value_numeric(tlv, 0, tlv->len * 2));
 }
 
 static void emv_tag_dump_yymmdd(const struct tlv *tlv, const struct emv_tag *tag, int level) {
     PrintAndLogEx(INFO, "%*s" NOLF, (level * 4), " ");
-    PrintAndLogEx(NORMAL, "    Date: 20%02lu.%lu.%lu",
+    PrintAndLogEx(NORMAL, "    Date: " _YELLOW_("20%02lu.%lu.%lu"),
                   emv_value_numeric(tlv, 0, 2),
                   emv_value_numeric(tlv, 2, 4),
                   emv_value_numeric(tlv, 4, 6)
@@ -557,14 +557,12 @@ static uint32_t emv_get_binary(const unsigned char *S) {
 // https://github.com/binaryfoo/emv-bertlv/blob/master/src/main/resources/fields/visa-cvr.txt
 static void emv_tag_dump_cvr(const struct tlv *tlv, const struct emv_tag *tag, int level) {
     if (tlv == NULL || tlv->len < 1) {
-        PrintAndLogEx(INFO, "%*s" NOLF, (level * 4), " ");
-        PrintAndLogEx(NORMAL, "    INVALID length!");
+        PrintAndLogEx(INFO, "%*s    INVALID length!", (level * 4), " ");
         return;
     }
 
     if (tlv->len != 5 && tlv->len != tlv->value[0] + 1) {
-        PrintAndLogEx(INFO, "%*s" NOLF, (level * 4), " ");
-        PrintAndLogEx(NORMAL, "    INVALID length!");
+        PrintAndLogEx(INFO, "%*s    INVALID length!", (level * 4), " ");
         return;
     }
 
@@ -633,8 +631,7 @@ static void emv_tag_dump_cvr(const struct tlv *tlv, const struct emv_tag *tag, i
 // EMV Book 3
 static void emv_tag_dump_cid(const struct tlv *tlv, const struct emv_tag *tag, int level) {
     if (tlv == NULL || tlv->len < 1) {
-        PrintAndLogEx(INFO, "%*s" NOLF, (level * 4), " ");
-        PrintAndLogEx(NORMAL, "    INVALID!");
+        PrintAndLogEx(INFO, "%*s    INVALID!", (level * 4), " ");
         return;
     }
 
@@ -682,8 +679,7 @@ static void emv_tag_dump_cvm_list(const struct tlv *tlv, const struct emv_tag *t
     int i;
 
     if (tlv->len < 10 || tlv->len % 2) {
-        PrintAndLogEx(INFO, "%*s" NOLF, (level * 4), " ");
-        PrintAndLogEx(NORMAL, "    INVALID!");
+        PrintAndLogEx(INFO, "%*s    INVALID!", (level * 4), " ");
         return;
     }
 
@@ -781,8 +777,7 @@ static void emv_tag_dump_cvm_list(const struct tlv *tlv, const struct emv_tag *t
 
 static void emv_tag_dump_afl(const struct tlv *tlv, const struct emv_tag *tag, int level) {
     if (tlv->len < 4 || tlv->len % 4) {
-        PrintAndLogEx(INFO, "%*s" NOLF, (level * 4), " ");
-        PrintAndLogEx(NORMAL, "    INVALID!");
+        PrintAndLogEx(INFO, "%*s    INVALID!", (level * 4), " ");
         return;
     }
 
