@@ -23,19 +23,20 @@
 #endif
 
 #include "tlv.h"
-
 #include <string.h>
 #include <stdlib.h>
 #include <stdbool.h>
+#include <stdint.h>
 
-#define TLV_TAG_CLASS_MASK  0xc0
+
+#define TLV_TAG_CLASS_MASK  0xC0
 #define TLV_TAG_COMPLEX     0x20
-#define TLV_TAG_VALUE_MASK  0x1f
-#define TLV_TAG_VALUE_CONT  0x1f
+#define TLV_TAG_VALUE_MASK  0x1F
+#define TLV_TAG_VALUE_CONT  0x1F
 #define TLV_TAG_INVALID     0
 
 #define TLV_LEN_LONG        0x80
-#define TLV_LEN_MASK        0x7f
+#define TLV_LEN_MASK        0x7F
 #define TLV_LEN_INVALID     (~0)
 
 // http://radek.io/2012/11/10/magical-container_of-macro/
@@ -43,19 +44,6 @@
 //  const typeof( ((type *)0)->member ) *__mptr = (ptr);
 //        (type *)( (char *)__mptr - offsetof(type,member) );})
 
-struct tlvdb {
-    struct tlv tag;
-    struct tlvdb *next;
-    struct tlvdb *parent;
-    struct tlvdb *children;
-};
-
-struct tlvdb_root {
-    struct tlvdb db;
-    size_t len;
-    unsigned char buf[0];
-};
-
 static tlv_tag_t tlv_parse_tag(const unsigned char **buf, size_t *len) {
     tlv_tag_t tag;
 
@@ -211,6 +199,7 @@ struct tlvdb *tlvdb_parse(const unsigned char *buf, size_t len) {
 err:
     tlvdb_free(&root->db);
 
+    free(root);
     return NULL;
 }
 
@@ -247,9 +236,53 @@ struct tlvdb *tlvdb_parse_multi(const unsigned char *buf, size_t len) {
 err:
     tlvdb_free(&root->db);
 
+    free(root);
     return NULL;
 }
 
+bool tlvdb_parse_root(struct tlvdb_root *root) {
+    if (root == NULL || root->len == 0) {
+        return false;
+    }
+
+    const uint8_t *tmp;
+    size_t left;
+
+    tmp = root->buf;
+    left = root->len;
+    if (tlvdb_parse_one(&root->db, NULL, &tmp, &left) == true) {
+        if (left == 0) {
+            return true;
+        }
+    }
+    return false;
+}
+
+bool tlvdb_parse_root_multi(struct tlvdb_root *root) {
+    if (root == NULL || root->len == 0) {
+        return false;
+    }
+
+    const uint8_t *tmp;
+    size_t left;
+
+    tmp = root->buf;
+    left = root->len;
+    if (tlvdb_parse_one(&root->db, NULL, &tmp, &left) == true) {
+        while (left > 0) {
+            struct tlvdb *db = calloc(1, sizeof(*db));
+            if (tlvdb_parse_one(db, NULL, &tmp, &left) == true) {
+                tlvdb_add(&root->db, db);
+            } else {
+                free(db);
+                return false;
+            }
+        }
+        return true;
+    }
+    return false;
+}
+
 struct tlvdb *tlvdb_fixed(tlv_tag_t tag, size_t len, const unsigned char *value) {
     struct tlvdb_root *root = calloc(1, sizeof(*root) + len);
 
@@ -290,6 +323,21 @@ void tlvdb_free(struct tlvdb *tlvdb) {
     }
 }
 
+void tlvdb_root_free(struct tlvdb_root *root) {
+    if (root == NULL) {
+        return;
+    }
+    if (root->db.children) {
+        tlvdb_free(root->db.children);
+        root->db.children = NULL;
+    }
+    if (root->db.next) {
+        tlvdb_free(root->db.next);
+        root->db.next = NULL;
+    }
+    free(root);
+}
+
 struct tlvdb *tlvdb_find_next(struct tlvdb *tlvdb, tlv_tag_t tag) {
     if (!tlvdb)
         return NULL;
