@@ -28,6 +28,7 @@
 #include "util.h"
 #include "cmdhficlass.h"  // pagemap
 #include "protocols.h"    // iclass defines
+#include "cmdhftopaz.h"   // TOPAZ defines
 
 #ifdef _WIN32
 #include "scandir.h"
@@ -213,18 +214,23 @@ bool create_path(const char *dirname) {
 }
 */
 
-bool setDefaultPath(savePaths_t pathIndex, const char *Path) {
+bool setDefaultPath(savePaths_t pathIndex, const char *path) {
 
     if (pathIndex < spItemCount) {
-        if ((Path == NULL) && (g_session.defaultPaths[pathIndex] != NULL)) {
+
+        if ((path == NULL) && (g_session.defaultPaths[pathIndex] != NULL)) {
             free(g_session.defaultPaths[pathIndex]);
             g_session.defaultPaths[pathIndex] = NULL;
         }
 
-        if (Path != NULL) {
-            g_session.defaultPaths[pathIndex] = (char *)realloc(g_session.defaultPaths[pathIndex], strlen(Path) + 1);
-            strcpy(g_session.defaultPaths[pathIndex], Path);
+        if (path == NULL) {
+            return false;
         }
+
+        size_t len = strlen(path);
+
+        g_session.defaultPaths[pathIndex] = (char *)realloc(g_session.defaultPaths[pathIndex], len + 1);
+        strcpy(g_session.defaultPaths[pathIndex], path);
         return true;
     }
     return false;
@@ -233,43 +239,70 @@ bool setDefaultPath(savePaths_t pathIndex, const char *Path) {
 static char *filenamemcopy(const char *preferredName, const char *suffix) {
     if (preferredName == NULL) return NULL;
     if (suffix == NULL) return NULL;
+
     char *fileName = (char *) calloc(strlen(preferredName) + strlen(suffix) + 1, sizeof(uint8_t));
-    if (fileName == NULL)
+    if (fileName == NULL) {
         return NULL;
+    }
+
     strcpy(fileName, preferredName);
-    if (str_endswith(fileName, suffix))
+    if (str_endswith(fileName, suffix)) {
         return fileName;
+    }
+
     strcat(fileName, suffix);
     return fileName;
 }
 
+static size_t path_size(savePaths_t a) {
+    if (a == spItemCount) {
+        return 0;
+    }
+    return strlen(g_session.defaultPaths[a]);
+}
+
 char *newfilenamemcopy(const char *preferredName, const char *suffix) {
-    if (preferredName == NULL) return NULL;
-    if (suffix == NULL) return NULL;
+    if (preferredName == NULL || suffix == NULL) {
+        return NULL;
+    }
 
     uint16_t p_namelen = strlen(preferredName);
     if (str_endswith(preferredName, suffix))
         p_namelen -= strlen(suffix);
 
-    char *fileName = (char *) calloc(p_namelen + strlen(suffix) + 1 + 10, sizeof(uint8_t)); // 10: room for filenum to ensure new filename
+    // 10: room for filenum to ensure new filename
+    const size_t len = p_namelen + strlen(suffix) + 1 + 10;
+
+    int foobar = path_size(spDefault);
+    (void) foobar;
+
+    char *fileName = (char *) calloc(len, sizeof(uint8_t));
     if (fileName == NULL) {
         return NULL;
     }
 
     int num = 1;
-    sprintf(fileName, "%.*s%s", p_namelen, preferredName, suffix);
+
+    snprintf(fileName, len, "%.*s%s", p_namelen, preferredName, suffix);
     while (fileExists(fileName)) {
-        sprintf(fileName, "%.*s-%d%s", p_namelen, preferredName, num, suffix);
+        snprintf(fileName, len, "%.*s-%d%s", p_namelen, preferredName, num, suffix);
         num++;
     }
+
+    PrintAndLogEx(INFO, "FILE PATH:  %s", fileName);
     return fileName;
 }
 
 int saveFile(const char *preferredName, const char *suffix, const void *data, size_t datalen) {
 
-    if (data == NULL) return PM3_EINVARG;
+    if (data == NULL || datalen == 0) {
+        return PM3_EINVARG;
+    }
+
     char *fileName = newfilenamemcopy(preferredName, suffix);
-    if (fileName == NULL) return PM3_EMALLOC;
+    if (fileName == NULL) {
+        return PM3_EMALLOC;
+    }
 
     /* We should have a valid filename now, e.g. dumpdata-3.bin */
 
@@ -290,9 +323,14 @@ int saveFile(const char *preferredName, const char *suffix, const void *data, si
 
 int saveFileEML(const char *preferredName, uint8_t *data, size_t datalen, size_t blocksize) {
 
-    if (data == NULL) return PM3_EINVARG;
+    if (data == NULL || datalen == 0) {
+        return PM3_EINVARG;
+    }
+
     char *fileName = newfilenamemcopy(preferredName, ".eml");
-    if (fileName == NULL) return PM3_EMALLOC;
+    if (fileName == NULL) {
+        return PM3_EMALLOC;
+    }
 
     int retval = PM3_SUCCESS;
     int blocks = datalen / blocksize;
@@ -338,10 +376,14 @@ int saveFileJSON(const char *preferredName, JSONFileType ftype, uint8_t *data, s
 }
 int saveFileJSONex(const char *preferredName, JSONFileType ftype, uint8_t *data, size_t datalen, bool verbose, void (*callback)(json_t *)) {
 
-    if (data == NULL) return PM3_EINVARG;
+    if (data == NULL || datalen == 0) {
+        return PM3_EINVARG;
+    }
 
     char *fileName = newfilenamemcopy(preferredName, ".json");
-    if (fileName == NULL) return PM3_EMALLOC;
+    if (fileName == NULL) {
+        return PM3_EMALLOC;
+    }
 
     int retval = PM3_SUCCESS;
 
@@ -361,7 +403,7 @@ int saveFileJSONex(const char *preferredName, JSONFileType ftype, uint8_t *data,
             JsonSaveBufAsHexCompact(root, "$.Card.SAK", &(xdump->card_info.sak), 1);
             for (size_t i = 0; i < (xdump->dumplen / 16); i++) {
                 char path[PATH_MAX_LENGTH] = {0};
-                sprintf(path, "$.blocks.%zu", i);
+                snprintf(path, sizeof(path), "$.blocks.%zu", i);
                 JsonSaveBufAsHexCompact(root, path, &xdump->dump[i * 16], 16);
                 if (mfIsSectorTrailer(i)) {
                     snprintf(path, sizeof(path), "$.SectorKeys.%d.KeyA", mfSectorNum(i));
@@ -392,6 +434,19 @@ int saveFileJSONex(const char *preferredName, JSONFileType ftype, uint8_t *data,
             }
             break;
         }
+        case jsfFudan: {
+            iso14a_mf_extdump_t *xdump = (iso14a_mf_extdump_t *)(void *) data;
+            JsonSaveStr(root, "FileType", "fudan");
+            JsonSaveBufAsHexCompact(root, "$.Card.UID", xdump->card_info.uid, xdump->card_info.uidlen);
+            JsonSaveBufAsHexCompact(root, "$.Card.ATQA", xdump->card_info.atqa, 2);
+            JsonSaveBufAsHexCompact(root, "$.Card.SAK", &(xdump->card_info.sak), 1);
+            for (size_t i = 0; i < (xdump->dumplen / 4); i++) {
+                char path[PATH_MAX_LENGTH] = {0};
+                snprintf(path, sizeof(path), "$.blocks.%zu", i);
+                JsonSaveBufAsHexCompact(root, path, &xdump->dump[i * 4], 4);
+            }
+            break;
+        }
         case jsfMfuMemory: {
             JsonSaveStr(root, "FileType", "mfu");
 
@@ -409,9 +464,9 @@ int saveFileJSONex(const char *preferredName, JSONFileType ftype, uint8_t *data,
             JsonSaveBufAsHexCompact(root, "$.Card.TBO_1", tmp->tbo1, sizeof(tmp->tbo1));
             JsonSaveBufAsHexCompact(root, "$.Card.Signature", tmp->signature, sizeof(tmp->signature));
             for (uint8_t i = 0; i < 3; i ++) {
-                sprintf(path, "$.Card.Counter%d", i);
+                snprintf(path, sizeof(path), "$.Card.Counter%d", i);
                 JsonSaveBufAsHexCompact(root, path, tmp->counter_tearing[i], 3);
-                sprintf(path, "$.Card.Tearing%d", i);
+                snprintf(path, sizeof(path), "$.Card.Tearing%d", i);
                 JsonSaveBufAsHexCompact(root, path, tmp->counter_tearing[i] + 3, 1);
             }
 
@@ -419,7 +474,7 @@ int saveFileJSONex(const char *preferredName, JSONFileType ftype, uint8_t *data,
             size_t len = (datalen - MFU_DUMP_PREFIX_LENGTH) / 4;
 
             for (size_t i = 0; i < len; i++) {
-                sprintf(path, "$.blocks.%zu", i);
+                snprintf(path, sizeof(path), "$.blocks.%zu", i);
                 JsonSaveBufAsHexCompact(root, path, tmp->data + (i * 4), 4);
             }
             break;
@@ -433,7 +488,7 @@ int saveFileJSONex(const char *preferredName, JSONFileType ftype, uint8_t *data,
 
             for (size_t i = 0; i < (datalen / 4); i++) {
                 char path[PATH_MAX_LENGTH] = {0};
-                sprintf(path, "$.blocks.%zu", i);
+                snprintf(path, sizeof(path), "$.blocks.%zu", i);
                 JsonSaveBufAsHexCompact(root, path, data + (i * 4), 4);
             }
             break;
@@ -458,7 +513,7 @@ int saveFileJSONex(const char *preferredName, JSONFileType ftype, uint8_t *data,
 
             for (size_t i = 0; i < (datalen / 8); i++) {
                 char path[PATH_MAX_LENGTH] = {0};
-                sprintf(path, "$.blocks.%zu", i);
+                snprintf(path, sizeof(path), "$.blocks.%zu", i);
                 JsonSaveBufAsHexCompact(root, path, data + (i * 8), 8);
             }
 
@@ -472,7 +527,7 @@ int saveFileJSONex(const char *preferredName, JSONFileType ftype, uint8_t *data,
 
             for (size_t i = 0; i < (datalen / 4); i++) {
                 char path[PATH_MAX_LENGTH] = {0};
-                sprintf(path, "$.blocks.%zu", i);
+                snprintf(path, sizeof(path), "$.blocks.%zu", i);
                 JsonSaveBufAsHexCompact(root, path, data + (i * 4), 4);
             }
             break;
@@ -500,7 +555,7 @@ int saveFileJSONex(const char *preferredName, JSONFileType ftype, uint8_t *data,
 
             for (size_t i = 0; i < (datalen / 4); i++) {
                 char path[PATH_MAX_LENGTH] = {0};
-                sprintf(path, "$.blocks.%zu", i);
+                snprintf(path, sizeof(path), "$.blocks.%zu", i);
                 JsonSaveBufAsHexCompact(root, path, data + (i * 4), 4);
             }
             break;
@@ -514,7 +569,7 @@ int saveFileJSONex(const char *preferredName, JSONFileType ftype, uint8_t *data,
 
             for (size_t i = 0; i < (datalen / 4); i++) {
                 char path[PATH_MAX_LENGTH] = {0};
-                sprintf(path, "$.blocks.%zu", i);
+                snprintf(path, sizeof(path), "$.blocks.%zu", i);
                 JsonSaveBufAsHexCompact(root, path, data + (i * 4), 4);
             }
             break;
@@ -527,7 +582,7 @@ int saveFileJSONex(const char *preferredName, JSONFileType ftype, uint8_t *data,
 
             for (size_t i = 0; i < (datalen / 4); i++) {
                 char path[PATH_MAX_LENGTH] = {0};
-                sprintf(path, "$.blocks.%zu", i);
+                snprintf(path, sizeof(path), "$.blocks.%zu", i);
                 JsonSaveBufAsHexCompact(root, path, data + (i * 4), 4);
             }
             break;
@@ -541,7 +596,7 @@ int saveFileJSONex(const char *preferredName, JSONFileType ftype, uint8_t *data,
 
             for (size_t i = 0; i < (datalen / 4); i++) {
                 char path[PATH_MAX_LENGTH] = {0};
-                sprintf(path, "$.blocks.%zu", i);
+                snprintf(path, sizeof(path), "$.blocks.%zu", i);
                 JsonSaveBufAsHexCompact(root, path, data + (i * 4), 4);
             }
             break;
@@ -562,14 +617,12 @@ int saveFileJSONex(const char *preferredName, JSONFileType ftype, uint8_t *data,
                 char path[PATH_MAX_LENGTH] = {0};
 
                 if (vdata[0][i][0]) {
-                    memset(path, 0x00, sizeof(path));
-                    sprintf(path, "$.SectorKeys.%d.KeyA", mfSectorNum(i));
+                    snprintf(path, sizeof(path), "$.SectorKeys.%d.KeyA", mfSectorNum(i));
                     JsonSaveBufAsHexCompact(root, path, &vdata[0][i][1], 16);
                 }
 
                 if (vdata[1][i][0]) {
-                    memset(path, 0x00, sizeof(path));
-                    sprintf(path, "$.SectorKeys.%d.KeyB", mfSectorNum(i));
+                    snprintf(path, sizeof(path), "$.SectorKeys.%d.KeyB", mfSectorNum(i));
                     JsonSaveBufAsHexCompact(root, path, &vdata[1][i][1], 16);
                 }
             }
@@ -591,24 +644,20 @@ int saveFileJSONex(const char *preferredName, JSONFileType ftype, uint8_t *data,
                 char path[PATH_MAX_LENGTH] = {0};
 
                 if (dvdata[0][i][0]) {
-                    memset(path, 0x00, sizeof(path));
-                    sprintf(path, "$.DES.%d.Key", i);
+                    snprintf(path, sizeof(path), "$.DES.%d.Key", i);
                     JsonSaveBufAsHexCompact(root, path, &dvdata[0][i][1], 8);
                 }
 
                 if (dvdata[1][i][0]) {
-                    memset(path, 0x00, sizeof(path));
-                    sprintf(path, "$.3DES.%d.Key", i);
+                    snprintf(path, sizeof(path), "$.3DES.%d.Key", i);
                     JsonSaveBufAsHexCompact(root, path, &dvdata[1][i][1], 16);
                 }
                 if (dvdata[2][i][0]) {
-                    memset(path, 0x00, sizeof(path));
-                    sprintf(path, "$.AES.%d.Key", i);
+                    snprintf(path, sizeof(path), "$.AES.%d.Key", i);
                     JsonSaveBufAsHexCompact(root, path, &dvdata[2][i][1], 16);
                 }
                 if (dvdata[3][i][0]) {
-                    memset(path, 0x00, sizeof(path));
-                    sprintf(path, "$.K3KDES.%d.Key", i);
+                    snprintf(path, sizeof(path), "$.K3KDES.%d.Key", i);
                     JsonSaveBufAsHexCompact(root, path, &dvdata[3][i][1], 24);
                 }
             }
@@ -621,6 +670,24 @@ int saveFileJSONex(const char *preferredName, JSONFileType ftype, uint8_t *data,
             (*callback)(root);
             break;
         }
+        case jsfTopaz: {
+            topaz_tag_t *tag = (topaz_tag_t *)(void *) data;
+            JsonSaveStr(root, "FileType", "topaz");
+            JsonSaveBufAsHexCompact(root, "$.Card.UID", tag->uid, sizeof(tag->uid));
+            JsonSaveBufAsHexCompact(root, "$.Card.H0R1", tag->HR01, sizeof(tag->HR01));
+            JsonSaveBufAsHexCompact(root, "$.Card.Size", (uint8_t *) & (tag->size), 2);
+            for (size_t i = 0; i < TOPAZ_STATIC_MEMORY / 8; i++) {
+                char path[PATH_MAX_LENGTH] = {0};
+                snprintf(path, sizeof(path), "$.blocks.%zu", i);
+                JsonSaveBufAsHexCompact(root, path, &tag->data_blocks[i][0], TOPAZ_BLOCK_SIZE);
+            }
+
+            // ICEMAN todo:  add dynamic memory.
+            // uint16_z Size
+            // uint8_t *dynamic_memory;
+
+            break;
+        }
         default:
             break;
     }
@@ -674,9 +741,15 @@ int saveFileJSONrootEx(const char *preferredName, void *root, size_t flags, bool
 
 int saveFileWAVE(const char *preferredName, const int *data, size_t datalen) {
 
-    if (data == NULL) return PM3_EINVARG;
+    if (data == NULL || datalen == 0) {
+        return PM3_EINVARG;
+    }
+
     char *fileName = newfilenamemcopy(preferredName, ".wav");
-    if (fileName == NULL) return PM3_EMALLOC;
+    if (fileName == NULL) {
+        return PM3_EMALLOC;
+    }
+
     int retval = PM3_SUCCESS;
 
     struct wave_info_t wave_info = {
@@ -701,11 +774,14 @@ int saveFileWAVE(const char *preferredName, const int *data, size_t datalen) {
         retval = PM3_EFILE;
         goto out;
     }
+
     fwrite(&wave_info, sizeof(wave_info), 1, wave_file);
+
     for (int i = 0; i < datalen; i++) {
         uint8_t sample = data[i] + 128;
         fwrite(&sample, 1, 1, wave_file);
     }
+
     fclose(wave_file);
 
     PrintAndLogEx(SUCCESS, "saved " _YELLOW_("%zu") " bytes to wave file " _YELLOW_("'%s'"), 2 * datalen, fileName);
@@ -717,9 +793,14 @@ out:
 
 int saveFilePM3(const char *preferredName, int *data, size_t datalen) {
 
-    if (data == NULL) return PM3_EINVARG;
+    if (data == NULL || datalen == 0) {
+        return PM3_EINVARG;
+    }
+
     char *fileName = newfilenamemcopy(preferredName, ".pm3");
-    if (fileName == NULL) return PM3_EMALLOC;
+    if (fileName == NULL) {
+        return PM3_EMALLOC;
+    }
 
     int retval = PM3_SUCCESS;
 
@@ -730,8 +811,9 @@ int saveFilePM3(const char *preferredName, int *data, size_t datalen) {
         goto out;
     }
 
-    for (uint32_t i = 0; i < datalen; i++)
+    for (uint32_t i = 0; i < datalen; i++) {
         fprintf(f, "%d\n", data[i]);
+    }
 
     fflush(f);
     fclose(f);
@@ -1080,27 +1162,57 @@ int loadFileJSONex(const char *preferredName, void *data, size_t maxdatalen, siz
         goto out;
     }
 
-    uint8_t *udata = (uint8_t *)data;
+    typedef union UDATA {
+        void *v;
+        uint8_t *bytes;
+        mfu_dump_t *mfu;
+        topaz_tag_t *topaz;
+    } UDATA;
+    UDATA udata = (UDATA)data;
     char ctype[100] = {0};
     JsonLoadStr(root, "$.FileType", ctype);
 
     if (!strcmp(ctype, "raw")) {
-        JsonLoadBufAsHex(root, "$.raw", udata, maxdatalen, datalen);
+        JsonLoadBufAsHex(root, "$.raw", udata.bytes, maxdatalen, datalen);
     }
 
     if (!strcmp(ctype, "mfcard")) {
         size_t sptr = 0;
         for (int i = 0; i < 256; i++) {
+            char blocks[30] = {0};
+            snprintf(blocks, sizeof(blocks), "$.blocks.%d", i);
+
+            size_t len = 0;
+            uint8_t block[16];
+            JsonLoadBufAsHex(root, blocks, block, 16, &len);
+            if (!len)
+                break;
+
             if (sptr + 16 > maxdatalen) {
                 retval = PM3_EMALLOC;
                 goto out;
             }
 
+            memcpy(&udata.bytes[sptr], block, 16);
+            sptr += len;
+        }
+
+        *datalen = sptr;
+    }
+
+    if (!strcmp(ctype, "fudan")) {
+        size_t sptr = 0;
+        for (int i = 0; i < 256; i++) {
+            if (sptr + 4 > maxdatalen) {
+                retval = PM3_EMALLOC;
+                goto out;
+            }
+
             char blocks[30] = {0};
-            sprintf(blocks, "$.blocks.%d", i);
+            snprintf(blocks, sizeof(blocks), "$.blocks.%d", i);
 
             size_t len = 0;
-            JsonLoadBufAsHex(root, blocks, &udata[sptr], 16, &len);
+            JsonLoadBufAsHex(root, blocks, &udata.bytes[sptr], 4, &len);
             if (!len)
                 break;
 
@@ -1111,6 +1223,19 @@ int loadFileJSONex(const char *preferredName, void *data, size_t maxdatalen, siz
     }
 
     if (!strcmp(ctype, "mfu")) {
+
+        JsonLoadBufAsHex(root, "$.Card.Version", udata.mfu->version, sizeof(udata.mfu->version), datalen);
+        JsonLoadBufAsHex(root, "$.Card.TBO_0", udata.mfu->tbo, sizeof(udata.mfu->tbo), datalen);
+        JsonLoadBufAsHex(root, "$.Card.TBO_1", udata.mfu->tbo1, sizeof(udata.mfu->tbo1), datalen);
+        JsonLoadBufAsHex(root, "$.Card.Signature", udata.mfu->signature, sizeof(udata.mfu->signature), datalen);
+        JsonLoadBufAsHex(root, "$.Card.Counter0", &udata.mfu->counter_tearing[0][0], 3, datalen);
+        JsonLoadBufAsHex(root, "$.Card.Tearing0", &udata.mfu->counter_tearing[0][3], 1, datalen);
+        JsonLoadBufAsHex(root, "$.Card.Counter1", &udata.mfu->counter_tearing[1][0], 3, datalen);
+        JsonLoadBufAsHex(root, "$.Card.Tearing1", &udata.mfu->counter_tearing[1][3], 1, datalen);
+        JsonLoadBufAsHex(root, "$.Card.Counter2", &udata.mfu->counter_tearing[2][0], 3, datalen);
+        JsonLoadBufAsHex(root, "$.Card.Tearing2", &udata.mfu->counter_tearing[2][3], 1, datalen);
+        *datalen = MFU_DUMP_PREFIX_LENGTH;
+
         size_t sptr = 0;
         for (int i = 0; i < 256; i++) {
             if (sptr + 4 > maxdatalen) {
@@ -1119,17 +1244,20 @@ int loadFileJSONex(const char *preferredName, void *data, size_t maxdatalen, siz
             }
 
             char blocks[30] = {0};
-            sprintf(blocks, "$.blocks.%d", i);
+            snprintf(blocks, sizeof(blocks), "$.blocks.%d", i);
 
             size_t len = 0;
-            JsonLoadBufAsHex(root, blocks, &udata[sptr], 4, &len);
+            JsonLoadBufAsHex(root, blocks, &udata.mfu->data[sptr], MFU_BLOCK_SIZE, &len);
             if (!len)
                 break;
 
             sptr += len;
+            udata.mfu->pages++;
         }
+        // remove one, since pages indicates a index rather than number of available pages
+        --udata.mfu->pages;
 
-        *datalen = sptr;
+        *datalen += sptr;
     }
 
     if (!strcmp(ctype, "hitag")) {
@@ -1141,10 +1269,10 @@ int loadFileJSONex(const char *preferredName, void *data, size_t maxdatalen, siz
             }
 
             char blocks[30] = {0};
-            sprintf(blocks, "$.blocks.%zu", i);
+            snprintf(blocks, sizeof(blocks), "$.blocks.%zu", i);
 
             size_t len = 0;
-            JsonLoadBufAsHex(root, blocks, &udata[sptr], 4, &len);
+            JsonLoadBufAsHex(root, blocks, &udata.bytes[sptr], 4, &len);
             if (!len)
                 break;
 
@@ -1163,10 +1291,10 @@ int loadFileJSONex(const char *preferredName, void *data, size_t maxdatalen, siz
             }
 
             char blocks[30] = {0};
-            sprintf(blocks, "$.blocks.%zu", i);
+            snprintf(blocks, sizeof(blocks), "$.blocks.%zu", i);
 
             size_t len = 0;
-            JsonLoadBufAsHex(root, blocks, &udata[sptr], 8, &len);
+            JsonLoadBufAsHex(root, blocks, &udata.bytes[sptr], 8, &len);
             if (!len)
                 break;
 
@@ -1184,10 +1312,10 @@ int loadFileJSONex(const char *preferredName, void *data, size_t maxdatalen, siz
             }
 
             char blocks[30] = {0};
-            sprintf(blocks, "$.blocks.%zu", i);
+            snprintf(blocks, sizeof(blocks), "$.blocks.%zu", i);
 
             size_t len = 0;
-            JsonLoadBufAsHex(root, blocks, &udata[sptr], 4, &len);
+            JsonLoadBufAsHex(root, blocks, &udata.bytes[sptr], 4, &len);
             if (!len)
                 break;
 
@@ -1205,10 +1333,10 @@ int loadFileJSONex(const char *preferredName, void *data, size_t maxdatalen, siz
             }
 
             char blocks[30] = {0};
-            sprintf(blocks, "$.blocks.%zu", i);
+            snprintf(blocks, sizeof(blocks), "$.blocks.%zu", i);
 
             size_t len = 0;
-            JsonLoadBufAsHex(root, blocks, &udata[sptr], 4, &len);
+            JsonLoadBufAsHex(root, blocks, &udata.bytes[sptr], 4, &len);
             if (!len)
                 break;
 
@@ -1218,9 +1346,44 @@ int loadFileJSONex(const char *preferredName, void *data, size_t maxdatalen, siz
     }
 
     if (!strcmp(ctype, "15693")) {
-        JsonLoadBufAsHex(root, "$.raw", udata, maxdatalen, datalen);
+        JsonLoadBufAsHex(root, "$.raw", udata.bytes, maxdatalen, datalen);
     }
 
+    if (!strcmp(ctype, "legic")) {
+        JsonLoadBufAsHex(root, "$.raw", udata.bytes, maxdatalen, datalen);
+    }
+
+    if (!strcmp(ctype, "topaz")) {
+
+        JsonLoadBufAsHex(root, "$.Card.UID", udata.topaz->uid, sizeof(udata.topaz->uid), datalen);
+        JsonLoadBufAsHex(root, "$.Card.HR01", udata.topaz->HR01, sizeof(udata.topaz->HR01), datalen);
+        JsonLoadBufAsHex(root, "$.Card.Size", (uint8_t *) & (udata.topaz->size), 2, datalen);
+
+        size_t sptr = 0;
+        for (int i = 0; i < (TOPAZ_STATIC_MEMORY / 8); i++) {
+
+            if (sptr + TOPAZ_BLOCK_SIZE > maxdatalen) {
+                retval = PM3_EMALLOC;
+                goto out;
+            }
+
+            char blocks[30] = {0};
+            snprintf(blocks, sizeof(blocks), "$.blocks.%d", i);
+
+            size_t len = 0;
+            JsonLoadBufAsHex(root, blocks, &udata.topaz->data_blocks[sptr][0], TOPAZ_BLOCK_SIZE, &len);
+            if (!len)
+                break;
+
+            sptr += len;
+
+            // ICEMAN todo:  add dynamic memory.
+            // uint16_z Size
+            // uint8_t *dynamic_memory;
+        }
+
+        *datalen += sptr;
+    }
 out:
 
     if (callback != NULL) {
@@ -1623,7 +1786,7 @@ static int filelist(const char *path, const char *ext, uint8_t last, bool tentat
 
     PrintAndLogEx(NORMAL, "%s── "_GREEN_("%s"), last ? "└" : "├", &path[strip]);
 
-    for (uint16_t i = 0; i < n; i++) {
+    for (int i = 0; i < n; i++) {
 
         char tmp_fullpath[1024] = {0};
         strncat(tmp_fullpath, path, sizeof(tmp_fullpath) - 1);
@@ -1754,14 +1917,18 @@ static int searchFinalFile(char **foundpath, const char *pm3dir, const char *sea
              (strcmp(PYTHON_SCRIPTS_SUBDIR, pm3dir) == 0) ||
              (strcmp(RESOURCES_SUBDIR, pm3dir) == 0))) {
         char *path = calloc(strlen(exec_path) + strlen(pm3dir) + strlen(filename) + 1, sizeof(char));
-        if (path == NULL)
+        if (path == NULL) {
             goto out;
+        }
+
         strcpy(path, exec_path);
         strcat(path, pm3dir);
         strcat(path, filename);
+
         if ((g_debugMode == 2) && (!silent)) {
             PrintAndLogEx(INFO, "Searching %s", path);
         }
+
         if (fileExists(path)) {
             free(filename);
             *foundpath = path;
@@ -1858,3 +2025,55 @@ int searchFile(char **foundpath, const char *pm3dir, const char *searchname, con
     free(filename);
     return PM3_SUCCESS;
 }
+
+int pm3_load_dump(const char *fn, void **pdump, size_t *dumplen, size_t maxdumplen) {
+
+    int res = 0;
+    DumpFileType_t dftype = getfiletype(fn);
+    switch (dftype) {
+        case BIN: {
+            res = loadFile_safe(fn, ".bin", pdump, dumplen);
+            break;
+        }
+        case EML: {
+            res = loadFileEML_safe(fn, pdump, dumplen);
+            break;
+        }
+        case JSON: {
+            *pdump = calloc(maxdumplen, sizeof(uint8_t));
+            if (*pdump == NULL) {
+                PrintAndLogEx(WARNING, "Fail, cannot allocate memory");
+                return PM3_EMALLOC;
+            }
+            res = loadFileJSON(fn, *pdump, maxdumplen, dumplen, NULL);
+            if (res != PM3_SUCCESS) {
+                free(*pdump);
+            }
+            break;
+        }
+        case DICTIONARY: {
+            PrintAndLogEx(ERR, "Error: Only BIN/EML/JSON formats allowed");
+            return PM3_EINVARG;
+        }
+    }
+
+    if (res != PM3_SUCCESS) {
+        PrintAndLogEx(FAILED, "File: " _YELLOW_("%s") ": not found or locked.", fn);
+        return PM3_EFILE;
+    }
+
+    return res;
+}
+
+int pm3_save_dump(const char *fn, uint8_t *d, size_t n, JSONFileType jsft, size_t blocksize) {
+
+    if (d == NULL || n == 0) {
+        PrintAndLogEx(INFO, "No data to save. Skipping...");
+        return PM3_EINVARG;
+    }
+
+    saveFile(fn, ".bin", d, n);
+    saveFileEML(fn, d, n, blocksize);
+    saveFileJSON(fn, jsft, d, n, NULL);
+    return PM3_SUCCESS;
+}
