@@ -71,10 +71,12 @@ int MFPGetSignature(bool activateField, bool leaveSignalON, uint8_t *dataout, in
 int MFPGetVersion(bool activateField, bool leaveSignalON, uint8_t *dataout, int maxdataoutlen, int *dataoutlen);
 
 bool mfValidateAccessConditions(const uint8_t *data);
+bool mfReadOnlyAccessConditions(uint8_t blockn, const uint8_t *data);
 const char *mfGetAccessConditionsDesc(uint8_t blockn, const uint8_t *data);
 
 uint8_t mfNumBlocksPerSector(uint8_t sectorNo);
 uint8_t mfFirstBlockOfSector(uint8_t sectorNo);
+uint8_t mfSectorTrailerOfSector(uint8_t sectorNo);
 uint8_t mfSectorTrailer(uint8_t blockNo);
 bool mfIsSectorTrailer(uint8_t blockNo);
 uint8_t mfSectorNum(uint8_t blockNo);
