@@ -22,20 +22,23 @@
 #include <stdlib.h>
 
 #include "ui.h"
-#include "util.h" // sprint_hex...
+#include "util.h"                // sprint_hex
 #include "crypto/asn1utils.h"
 #include "crypto/libpcrypto.h"
 #include "ecp.h"
-#include "commonutil.h"  // ARRAYLEN
+#include "commonutil.h"         // ARRAYLEN
 #include "pm3_cmd.h"
+#include "proxgui.h"            // Base64 Picture Window
 
 #define STRBOOL(p) ((p) ? "1" : "0")
 
-#define NDEF_WIFIAPPL   "application/vnd.wfa"
-#define NDEF_BLUEAPPL   "application/vnd.bluetooth"
-#define NDEF_JSONAPPL   "application/json"
-#define NDEF_VCARDTEXT  "text/vcard"
-#define NDEF_XVCARDTEXT "text/x-vcard"
+#define NDEF_WIFIAPPL_WSC "application/vnd.wfa.wsc"
+#define NDEF_WIFIAPPL_P2P "application/vnd.wfa.p2p"
+#define NDEF_BLUEAPPL     "application/vnd.bluetooth"
+#define NDEF_JSONAPPL     "application/json"
+#define NDEF_VCARDTEXT    "text/vcard"
+#define NDEF_XVCARDTEXT   "text/x-vcard"
+
 
 
 static const char *TypeNameFormat_s[] = {
@@ -110,12 +113,12 @@ static const char *URI_s[] = {
     "urn:nfc:"                    // 0x23
 };
 
-static int ndefRecordDecodeAndPrint(uint8_t *ndefRecord, size_t ndefRecordLen);
-static int ndefDecodePayload(NDEFHeader_t *ndef);
+static int ndefRecordDecodeAndPrint(uint8_t *ndefRecord, size_t ndefRecordLen, bool verbose);
+static int ndefDecodePayload(NDEFHeader_t *ndef, bool verbose);
 
 static uint16_t ndefTLVGetLength(const uint8_t *data, size_t *indx) {
     uint16_t len = 0;
-    if (data[0] == 0xff) {
+    if (data[0] == 0xFF) {
         len = (data[1] << 8) + data[2];
         *indx += 3;
     } else {
@@ -163,23 +166,25 @@ static int ndefDecodeHeader(uint8_t *data, size_t datalen, NDEFHeader_t *header)
     return PM3_SUCCESS;
 }
 
-static int ndefPrintHeader(NDEFHeader_t *header) {
-    PrintAndLogEx(INFO, _CYAN_("Header info"));
+static int ndefPrintHeader(NDEFHeader_t *header, bool verbose) {
 
-    PrintAndLogEx(SUCCESS, "  %s ....... Message begin", STRBOOL(header->MessageBegin));
-    PrintAndLogEx(SUCCESS, "   %s ...... Message end", STRBOOL(header->MessageEnd));
-    PrintAndLogEx(SUCCESS, "    %s ..... Chunk flag", STRBOOL(header->ChunkFlag));
-    PrintAndLogEx(SUCCESS, "     %s .... Short record bit", STRBOOL(header->ShortRecordBit));
-    PrintAndLogEx(SUCCESS, "      %s ... ID Len present", STRBOOL(header->IDLenPresent));
-    PrintAndLogEx(SUCCESS, "");
+    if (verbose) {
+        PrintAndLogEx(INFO, _CYAN_("Header info"));
+        PrintAndLogEx(SUCCESS, "  %s ....... Message begin", STRBOOL(header->MessageBegin));
+        PrintAndLogEx(SUCCESS, "   %s ...... Message end", STRBOOL(header->MessageEnd));
+        PrintAndLogEx(SUCCESS, "    %s ..... Chunk flag", STRBOOL(header->ChunkFlag));
+        PrintAndLogEx(SUCCESS, "     %s .... Short record bit", STRBOOL(header->ShortRecordBit));
+        PrintAndLogEx(SUCCESS, "      %s ... ID Len present", STRBOOL(header->IDLenPresent));
+        PrintAndLogEx(SUCCESS, "");
 
-    PrintAndLogEx(SUCCESS, " Header length...... %zu", header->len);
-    PrintAndLogEx(SUCCESS, " Type length........ %zu", header->TypeLen);
-    PrintAndLogEx(SUCCESS, " Payload length..... %zu", header->PayloadLen);
-    PrintAndLogEx(SUCCESS, " ID length.......... %zu", header->IDLen);
-    PrintAndLogEx(SUCCESS, " Record length...... %zu", header->RecLen);
+        PrintAndLogEx(SUCCESS, " Header length...... %zu", header->len);
+        PrintAndLogEx(SUCCESS, " Type length........ %zu", header->TypeLen);
+        PrintAndLogEx(SUCCESS, " Payload length..... %zu", header->PayloadLen);
+        PrintAndLogEx(SUCCESS, " ID length.......... %zu", header->IDLen);
 
-    PrintAndLogEx(SUCCESS, " Type name format... [ 0x%02x ] " _YELLOW_("%s"), header->TypeNameFormat, TypeNameFormat_s[header->TypeNameFormat]);
+        PrintAndLogEx(SUCCESS, " Type name format... [ 0x%02x ] " _YELLOW_("%s"), header->TypeNameFormat, TypeNameFormat_s[header->TypeNameFormat]);
+        PrintAndLogEx(SUCCESS, " Record length...... %zu", header->RecLen);
+    }
     return PM3_SUCCESS;
 }
 
@@ -450,15 +455,15 @@ static int ndefDecodePayloadDeviceInfo(uint8_t *payload, size_t len) {
     // record.uuid_string = '123e4567-e89b-12d3-a456-426655440000'
     //  8-4-4-4-12
     char uuid[37] = {0};
-    sprintf(uuid, "%s-", sprint_hex_inrow(p, 4));
+    snprintf(uuid, sizeof(uuid), "%s-", sprint_hex_inrow(p, 4));
     p += 4;
-    sprintf(uuid + strlen(uuid), "%s-", sprint_hex_inrow(p, 2));
+    snprintf(uuid + strlen(uuid), sizeof(uuid) - strlen(uuid), "%s-", sprint_hex_inrow(p, 2));
     p += 2;
-    sprintf(uuid + strlen(uuid), "%s-", sprint_hex_inrow(p, 2));
+    snprintf(uuid + strlen(uuid), sizeof(uuid) - strlen(uuid), "%s-", sprint_hex_inrow(p, 2));
     p += 2;
-    sprintf(uuid + strlen(uuid), "%s-", sprint_hex_inrow(p, 2));
+    snprintf(uuid + strlen(uuid), sizeof(uuid) - strlen(uuid), "%s-", sprint_hex_inrow(p, 2));
     p += 2;
-    sprintf(uuid + strlen(uuid), "%s", sprint_hex_inrow(p, 6));
+    snprintf(uuid + strlen(uuid), sizeof(uuid) - strlen(uuid), "%s", sprint_hex_inrow(p, 6));
     p += 6;
     PrintAndLogEx(INFO, "UUID.......... " _YELLOW_("%s"), uuid);
     p++;
@@ -480,26 +485,27 @@ static int ndefDecodePayloadSmartPoster(uint8_t *ndef, size_t ndeflen, bool prin
         return res;
     }
 
-    if (verbose) {
-        ndefPrintHeader(&NDEFHeader);
-    }
+    ndefPrintHeader(&NDEFHeader, verbose);
 
     if (NDEFHeader.TypeLen && NDEFHeader.PayloadLen) {
-        ndefDecodePayload(&NDEFHeader);
+        ndefDecodePayload(&NDEFHeader, verbose);
     }
 
-    if (NDEFHeader.TypeLen) {
-        PrintAndLogEx(INFO, "Type data");
-        print_buffer(NDEFHeader.Type, NDEFHeader.TypeLen, 1);
-    }
-    if (NDEFHeader.IDLen) {
-        PrintAndLogEx(INFO, "ID data");
-        print_buffer(NDEFHeader.ID, NDEFHeader.IDLen, 1);
-    }
-    if (NDEFHeader.PayloadLen) {
-        PrintAndLogEx(INFO, "Payload data");
-        print_buffer(NDEFHeader.Payload, NDEFHeader.PayloadLen, 1);
+    if (verbose) {
+        if (NDEFHeader.TypeLen) {
+            PrintAndLogEx(INFO, "Type data");
+            print_buffer(NDEFHeader.Type, NDEFHeader.TypeLen, 1);
+        }
+        if (NDEFHeader.IDLen) {
+            PrintAndLogEx(INFO, "ID data");
+            print_buffer(NDEFHeader.ID, NDEFHeader.IDLen, 1);
+        }
+        if (NDEFHeader.PayloadLen) {
+            PrintAndLogEx(INFO, "Payload data");
+            print_buffer(NDEFHeader.Payload, NDEFHeader.PayloadLen, 1);
+        }
     }
+
     // recursive
     if (NDEFHeader.MessageEnd == false) {
         ndefDecodePayloadSmartPoster(ndef + NDEFHeader.RecLen, ndeflen - NDEFHeader.RecLen, false, false);
@@ -511,75 +517,318 @@ static int ndefDecodePayloadSmartPoster(uint8_t *ndef, size_t ndeflen, bool prin
     return PM3_SUCCESS;
 }
 
-static int ndefDecodeMime_wifi(NDEFHeader_t *ndef) {
-    PrintAndLogEx(INFO, _CYAN_("WiFi details"));
-    if (ndef->PayloadLen > 1) {
-        PrintAndLogEx(INFO, ">>> decorder, to be implemented <<<");
+
+typedef struct ndef_wifi_type_s {
+    const char *description;
+    uint8_t bytes[2];
+} ndef_wifi_type_t;
+
+static const ndef_wifi_type_t wifi_crypt_types[] = {
+    {"NONE",  {0x00, 0x01}},
+    {"WEP", {0x00, 0x02}},
+    {"TKIP", {0x00, 0x04}},
+    {"AES", {0x00, 0x08}},
+    {"AES/TKIP", {0x00, 0x0C}}
+};
+
+static const char *ndef_wifi_crypt_lookup(uint8_t *d) {
+    for (int i = 0; i < ARRAYLEN(wifi_crypt_types); ++i) {
+        if (memcmp(d, wifi_crypt_types[i].bytes, 2) == 0) {
+            return wifi_crypt_types[i].description;
+        }
     }
+    return "";
+}
+
+static const ndef_wifi_type_t wifi_auth_types[] = {
+    {"OPEN", {0x00, 0x01}},
+    {"WPA PERSONAL", {0x00, 0x02}},
+    {"SHARED", {0x00, 0x04}},
+    {"WPA ENTERPRISE", {0x00, 0x08}},
+    {"WPA2 ENTERPRISE", {0x00, 0x10}},
+    {"WPA2 PERSONAL", {0x00, 0x20}},
+    {"WPA/WPA2 PERSONAL", {0x00, 0x22}}
+};
+
+static const char *ndef_wifi_auth_lookup(uint8_t *d) {
+    for (int i = 0; i < ARRAYLEN(wifi_auth_types); ++i) {
+        if (memcmp(d, wifi_auth_types[i].bytes, 2) == 0) {
+            return wifi_auth_types[i].description;
+        }
+    }
+    return "";
+}
+
+static int ndefDecodeMime_wifi_wsc(NDEFHeader_t *ndef) {
+    if (ndef->PayloadLen == 0) {
+        PrintAndLogEx(INFO, "no payload");
+        return PM3_SUCCESS;
+    }
+
+    PrintAndLogEx(INFO, _CYAN_("NDEF Wifi Simple Config Record"));
+    PrintAndLogEx(INFO, "Type............ " _YELLOW_("%.*s"), (int)ndef->TypeLen, ndef->Type);
+
+    size_t n = ndef->PayloadLen;
+    size_t pos = 0;
+    while (n) {
+
+        if (ndef->Payload[pos] != 0x10) {
+            n -= 1;
+            pos -= 1;
+            continue;
+        }
+
+        // VERSION
+        if (memcmp(&ndef->Payload[pos], "\x10\x4A", 2) == 0) {
+            uint8_t len = 1;
+            PrintAndLogEx(INFO, "Version......... %s", sprint_hex(&ndef->Payload[pos + 2], len));
+            n -= 2;
+            n -= len;
+            pos += 2;
+            pos += len;
+        }
+
+        // CREDENTIAL
+        if (memcmp(&ndef->Payload[pos], "\x10\x0E", 2) == 0) {
+            //  10 0E 00 39
+            uint8_t len = 2;
+            PrintAndLogEx(INFO, "Credential...... %s", sprint_hex(&ndef->Payload[pos + 2], len));
+            n -= 2;
+            n -= len;
+            pos += 2;
+            pos += len;
+        }
+
+        // AUTH_TYPE
+        if (memcmp(&ndef->Payload[pos], "\x10\x03", 2) == 0) {
+            // 10 03 00 02 00 20
+            uint8_t len = 4;
+            PrintAndLogEx(INFO, "Auth type....... %s ( " _YELLOW_("%s")" )",
+                          sprint_hex(&ndef->Payload[pos + 2], len),
+                          ndef_wifi_auth_lookup(&ndef->Payload[pos + 2])
+                         );
+            n -= 2;
+            n -= len;
+            pos += 2;
+            pos += len;
+        }
+
+        // CRYPT_TYPE
+        if (memcmp(&ndef->Payload[pos], "\x10\x0F", 2) == 0) {
+            // 10 0F 00 02 00 04
+            uint8_t len = 4;
+            PrintAndLogEx(INFO, "Crypt type...... %s ( " _YELLOW_("%s")" )",
+                          sprint_hex(&ndef->Payload[pos + 2], len),
+                          ndef_wifi_crypt_lookup(&ndef->Payload[pos + 2])
+                         );
+            n -= 2;
+            n -= len;
+            pos += 2;
+            pos += len;
+        }
+
+        // MAC_ADDRESS
+        if (memcmp(&ndef->Payload[pos], "\x10\x20", 2) == 0) {
+            // 10 20 00 06 FF FF FF FF FF FF
+            uint8_t len = ndef->Payload[pos + 3];
+            PrintAndLogEx(INFO, "MAC Address..... %s", sprint_hex_ascii(&ndef->Payload[pos + 4], len));
+            n -= 4;
+            n -= len;
+            pos += 4;
+            pos += len;
+        }
+
+        // NETWORK_IDX
+        if (memcmp(&ndef->Payload[pos], "\x10\x26", 2) == 0) {
+            // 10 26 00 01 01
+            uint8_t len = 3;
+            PrintAndLogEx(INFO, "Network Index... %s", sprint_hex(&ndef->Payload[pos + 2], len));
+            n -= 2;
+            n -= len;
+            pos += 2;
+            pos += len;
+        }
+
+        // NETWORK_KEY
+        if (memcmp(&ndef->Payload[pos], "\x10\x27", 2) == 0) {
+            // 10 27 00 10 74 72 69 73 74 61 6E 2D 73 70 72 69 6E 67 65 72
+            uint8_t len = ndef->Payload[pos + 3];
+            PrintAndLogEx(INFO, "Network key..... %s", sprint_hex_ascii(&ndef->Payload[pos + 4], len));
+            n -= 4;
+            n -= len;
+            pos += 4;
+            pos += len;
+        }
+        // NETWORK_NAME
+        if (memcmp(&ndef->Payload[pos], "\x10\x45", 2) == 0) {
+            // 10 45 00 06 69 63 65 73 71 6C
+            uint8_t len = ndef->Payload[pos + 3];
+            PrintAndLogEx(INFO, "Network Name.... %s", sprint_hex_ascii(&ndef->Payload[pos + 4], len));
+            n -= 4;
+            n -= len;
+            pos += 4;
+            pos += len;
+        }
+
+        // OOB_PASSWORD
+        // unknown the length.
+        if (memcmp(&ndef->Payload[pos], "\x10\x2C", 2) == 0) {
+            uint8_t len = 1;
+            PrintAndLogEx(INFO, "OOB Password......... %s", sprint_hex(&ndef->Payload[pos + 2], len));
+            n -= 2;
+            n -= len;
+            pos += 2;
+            pos += len;
+        }
+        // VENDOR_EXT
+        // unknown the length.
+        if (memcmp(&ndef->Payload[pos], "\x10\x49", 2) == 0) {
+            uint8_t len = 1;
+            PrintAndLogEx(INFO, "Vendor Ext......... %s", sprint_hex(&ndef->Payload[pos + 2], len));
+            n -= 2;
+            n -= len;
+            pos += 2;
+            pos += len;
+        }
+        // VENDOR_WFA
+        // unknown the length.
+        if (memcmp(&ndef->Payload[pos], "\x10\x30\x2A", 3) == 0) {
+            uint8_t len = 1;
+            PrintAndLogEx(INFO, "Vendor WFA......... %s", sprint_hex(&ndef->Payload[pos + 2], len));
+            n -= 2;
+            n -= len;
+            pos += 2;
+            pos += len;
+        }
+    }
+
+    /*
+        ap-channel   0,  6
+    +        credential
+        device-name
+        mac-address
+        manufacturer
+        model-name
+        model-number
+    +        oob-password
+        primary-device-type
+        rf-bands
+        secondary-device-type-list
+        serial-number
+        ssid
+        uuid-enrollee
+        uuid-registrar
+    +        vendor-extension
+        version-1
+     */
+
+    return PM3_SUCCESS;
+}
+static int ndefDecodeMime_wifi_p2p(NDEFHeader_t *ndef) {
+    if (ndef->PayloadLen == 0) {
+        PrintAndLogEx(INFO, "no payload");
+        return PM3_SUCCESS;
+    }
+
+    PrintAndLogEx(INFO, _CYAN_("NDEF Wifi Peer To Peer Record"));
+    PrintAndLogEx(INFO, "Type............ " _YELLOW_("%.*s"), (int)ndef->TypeLen, ndef->Type);
     return PM3_SUCCESS;
 }
 
 static int ndefDecodeMime_vcard(NDEFHeader_t *ndef) {
+    if (ndef->PayloadLen == 0) {
+        PrintAndLogEx(INFO, "no payload");
+        return PM3_SUCCESS;
+    }
     PrintAndLogEx(INFO, _CYAN_("VCARD details"));
-    if (ndef->PayloadLen > 1) {
-        PrintAndLogEx(INFO, "");
-        PrintAndLogEx(INFO, "%.*s", (int)ndef->PayloadLen, ndef->Payload);
+    PrintAndLogEx(INFO, "");
+    PrintAndLogEx(INFO, "%.*s", (int)ndef->PayloadLen, ndef->Payload);
+
+    char *s = strstr((char *)ndef->Payload, "PHOTO");
+    if (s) {
+        s = strtok(s, ";");
+        while (s) {
+            char *tmp = s;
+            if (strncmp(tmp, "ENCODING", 8) == 0) {
+            } else if (strncmp(tmp, "TYPE", 4) == 0) {
+
+                char *part = strtok(tmp + 4, ":");
+                while (part) {
+
+                    if (strncmp(part, "=image/", 7) == 0) {
+                    } else if (strncmp(part, "VCARD", 5) == 0) {
+                    } else  {
+                        // should be in the BASE64 data part now.
+                        ShowBase64PictureWindow(part);
+                    }
+                    part = strtok(NULL, ":");
+                }
+            }
+            s = strtok(NULL, ";");
+        }
     }
     return PM3_SUCCESS;
 }
+
 static int ndefDecodeMime_json(NDEFHeader_t *ndef) {
-    PrintAndLogEx(INFO, _CYAN_("JSON details"));
-    if (ndef->PayloadLen > 1) {
-        PrintAndLogEx(INFO, "");
-        PrintAndLogEx(INFO, _GREEN_("%.*s"), (int)ndef->PayloadLen, ndef->Payload);
+    if (ndef->PayloadLen == 0) {
+        PrintAndLogEx(INFO, "no payload");
+        return PM3_SUCCESS;
     }
+
+    PrintAndLogEx(INFO, _CYAN_("JSON details"));
+    PrintAndLogEx(INFO, "Type............ " _YELLOW_("%.*s"), (int)ndef->TypeLen, ndef->Type);
+    PrintAndLogEx(INFO, "");
+    PrintAndLogEx(INFO, _GREEN_("%.*s"), (int)ndef->PayloadLen, ndef->Payload);
     return PM3_SUCCESS;
 }
 
 static int ndefDecodeMime_bt(NDEFHeader_t *ndef) {
-    PrintAndLogEx(INFO, "Type............ " _YELLOW_("%.*s"), (int)ndef->TypeLen, ndef->Type);
-    if (ndef->PayloadLen > 1) {
-        uint16_t ooblen = (ndef->Payload[1] << 8 | ndef->Payload[0]);
-        PrintAndLogEx(INFO, "OOB data len.... %u", ooblen);
-        PrintAndLogEx(INFO, "BT MAC.......... " _YELLOW_("%s"), sprint_hex(ndef->Payload + 2, 6));
-        // Let's check payload[8]. Tells us a bit about the UUID's. If 0x07 then it tells us a service UUID is 128bit
-        switch (ndef->Payload[8]) {
-            case 0x02:
-                PrintAndLogEx(INFO, "Optional Data... incomplete list 16-bit UUID's");
-                break;
-            case 0x03:
-                PrintAndLogEx(INFO, "Optional Data... complete list 16-bit UUID's");
-                break;
-            case 0x04:
-                PrintAndLogEx(INFO, "Optional Data... incomplete list 32-bit UUID's");
-                break;
-            case 0x05:
-                PrintAndLogEx(INFO, "Optional Data... complete list 32-bit UUID's");
-                break;
-            case 0x06:
-                PrintAndLogEx(INFO, "Optional Data... incomplete list 128-bit UUID's");
-                break;
-            case 0x07:
-                PrintAndLogEx(INFO, "Optional Data... complete list 128-bit UUID's");
-                break;
-            default:
-                PrintAndLogEx(INFO, "Optional Data... [ %02x ]", ndef->Payload[8]);
-                break;
-        }
-        // Let's check payload[9]. If 0x08 then SHORT_NAME or if 0x09 then COMPLETE_NAME
-        if (ndef->Payload[9] == 0x08) {
-            PrintAndLogEx(INFO, "Short name...... " _YELLOW_("%.*s"), (int)(ndef->PayloadLen - 10), ndef->Payload + 10);
-        } else if (ndef->Payload[9] == 0x09) {
-            PrintAndLogEx(INFO, "Complete name... " _YELLOW_("%.*s"), (int)(ndef->PayloadLen - 10), ndef->Payload + 10);
-        } else {
-            PrintAndLogEx(INFO, "[ %02x ]", ndef->Payload[9]);
-        }
-        PrintAndLogEx(NORMAL, "");
+    if (ndef->PayloadLen == 0) {
+        PrintAndLogEx(INFO, "no payload");
+        return PM3_SUCCESS;
     }
+    PrintAndLogEx(INFO, "Type............ " _YELLOW_("%.*s"), (int)ndef->TypeLen, ndef->Type);
+    uint16_t ooblen = (ndef->Payload[1] << 8 | ndef->Payload[0]);
+    PrintAndLogEx(INFO, "OOB data len.... %u", ooblen);
+    PrintAndLogEx(INFO, "BT MAC.......... " _YELLOW_("%s"), sprint_hex(ndef->Payload + 2, 6));
+    // Let's check payload[8]. Tells us a bit about the UUID's. If 0x07 then it tells us a service UUID is 128bit
+    switch (ndef->Payload[8]) {
+        case 0x02:
+            PrintAndLogEx(INFO, "Optional Data... incomplete list 16-bit UUID's");
+            break;
+        case 0x03:
+            PrintAndLogEx(INFO, "Optional Data... complete list 16-bit UUID's");
+            break;
+        case 0x04:
+            PrintAndLogEx(INFO, "Optional Data... incomplete list 32-bit UUID's");
+            break;
+        case 0x05:
+            PrintAndLogEx(INFO, "Optional Data... complete list 32-bit UUID's");
+            break;
+        case 0x06:
+            PrintAndLogEx(INFO, "Optional Data... incomplete list 128-bit UUID's");
+            break;
+        case 0x07:
+            PrintAndLogEx(INFO, "Optional Data... complete list 128-bit UUID's");
+            break;
+        default:
+            PrintAndLogEx(INFO, "Optional Data... [ %02x ]", ndef->Payload[8]);
+            break;
+    }
+    // Let's check payload[9]. If 0x08 then SHORT_NAME or if 0x09 then COMPLETE_NAME
+    if (ndef->Payload[9] == 0x08) {
+        PrintAndLogEx(INFO, "Short name...... " _YELLOW_("%.*s"), (int)(ndef->PayloadLen - 10), ndef->Payload + 10);
+    } else if (ndef->Payload[9] == 0x09) {
+        PrintAndLogEx(INFO, "Complete name... " _YELLOW_("%.*s"), (int)(ndef->PayloadLen - 10), ndef->Payload + 10);
+    } else {
+        PrintAndLogEx(INFO, "[ %02x ]", ndef->Payload[9]);
+    }
+    PrintAndLogEx(NORMAL, "");
     return PM3_SUCCESS;
 }
 
-static int ndefDecodePayload(NDEFHeader_t *ndef) {
+static int ndefDecodePayload(NDEFHeader_t *ndef, bool verbose) {
 
     PrintAndLogEx(INFO, "");
     switch (ndef->TypeNameFormat) {
@@ -622,7 +871,7 @@ static int ndefDecodePayload(NDEFHeader_t *ndef) {
             }
 
             if (!strncmp((char *)ndef->Type, "Sp", ndef->TypeLen)) {
-                ndefDecodePayloadSmartPoster(ndef->Payload, ndef->PayloadLen, true, false);
+                ndefDecodePayloadSmartPoster(ndef->Payload, ndef->PayloadLen, true, verbose);
             }
 
             if (!strncmp((char *)ndef->Type, "Di", ndef->TypeLen)) {
@@ -660,8 +909,11 @@ static int ndefDecodePayload(NDEFHeader_t *ndef) {
             memcpy(begin, ndef->Type, ndef->TypeLen);
             str_lower(begin);
 
-            if (str_startswith(begin, NDEF_WIFIAPPL)) {
-                ndefDecodeMime_wifi(ndef);
+            if (str_startswith(begin, NDEF_WIFIAPPL_WSC)) {
+                ndefDecodeMime_wifi_wsc(ndef);
+            }
+            if (str_startswith(begin, NDEF_WIFIAPPL_P2P)) {
+                ndefDecodeMime_wifi_p2p(ndef);
             }
             if (str_startswith(begin, NDEF_VCARDTEXT) || str_startswith(begin, NDEF_XVCARDTEXT)) {
                 ndefDecodeMime_vcard(ndef);
@@ -698,36 +950,39 @@ static int ndefDecodePayload(NDEFHeader_t *ndef) {
     return PM3_SUCCESS;
 }
 
-static int ndefRecordDecodeAndPrint(uint8_t *ndefRecord, size_t ndefRecordLen) {
+static int ndefRecordDecodeAndPrint(uint8_t *ndefRecord, size_t ndefRecordLen, bool verbose) {
     NDEFHeader_t NDEFHeader = {0};
     int res = ndefDecodeHeader(ndefRecord, ndefRecordLen, &NDEFHeader);
     if (res != PM3_SUCCESS)
         return res;
 
-    ndefPrintHeader(&NDEFHeader);
+    ndefPrintHeader(&NDEFHeader, verbose);
     PrintAndLogEx(INFO, "");
     PrintAndLogEx(INFO, _CYAN_("Payload info"));
 
-    if (NDEFHeader.TypeLen) {
-        PrintAndLogEx(INFO, "Type data");
-        print_buffer(NDEFHeader.Type, NDEFHeader.TypeLen, 1);
-    }
-    if (NDEFHeader.IDLen) {
-        PrintAndLogEx(INFO, "ID data");
-        print_buffer(NDEFHeader.ID, NDEFHeader.IDLen, 1);
-    }
-    if (NDEFHeader.PayloadLen) {
-        PrintAndLogEx(INFO, "Payload data");
-        print_buffer(NDEFHeader.Payload, NDEFHeader.PayloadLen, 1);
+    if (verbose) {
+        if (NDEFHeader.TypeLen) {
+            PrintAndLogEx(INFO, "Type data");
+            print_buffer(NDEFHeader.Type, NDEFHeader.TypeLen, 1);
+        }
+        if (NDEFHeader.IDLen) {
+            PrintAndLogEx(INFO, "ID data");
+            print_buffer(NDEFHeader.ID, NDEFHeader.IDLen, 1);
+        }
+        if (NDEFHeader.PayloadLen) {
+            PrintAndLogEx(INFO, "Payload data");
+            print_buffer(NDEFHeader.Payload, NDEFHeader.PayloadLen, 1);
+        }
     }
+
     if (NDEFHeader.TypeLen && NDEFHeader.PayloadLen) {
-        ndefDecodePayload(&NDEFHeader);
+        ndefDecodePayload(&NDEFHeader, verbose);
     }
 
     return PM3_SUCCESS;
 }
 
-int NDEFRecordsDecodeAndPrint(uint8_t *ndefRecord, size_t ndefRecordLen) {
+int NDEFRecordsDecodeAndPrint(uint8_t *ndefRecord, size_t ndefRecordLen, bool verbose) {
     bool firstRec = true;
     size_t len = 0;
     size_t counter = 0;
@@ -756,7 +1011,7 @@ int NDEFRecordsDecodeAndPrint(uint8_t *ndefRecord, size_t ndefRecordLen) {
         PrintAndLogEx(NORMAL, "");
         PrintAndLogEx(SUCCESS, _CYAN_("Record") " " _YELLOW_("%zu"), counter);
         PrintAndLogEx(INFO, "-----------------------------------------------------");
-        ndefRecordDecodeAndPrint(&ndefRecord[len], NDEFHeader.RecLen);
+        ndefRecordDecodeAndPrint(&ndefRecord[len], NDEFHeader.RecLen, verbose);
 
         len += NDEFHeader.RecLen;
 
@@ -788,21 +1043,22 @@ int NDEFDecodeAndPrint(uint8_t *ndef, size_t ndefLen, bool verbose) {
             case 0x01: {
                 indx++;
                 uint16_t len = ndefTLVGetLength(&ndef[indx], &indx);
+                PrintAndLogEx(NORMAL, "");
                 PrintAndLogEx(SUCCESS, "--- " _CYAN_("NDEF Lock Control") " ---");
                 if (len != 3) {
-                    PrintAndLogEx(WARNING, "NDEF Lock Control block size must be 3 instead of %d.", len);
+                    PrintAndLogEx(WARNING, "NDEF Lock Control block size must be 3 instead of %d", len);
                 } else {
                     uint8_t pages_addr = (ndef[indx] >> 4) & 0x0f;
                     uint8_t byte_offset = ndef[indx] & 0x0f;
                     uint8_t Size = ndef[indx + 1];
                     uint8_t BytesLockedPerLockBit = (ndef[indx + 2] >> 4) & 0x0f;
                     uint8_t bytes_per_page = ndef[indx + 2] & 0x0f;
-                    PrintAndLogEx(SUCCESS, " Pages addr (number of pages)... %d", pages_addr);
-                    PrintAndLogEx(SUCCESS, "Byte offset (number of bytes)... %d", byte_offset);
-                    PrintAndLogEx(SUCCESS, "Size in bits of the lock area %d. bytes approx %d", Size, Size / 8);
-                    PrintAndLogEx(SUCCESS, "       Number of bytes / page... %d", bytes_per_page);
-                    PrintAndLogEx(SUCCESS, "Bytes Locked Per LockBit");
-                    PrintAndLogEx(SUCCESS, "   number of bytes that each dynamic lock bit is able to lock: %d", BytesLockedPerLockBit);
+                    PrintAndLogEx(SUCCESS, " Pages addr (number of pages).... %d", pages_addr);
+                    PrintAndLogEx(SUCCESS, " Byte offset (number of bytes)... %d", byte_offset);
+                    PrintAndLogEx(SUCCESS, " Lock Area size in bits.......... %d ( %d bytes )", Size, Size / 8);
+                    PrintAndLogEx(SUCCESS, "        Number of bytes / page... %d", bytes_per_page);
+                    PrintAndLogEx(SUCCESS, " Bytes Locked Per LockBit");
+                    PrintAndLogEx(SUCCESS, "  number of bytes that each dynamic lock bit locks... %d", BytesLockedPerLockBit);
                 }
                 indx += len;
                 break;
@@ -810,18 +1066,19 @@ int NDEFDecodeAndPrint(uint8_t *ndef, size_t ndefLen, bool verbose) {
             case 0x02: {
                 indx++;
                 uint16_t len = ndefTLVGetLength(&ndef[indx], &indx);
+                PrintAndLogEx(NORMAL, "");
                 PrintAndLogEx(SUCCESS, "--- " _CYAN_("NDEF Memory Control") " ---");
                 if (len != 3) {
-                    PrintAndLogEx(WARNING, "NDEF Memory Control block size must be 3 instead of %d.", len);
+                    PrintAndLogEx(WARNING, "NDEF Memory Control block size must be 3 instead of %d", len);
                 } else {
                     uint8_t pages_addr = (ndef[indx] >> 4) & 0x0f;
                     uint8_t byte_offset = ndef[indx] & 0x0f;
                     uint8_t Size = ndef[indx + 1];
                     uint8_t bytes_per_page = ndef[indx + 2] & 0x0f;
-                    PrintAndLogEx(SUCCESS, " Pages addr (number of pages) : %d", pages_addr);
-                    PrintAndLogEx(SUCCESS, "Byte offset (number of bytes) : %d", byte_offset);
-                    PrintAndLogEx(SUCCESS, "Size in bits of the reserved area : %d. bytes approx: %d", Size, Size / 8);
-                    PrintAndLogEx(SUCCESS, "       Number of bytes / page : %d", bytes_per_page);
+                    PrintAndLogEx(SUCCESS, "Pages addr (number of pages).... %d", pages_addr);
+                    PrintAndLogEx(SUCCESS, "Byte offset (number of bytes)... %d", byte_offset);
+                    PrintAndLogEx(SUCCESS, "Reserved area size in bits...... %d ( %d bytes )", Size, Size / 8);
+                    PrintAndLogEx(SUCCESS, "       Number of bytes / page... %d", bytes_per_page);
                 }
                 indx += len;
                 break;
@@ -829,13 +1086,14 @@ int NDEFDecodeAndPrint(uint8_t *ndef, size_t ndefLen, bool verbose) {
             case 0x03: {
                 indx++;
                 uint16_t len = ndefTLVGetLength(&ndef[indx], &indx);
+                PrintAndLogEx(NORMAL, "");
                 PrintAndLogEx(SUCCESS, "--- " _CYAN_("NDEF Message") " ---");
                 if (len == 0) {
                     PrintAndLogEx(SUCCESS, "Found NDEF message w zero length");
                 } else {
-                    PrintAndLogEx(SUCCESS, "Found NDEF message (%d bytes)", len);
+                    PrintAndLogEx(SUCCESS, "Found NDEF message ( " _YELLOW_("%d") " bytes )", len);
 
-                    int res = NDEFRecordsDecodeAndPrint(&ndef[indx], len);
+                    int res = NDEFRecordsDecodeAndPrint(&ndef[indx], len, verbose);
                     if (res != PM3_SUCCESS)
                         return res;
                 }
@@ -843,15 +1101,16 @@ int NDEFDecodeAndPrint(uint8_t *ndef, size_t ndefLen, bool verbose) {
                 indx += len;
                 break;
             }
-            case 0xfd: {
+            case 0xFD: {
                 indx++;
                 uint16_t len = ndefTLVGetLength(&ndef[indx], &indx);
+                PrintAndLogEx(NORMAL, "");
                 PrintAndLogEx(SUCCESS, "--- " _CYAN_("Proprietary info") " ---");
                 PrintAndLogEx(SUCCESS, "  Can't decode, skipping %d bytes", len);
                 indx += len;
                 break;
             }
-            case 0xfe: {
+            case 0xFE: {
                 PrintAndLogEx(SUCCESS, "NDEF Terminator detected");
                 return PM3_SUCCESS;
             }
