@@ -1,37 +1,20 @@
-/*
- * Generic uart / rs232/ serial port library
- *
- * Copyright (c) 2013, Roel Verdult
- * Copyright (c) 2018 Google
- * All rights reserved.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions are met:
- * 1. Redistributions of source code must retain the above copyright
- * notice, this list of conditions and the following disclaimer.
- * 2. Redistributions in binary form must reproduce the above copyright
- * notice, this list of conditions and the following disclaimer in the
- * documentation and/or other materials provided with the distribution.
- * 3. Neither the name of the copyright holders nor the
- * names of its contributors may be used to endorse or promote products
- * derived from this software without specific prior written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS ''AS IS'' AND ANY
- * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
- * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
- * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY
- * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
- * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
- * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
- * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
- * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
- * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
- *
- * @file uart_posix.c
- *
- * This version of the library has functionality removed which was not used by
- * proxmark3 project.
- */
+//-----------------------------------------------------------------------------
+// Copyright (C) Proxmark3 contributors. See AUTHORS.md for details.
+//
+// This program is free software: you can redistribute it and/or modify
+// it under the terms of the GNU General Public License as published by
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
+//
+// This program is distributed in the hope that it will be useful,
+// but WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU General Public License for more details.
+//
+// See LICENSE.txt for the text of the license.
+//-----------------------------------------------------------------------------
+// Generic uart / rs232/ serial port library
+//-----------------------------------------------------------------------------
 
 // Test if we are dealing with posix operating systems
 #ifndef _WIN32
@@ -99,7 +82,7 @@ serial_port uart_open(const char *pcPortName, uint32_t speed) {
 
     char *prefix = strdup(pcPortName);
     if (prefix == NULL) {
-        PrintAndLogEx(ERR, "error: malloc");
+        PrintAndLogEx(ERR, "error:  string duplication");
         free(sp);
         return INVALID_SERIAL_PORT;
     }
@@ -117,7 +100,7 @@ serial_port uart_open(const char *pcPortName, uint32_t speed) {
 
         char *addrstr = strdup(pcPortName + 4);
         if (addrstr == NULL) {
-            PrintAndLogEx(ERR, "error: malloc");
+            PrintAndLogEx(ERR, "error: string duplication");
             free(sp);
             return INVALID_SERIAL_PORT;
         }
@@ -161,17 +144,15 @@ serial_port uart_open(const char *pcPortName, uint32_t speed) {
             close(sfd);
         }
 
+        freeaddrinfo(addr);
+        free(addrstr);
+
         if (rp == NULL) {               /* No address succeeded */
             PrintAndLogEx(ERR, "error: Could not connect");
-            freeaddrinfo(addr);
-            free(addrstr);
             free(sp);
             return INVALID_SERIAL_PORT;
         }
 
-        freeaddrinfo(addr);
-        free(addrstr);
-
         sp->fd = sfd;
 
         int one = 1;
@@ -194,7 +175,7 @@ serial_port uart_open(const char *pcPortName, uint32_t speed) {
 
         char *addrstr = strndup(pcPortName + 3, 17);
         if (addrstr == NULL) {
-            PrintAndLogEx(ERR, "error: malloc");
+            PrintAndLogEx(ERR, "error: string duplication");
             free(sp);
             return INVALID_SERIAL_PORT;
         }
