@@ -49,6 +49,29 @@ void FormatVersionInformation(char *dst, int len, const char *prefix, void *vers
 
     strncat(dst, " ", len - strlen(dst) - 1);
     strncat(dst, v->buildtime, len - strlen(dst) - 1);
+    strncat(dst, " ", len - strlen(dst) - 1);
+    strncat(dst, v->armsrc, len - strlen(dst) - 1);
+}
+
+void format_version_information_short(char *dst, int len, void *version_info) {
+    struct version_information_t *v = (struct version_information_t *)version_info;
+    dst[0] = 0;
+    if (v->magic != VERSION_INFORMATION_MAGIC) {
+        strncat(dst, "Missing/Invalid version information", len - strlen(dst) - 1);
+        return;
+    }
+    if (v->versionversion != 1) {
+        strncat(dst, "Version information not understood", len - strlen(dst) - 1);
+        return;
+    }
+    if (!v->present) {
+        strncat(dst, "Version information not available", len - strlen(dst) - 1);
+        return;
+    }
+
+    strncat(dst, v->gitversion, len - strlen(dst) - 1);
+    strncat(dst, " ", len - strlen(dst) - 1);
+    strncat(dst, v->buildtime, len - strlen(dst) - 1);
 }
 
 /*
@@ -239,10 +262,10 @@ uint32_t rotr(uint32_t a, uint8_t n) {
     return (a >> n) | (a << (32 - n));
 }
 
-uint16_t get_sw(const uint8_t *d, uint8_t n) {
+uint16_t get_sw(const uint8_t *d, uint16_t n) {
     if (n < 2)
         return 0;
 
     n -= 2;
-    return d[n] * 0x0100 + d[n + 1];
+    return (d[n] << 8 | d[n + 1]);
 }
