@@ -1,22 +1,19 @@
-/*  crapto1.c
-
-    This program is free software; you can redistribute it and/or
-    modify it under the terms of the GNU General Public License
-    as published by the Free Software Foundation; either version 2
-    of the License, or (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin Street, Fifth Floor,
-    Boston, MA  02110-1301, US$
-
-    Copyright (C) 2008-2014 bla <blapost@gmail.com>
-*/
+//-----------------------------------------------------------------------------
+// Copyright (C) 2008-2014 bla <blapost@gmail.com>
+// Copyright (C) Proxmark3 contributors. See AUTHORS.md for details.
+//
+// This program is free software: you can redistribute it and/or modify
+// it under the terms of the GNU General Public License as published by
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
+//
+// This program is distributed in the hope that it will be useful,
+// but WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU General Public License for more details.
+//
+// See LICENSE.txt for the text of the license.
+//-----------------------------------------------------------------------------
 #include "crapto1.h"
 
 #include "bucketsort.h"
@@ -144,9 +141,9 @@ struct Crypto1State *lfsr_recovery32(uint32_t ks2, uint32_t in) {
     for (i = 30; i >= 0; i -= 2)
         eks = eks << 1 | BEBIT(ks2, i);
 
-    odd_head = odd_tail = malloc(sizeof(uint32_t) << 21);
-    even_head = even_tail = malloc(sizeof(uint32_t) << 21);
-    statelist =  malloc(sizeof(struct Crypto1State) << 18);
+    odd_head = odd_tail = calloc(1, sizeof(uint32_t) << 21);
+    even_head = even_tail = calloc(1, sizeof(uint32_t) << 21);
+    statelist =  calloc(1, sizeof(struct Crypto1State) << 18);
     if (!odd_tail-- || !even_tail-- || !statelist) {
         free(statelist);
         statelist = 0;
@@ -160,7 +157,7 @@ struct Crypto1State *lfsr_recovery32(uint32_t ks2, uint32_t in) {
 
     for (i = 0; i < 2; i++) {
         for (uint32_t j = 0; j <= 0xff; j++) {
-            bucket[i][j].head = malloc(sizeof(uint32_t) << 14);
+            bucket[i][j].head = calloc(1, sizeof(uint32_t) << 14);
             if (!bucket[i][j].head) {
                 goto out;
             }
@@ -230,7 +227,7 @@ struct Crypto1State *lfsr_recovery64(uint32_t ks2, uint32_t ks3) {
     uint32_t *tail, table[1 << 16];
     int i, j;
 
-    sl = statelist = malloc(sizeof(struct Crypto1State) << 4);
+    sl = statelist = calloc(1, sizeof(struct Crypto1State) << 4);
     if (!sl)
         return 0;
     sl->odd = sl->even = 0;
@@ -418,7 +415,7 @@ static uint32_t fastfwd[2][8] = {
  * encrypt the NACK which is observed when varying only the 3 last bits of Nr
  * only correct iff [NR_3] ^ NR_3 does not depend on Nr_3
  */
-uint32_t *lfsr_prefix_ks(uint8_t ks[8], int isodd) {
+uint32_t *lfsr_prefix_ks(const uint8_t ks[8], int isodd) {
     uint32_t *candidates = calloc(4 << 10, sizeof(uint8_t));
     if (!candidates) return 0;
 
@@ -475,7 +472,7 @@ static struct Crypto1State *check_pfx_parity(uint32_t prefix, uint32_t rresp, ui
 
 #if !defined(__arm__) || defined(__linux__) || defined(_WIN32) || defined(__APPLE__) // bare metal ARM Proxmark lacks malloc()/free()
 /** lfsr_common_prefix
- * Implentation of the common prefix attack.
+ * Implementation of the common prefix attack.
  * Requires the 28 bit constant prefix used as reader nonce (pfx)
  * The reader response used (rr)
  * The keystream used to encrypt the observed NACK's (ks)
@@ -491,7 +488,7 @@ struct Crypto1State *lfsr_common_prefix(uint32_t pfx, uint32_t rr, uint8_t ks[8]
     odd = lfsr_prefix_ks(ks, 1);
     even = lfsr_prefix_ks(ks, 0);
 
-    s = statelist = malloc((sizeof * statelist) << 24); // was << 20. Need more for no_par special attack. Enough???
+    s = statelist = calloc(1, (sizeof * statelist) << 24); // was << 20. Need more for no_par special attack. Enough???
     if (!s || !odd || !even) {
         free(statelist);
         statelist = 0;
