@@ -205,6 +205,7 @@ typedef struct {
     // rdv4
     bool hw_available_flash            : 1;
     bool hw_available_smartcard        : 1;
+    bool is_rdv4                       : 1;
 } PACKED capabilities_t;
 #define CAPABILITIES_VERSION 6
 extern capabilities_t g_pm3_capabilities;
@@ -378,6 +379,7 @@ typedef struct {
 #define CMD_TIA                                                           0x0117
 #define CMD_BREAK_LOOP                                                    0x0118
 #define CMD_SET_TEAROFF                                                   0x0119
+#define CMD_GET_DBGMODE                                                   0x0120
 
 // RDV40, Flash memory operations
 #define CMD_FLASHMEM_WRITE                                                0x0121
@@ -415,16 +417,15 @@ typedef struct {
 #define CMD_SPIFFS_WIPE                                                   0x013A
 
 // This take a +0x2000 as they are high level helper and special functions
-// As the others, they may have safety level argument if it makkes sense
+// As the others, they may have safety level argument if it makes sense
 #define CMD_SPIFFS_PRINT_TREE                                             0x2130
 #define CMD_SPIFFS_GET_TREE                                               0x2131
 #define CMD_SPIFFS_TEST                                                   0x2132
 #define CMD_SPIFFS_PRINT_FSINFO                                           0x2133
 #define CMD_SPIFFS_DOWNLOAD                                               0x2134
 #define CMD_SPIFFS_DOWNLOADED                                             0x2135
+#define CMD_SPIFFS_ELOAD                                                  0x2136
 #define CMD_SPIFFS_CHECK                                                  0x3000
-// more ?
-
 
 // RDV40,  Smart card operations
 #define CMD_SMART_RAW                                                     0x0140
@@ -455,7 +456,7 @@ typedef struct {
 #define CMD_LF_SIMULATE_BIDIR                                             0x020E
 #define CMD_SET_ADC_MUX                                                   0x020F
 #define CMD_LF_HID_CLONE                                                  0x0210
-#define CMD_LF_EM410X_WRITE                                               0x0211
+#define CMD_LF_EM410X_CLONE                                               0x0211
 #define CMD_LF_T55XX_READBL                                               0x0214
 #define CMD_LF_T55XX_WRITEBL                                              0x0215
 #define CMD_LF_T55XX_RESET_READ                                           0x0216
@@ -510,6 +511,7 @@ typedef struct {
 
 // For the 13.56 MHz tags
 #define CMD_HF_ISO15693_ACQ_RAW_ADC                                       0x0300
+#define CMD_HF_ACQ_RAW_ADC                                                0x0301
 #define CMD_HF_SRI_READ                                                   0x0303
 #define CMD_HF_ISO14443B_COMMAND                                          0x0305
 #define CMD_HF_ISO15693_READER                                            0x0310
@@ -518,7 +520,17 @@ typedef struct {
 #define CMD_HF_ISO15693_COMMAND                                           0x0313
 #define CMD_HF_ISO15693_FINDAFI                                           0x0315
 #define CMD_HF_ISO15693_CSETUID                                           0x0316
-#define CMD_HF_ISO15693_SLIX_L_DISABLE_PRIVACY                            0x0317
+#define CMD_HF_ISO15693_SLIX_ENABLE_PRIVACY                               0x0867
+#define CMD_HF_ISO15693_SLIX_DISABLE_PRIVACY                              0x0317
+#define CMD_HF_ISO15693_SLIX_DISABLE_EAS                                  0x0318
+#define CMD_HF_ISO15693_SLIX_ENABLE_EAS                                   0x0862
+#define CMD_HF_ISO15693_SLIX_PASS_PROTECT_AFI                             0x0863
+#define CMD_HF_ISO15693_SLIX_PASS_PROTECT_EAS                             0x0864
+#define CMD_HF_ISO15693_SLIX_WRITE_PWD                                    0x0865
+#define CMD_HF_ISO15693_WRITE_AFI                                         0x0866
+#define CMD_HF_TEXKOM_SIMULATE                                            0x0320
+#define CMD_HF_ISO15693_EML_CLEAR                                         0x0330
+#define CMD_HF_ISO15693_EML_SETMEM                                        0x0331
 
 #define CMD_LF_SNIFF_RAW_ADC                                              0x0360
 
@@ -550,6 +562,7 @@ typedef struct {
 
 #define CMD_HF_EPA_COLLECT_NONCE                                          0x038A
 #define CMD_HF_EPA_REPLAY                                                 0x038B
+#define CMD_HF_EPA_PACE_SIMULATE                                          0x039C
 
 #define CMD_HF_LEGIC_INFO                                                 0x03BC
 #define CMD_HF_LEGIC_ESET                                                 0x03BD
@@ -613,6 +626,7 @@ typedef struct {
 #define CMD_HF_MIFARE_READSC                                              0x0621
 #define CMD_HF_MIFAREU_READCARD                                           0x0721
 #define CMD_HF_MIFARE_WRITEBL                                             0x0622
+#define CMD_HF_MIFARE_VALUE                                               0x0627
 #define CMD_HF_MIFAREU_WRITEBL                                            0x0722
 #define CMD_HF_MIFAREU_WRITEBL_COMPAT                                     0x0723
 
@@ -669,6 +683,7 @@ typedef struct {
 
 // Gen 4 GTU magic cards
 #define CMD_HF_MIFARE_G4_RDBL                                             0x0860
+#define CMD_HF_MIFARE_G4_WRBL                                             0x0861
 
 #define CMD_UNKNOWN                                                       0xFFFF
 
@@ -706,6 +721,8 @@ typedef struct {
 
 // Error codes                          Usages:
 
+// Success, regular quit
+#define PM3_SQUIT               2
 // Success, transfer nonces            pm3:        Sending nonces back to client
 #define PM3_SNONCES             1
 // Success (no error)
@@ -763,11 +780,15 @@ typedef struct {
 // Got bad CRC                          client/pm3: error in transfer of data,  crc mismatch.
 #define PM3_ECRC              -24
 
+// STATIC Nonce detect                  pm3:  when collecting nonces for hardnested
+#define PM3_ESTATIC_NONCE     -25
+
 // No data                              pm3:        no data available, no host frame available (not really an error)
 #define PM3_ENODATA           -98
 // Quit program                         client:     reserved, order to quit the program
 #define PM3_EFATAL            -99
 
+
 // LF
 #define LF_FREQ2DIV(f) ((int)(((12000.0 + (f)/2.0)/(f))-1))
 #define LF_DIVISOR_125 LF_FREQ2DIV(125)
